Package: HPiP
Type: Package
Title: Host-Pathogen Interaction Prediction
Version: 1.19.0
Authors@R: c(
    person("Matineh", "Rahmatbakhsh", ,"matinerb.94@gmail.com",c("aut","trl","cre")),
    person("Mohan", "Babu", ,"mohan.babu@uregina.ca", c("led")))
Description: HPiP (Host-Pathogen Interaction Prediction) uses an ensemble learning 
    algorithm for prediction of host-pathogen protein-protein interactions (HP-PPIs) 
    using structural and physicochemical descriptors computed from amino 
    acid-composition of host and pathogen proteins.The proposed package can effectively address 
    data shortages and data unavailability for HP-PPI network reconstructions. 
    Moreover, establishing computational frameworks in that regard will reveal 
    mechanistic insights into infectious diseases and suggest potential HP-PPI targets, 
    thus narrowing down the range of possible candidates for subsequent wet-lab experimental validations.
Depends: R (>= 4.1)
Imports: dplyr (>= 1.0.6), httr (>= 1.4.2), readr, tidyr, tibble,
        utils, stringr, magrittr, caret, corrplot, ggplot2, pROC,
        PRROC, igraph, graphics, stats, purrr, grDevices, protr, MCL
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/mrbakhsh/HPiP
BugReports: https://github.com/mrbakhsh/HPiP/issues
VignetteBuilder: knitr
Suggests: rmarkdown, colorspace, e1071, kernlab, ranger,
        SummarizedExperiment, Biostrings, randomForest, gprofiler2,
        gridExtra, ggthemes, BiocStyle, BiocGenerics, RUnit, tools,
        knitr
biocViews: Proteomics, SystemsBiology, NetworkInference,
        StructuralPrediction, GenePrediction, Network
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/HPiP
git_branch: devel
git_last_commit: a85ff66
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:19:40 UTC; biocbuild
Author: Matineh Rahmatbakhsh [aut, trl, cre],
  Mohan Babu [led]
Maintainer: Matineh Rahmatbakhsh <matinerb.94@gmail.com>

# Generated by roxygen2: do not edit by hand

export(FSmethod)
export(FreqInteractors)
export(calculateAAC)
export(calculateAutocor)
export(calculateBE)
export(calculateCTDC)
export(calculateCTDD)
export(calculateCTDT)
export(calculateCTriad)
export(calculateDC)
export(calculateF)
export(calculateKSAAP)
export(calculateQD_Sm)
export(calculateTC)
export(calculateTC_Sm)
export(corr_plot)
export(enrichfindP)
export(enrichfind_cpx)
export(enrichfind_hp)
export(enrichplot)
export(filter_missing_values)
export(getFASTA)
export(getHPI)
export(get_negativePPI)
export(get_positivePPI)
export(impute_missing_data)
export(plotPPI)
export(pred_ensembel)
export(run_clustering)
export(var_imp)
importFrom(MCL,mcl)
importFrom(PRROC,pr.curve)
importFrom(caret,predictors)
importFrom(caret,rfFuncs)
importFrom(caret,rfeControl)
importFrom(caret,train)
importFrom(caret,trainControl)
importFrom(caret,twoClassSummary)
importFrom(caret,varImp)
importFrom(corrplot,corrplot)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_gradient2)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(igraph,E)
importFrom(igraph,cluster_label_prop)
importFrom(igraph,cluster_louvain)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,fastgreedy.community)
importFrom(igraph,get.adjacency)
importFrom(igraph,graph.adjacency)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,membership)
importFrom(igraph,simplify)
importFrom(magrittr,"%>%")
importFrom(magrittr,set_rownames)
importFrom(pROC,ggroc)
importFrom(pROC,plot.roc)
importFrom(pROC,roc)
importFrom(protr,readFASTA)
importFrom(purrr,map_df)
importFrom(readr,col_character)
importFrom(readr,col_double)
importFrom(readr,cols)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_split)
importFrom(tibble,add_column)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(tidyr,spread)
importFrom(utils,read.csv)
importFrom(utils,read.delim)

<!-- badges: start -->
[![](https://img.shields.io/badge/lifecycle-stable-green.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![](https://img.shields.io/github/last-commit/mrbakhsh/HPiP.svg)](https://github.com/mrbakhsh/HPiP/commits/main)
[![License: MIT (&gt;=
3)](https://img.shields.io/badge/license-MIT-blue.svg)](https://cran.r-project.org/web/licenses/MIT)
[![DOI:10.1093/bioadv/vbac038](http://img.shields.io/badge/DOI-10./bioadv/vbac038-5680C1.svg)](https://doi.org/10.1093/bioadv/vbac038)
<!-- badges: end -->


# Host-Pathogen Interaction Prediction (HPiP)

HPiP (host-pathogen interaction prediction) is an R package for automated prediction of host-pathogen protein-protein interactions (HP-PPIs) using structural and physicochemical descriptors computed from amino acid-composition of host and pathogen proteins. Briefly, HPiP extracts gold-standard of experimentally verified HP-PPIs (i.e., positive interactions) from public repository, construct negative interactions via negative sampling, retrieve and convert protein sequences to numerical representation via various descriptors, applies multivariate feature selection based on correlation and recursive feature elimination (RFE)-embedded, and finally applies ensemble averaging to predict interactions. Taken together, we hope that the HPiP package not only contributes a useful predictor to accelerate the exploration of host-pathogen PPIs, but also provides some meaningful insights into host-pathogen relationships.
## Installation

You can install the `HPiP` from bioconductor using:

```r
if(!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager") 
}
BiocManager::install("HPiP")
```

To view documentation for the version of this package installed in your system, start R and enter:

```r
browseVignettes("HPiP")
```

To install the development version in `R`, run:
  
```r
if(!requireNamespace("devtools", quietly = TRUE)) {
  install.packages("devtools") 
}
devtools::install_github("mrbakhsh/HPiP")
```

## Adapting the pipeline for different datasets

The utility and performance of the proposed package were demonstrated using three different case studies, and data analysis codes are available from https://github.com/BabuLab-UofR/HPiP_pub, where guidelines and sample datasets are also offered for testing purposes.

## Contribute

Check the github page for [source code](https://github.com/BabuLab-UofR/HPiP)

## License
This project is licensed under the MIT License - see the LICENSE.md file for more details.

If using these scripts in your data analyses pipelines, please cite our paper:

Rahmatbakhsh,M. et al. (2022) HPiP: an R/Bioconductor package for predicting host–pathogen protein–protein interactions from protein sequences using ensemble machine learning approach. Bioinforma. Adv., 2, vbac038.

# README
- `AAidx.csv` includes amino acid properties that are retrieved
from [AAindex Database](https://www.genome.jp/dbget/aaindex.html).
- `UP000464024.fasta` includes fasta sequences for SARS-CoV-2 viral proteins,
extracted directly from [UniProt database](https://www.uniprot.org/).
- `TP_set.csv` includes interaction between SARS-CoV-2 and host proteins, 
retrieved from [BioGrid database](https://thebiogrid.org/), and for the sake of 
example, we randomly selected 100 interactions from all pairs.

    #' get_positivePPI
    #' @title Fetch Positive Reference Host-Pathogen Protein-Protein
    #' Interactions (HP-PPIs) from the BioGRID Database
    #' @param organism.taxID Taxonomy identifier for the pathogen.
    #' @param access.key Access key for using BioGRID webpage. To retrieve
    #' interactions from the BioGRID database, the users are first required to
    #' register for access key at
    #' \url{https://webservice.thebiogrid.org/}.
    #' @param filename A character string, indicating the output filename as an
    #' RData object to store the retrieved interactions.
    #' @param path A character string indicating the path to the project
    #' directory that contains the interaction data. If the directory is
    #' missing, it will be stored in the current directory.
    #' Default is PositiveInt.
    #' @return A Data.frame containing true positive protein-protein
    #' interactions for the selected pathogen.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @seealso See \code{\link{get_negativePPI}} for generating negative
    #' protein-protein interaction.
    #' @importFrom readr cols
    #' @importFrom readr col_double
    #' @importFrom readr col_character
    #' @description This function retrieves positive reference host-pathogen
    #' protein-protein interactions directly from BioGRID database.
    #' @export
    #' @examples
    #' \donttest{
    #' local = tempdir()
    #' try(get_positivePPI(organism.taxID = 2697049,
    #' access.key = 'XXXX',
    #' filename = "PositiveInt.RData",
    #' path = local))
    #' }


    get_positivePPI <- function(organism.taxID,
                                access.key,
                                filename = "PositiveInt.RData",
                                path = "PositiveInt") {
      `Official Symbol Interactor A` <- NULL
      `Official Symbol Interactor B` <- NULL

      # construct query to retrive all interactiosn for the requested organism
      url <-
        "http://webservice.thebiogrid.org/interactions/?"
      query <- paste(url,
        paste0("accessKey=", access.key),
        "interSpeciesExcluded=false",
        "selfInteractionsExcluded=true",
        "includeHeader=true",
        paste0("taxId=", organism.taxID),
        sep = "&"
      )


      # number of results
      n <-
        httr::GET(paste(query, "format=count", sep = "&"))
      httr::stop_for_status(n)


      n <- as.numeric(
        httr::content(n, type = "text/csv", encoding = "UTF-8", col_types = "i")
      )

      # retrieve results in batches
      tpINT <- lapply(seq(1, n, 10000), function(m) {
        l <- paste(query, paste0("start=", m), sep = "&")
        request <-
          httr::GET(l)
        httr::stop_for_status(request)

        # parse the biogird results
        biog_result <- httr::content(request,
          type = "text/tab-separated-values",
          encoding = "UTF-8",
          col_types = "cccccccccccccccccccccccc"
        )
        q <- subset(biog_result, select = c(
          "Official Symbol Interactor A",
          "Official Symbol Interactor B",
          "Organism Interactor A",
          "Organism Interactor B",
          "Experimental System Type"
        ))

        return(q)
      })

      tpINT <- do.call("rbind", tpINT)
      dataTP <-
        tpINT
      dataTP <- # remove duplicate
        tpINT[!duplicated(apply(tpINT, 1, function(x) {
          paste(sort(x), collapse = "")
        })), ]

      dataTP <-
        dataTP %>%
        mutate(PPI = paste(`Official Symbol Interactor A`,
          `Official Symbol Interactor B`,
          sep = "~"
        )) %>%
        dplyr::select(6, seq_len(5))

      if (!file.exists(path)) {
        dir.create(path, recursive = TRUE)
        localPath <- tools::file_path_as_absolute(path)
        fname <- file.path(localPath, filename)
        save(dataTP, file = fname)
      } else {
        fname <- file.path(path, filename)
        save(dataTP, file = fname)
      }

      return(dataTP)
    }

test_getpositivePPI = function() {
  local = tempdir()
  df <- get_positivePPI(organism.taxID = 2697049,
  access.key = '81bb3b5a6bd9a8084a7be71f0963ab1e',
  filename = "PositiveInt.RData", path = local)
  checkTrue(is.data.frame(df) == TRUE)
  checkEquals(ncol(df), 6L)

}

    #' Input Data for Prediction Algorithm
    #' @description Input data for \code{\link[HPiP]{pred_ensembel}}
    #' @name example_data
    #' @usage data(example_data)
    #' @docType data
    #' @format
    #' a data.frame containing unlabeled or labeled HP-PPIs and pre-computed
    #' numerical features.
    NULL

    .nchar <- function(x) {
      . <- NULL
      # nchar of each fasta sequence
      nchar_count <-
        x %>%
        lapply(., function(x) strsplit(x[[1]], "")) %>%
        lapply(., function(x) length(x[[1]]))

      # convert the list to the df
      nchar_df <-
        bind_rows(nchar_count) %>%
        gather("identifier", "nchar", seq_len(ncol(.))) %>%
        mutate(nchar = nchar)

      return(nchar_df)
    }



    .cld <- function(x, g, nchar_df) {
      . <- NULL
      V1 <- NULL
      value <- NULL
      ID <- NULL
      group <- NULL
      df <- do.call(rbind, x) %>%
        as.data.frame(.) %>%
        rename(value = V1) %>%
        rownames_to_column("ID") %>%
        separate(ID, c("group", "identifier"), sep = "\\.") %>%
        mutate(group = paste0(g, ".", group)) %>%
        left_join(., nchar_df, by = "identifier") %>%
        mutate(value = value / nchar) %>%
        select(2, 1, 3)
      return(df)
    }




    #' calculateCTDC
    #' @title Calculate CTD Descriptors - Composition (C)
    #' @param x A data.frame containing gene/protein names and
    #' their fasta sequences.
    #' @return A length 21 named vector for the data input.
    #' @seealso See \code{\link{calculateCTDT}} and \code{\link{calculateCTDD}}
    #' for Transition and Distribution descriptors.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom dplyr rename
    #' @description This function calculates Composition (C)
    #' descriptor for data input.
    #' @export
    #' @references
    #' Dubchak, I., Muchnik, I., Holbrook, S. R., and Kim, S.-H. (1995).
    #' Prediction of protein folding class using global description of
    #' amino acid sequence.\emph{Proc. Natl. Acad. Sci.} 92, 8700–8704.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateCTDC(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateCTDC <- function(x) {
      . <- NULL
      V1 <- NULL
      value <- NULL
      group <- NULL

      if (!is.data.frame(x)) {
        stop("Input data must be data.frame")
      }

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }

      # three-group classification of the 20 amino acids by each attribute

      group1 <- list(
        "hydrophobicity" = c("R", "K", "E", "D", "Q", "N"),
        "normwaalsvolume" = c("G", "A", "S", "T", "P", "D", "C"),
        "polarity" = c("L", "I", "F", "W", "C", "M", "V", "Y"),
        "polarizability" = c("G", "A", "S", "D", "T"),
        "charge" = c("K", "R"),
        "secondarystruct" = c("E", "A", "L", "M", "Q", "K", "R", "H"),
        "solventaccess" = c("A", "L", "F", "C", "G", "I", "V", "W")
      )

      group2 <- list(
        "hydrophobicity" = c("G", "A", "S", "T", "P", "H", "Y"),
        "normwaalsvolume" = c("N", "V", "E", "Q", "I", "L"),
        "polarity" = c("P", "A", "T", "G", "S"),
        "polarizability" = c("C", "P", "N", "V", "E", "Q", "I", "L"),
        "charge" = c(
          "A", "N", "C", "Q", "G", "H", "I", "L",
          "M", "F", "P", "S", "T", "W", "Y", "V"
        ),
        "secondarystruct" = c("V", "I", "Y", "C", "W", "F", "T"),
        "solventaccess" = c("R", "K", "Q", "E", "N", "D")
      )

      group3 <- list(
        "hydrophobicity" = c("C", "L", "V", "I", "M", "F", "W"),
        "normwaalsvolume" = c("M", "H", "K", "F", "R", "Y", "W"),
        "polarity" = c("H", "Q", "R", "K", "N", "E", "D"),
        "polarizability" = c("K", "M", "H", "F", "R", "Y", "W"),
        "charge" = c("D", "E"),
        "secondarystruct" = c("G", "N", "P", "S", "D"),
        "solventaccess" = c("M", "S", "P", "T", "H", "Y")
      )


      # nchar of each fasta sequence
      nchar_df <- .nchar(fastalist)


      fastaSplitted <-
        fastalist %>% lapply(., function(x) strsplit(x[[1]], ""))


      g1 <- unlist(lapply(group1, function(X) {
        lapply(fastaSplitted, function(Y) {
          length(which(Y[[1]] %in% X))
        })
      }), recursive = FALSE)


      g2 <- unlist(lapply(group2, function(X) {
        lapply(fastaSplitted, function(Y) {
          length(which(Y[[1]] %in% X))
        })
      }), recursive = FALSE)

      g3 <- unlist(lapply(group3, function(X) {
        lapply(fastaSplitted, function(Y) {
          length(which(Y[[1]] %in% X))
        })
      }), recursive = FALSE)


      g1 <- .cld(g1, "G1", nchar_df)
      g2 <- .cld(g2, "G2", nchar_df)
      g3 <- .cld(g3, "G3", nchar_df)


      dfOut <-
        rbind(g1, g2, g3) %>% spread(group, value)
      dfOut[is.na(dfOut)] <- 0

      return(dfOut)
    }

    #' calculateTC_Sm
    #' @title Calculate Tripeptide Composition (TC) Descriptor from Biochemical
    #' Similarity Classes
    #' @param x A data.frame containing gene/protein names and their fasta
    #' sequences.
    #' @return A length 216  named vector for the data input.
    #' @seealso See \code{\link{calculateTC}} for Tripeptide Composition
    #' descriptor.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom magrittr set_rownames
    #' @description This function calculates Tripeptide Composition (TC)
    #' descriptor from biochemical similarity classes.
    #' @export
    #' @references
    #' Ahmed, I., Witbooi, P., and Christoffels, A. (2018). Prediction of
    #' human-Bacillus anthracis protein–protein interactions using multi-layer
    #' neural network.\emph{Bioinformatics} 34, 4159–4164.
    #'
    #' Cui, G., Fang, C., and Han, K. (2012). Prediction of protein-protein
    #' interactions between viruses and human by an SVM model.
    #' \emph{BMC bioinformatics}, 1–10.
    #'
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateTC_Sm(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateTC_Sm <- function(x) {
      if (!is.data.frame(x)) x <- is.data.frame(x)

      . <- NULL
      V1 <- NULL

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }

      classes <- vector("list", 6)
      classes[[1]] <- c("I", "V", "L", "M")
      classes[[2]] <- c("F", "Y", "W")
      classes[[3]] <- c("H", "K", "R")
      classes[[4]] <- c("D", "E")
      classes[[5]] <- c("Q", "N", "T", "P")
      classes[[6]] <- c("A", "C", "G", "S")


      vspace <- expand.grid(seq_len(6), seq_len(6), seq_len(6))

      BSlict <- vector("list", 216)

      for (i in seq_len(216)) {
        tmp <- as.vector(outer(classes[[vspace[i, 1]]],
          classes[[vspace[i, 2]]], paste,
          sep = ""
        ))
        BSlict[[i]] <- as.vector(outer(tmp,
          classes[[vspace[i, 3]]], paste,
          sep = ""
        ))
      }
      names(BSlict) <- paste(seq_len(length(BSlict)))



      # compute TC_Sm
      p <- function(x, BSlict) {
        f <-
          x %>%
          lapply(., function(x) strsplit(x[[1]], "")) %>%
          lapply(., function(x) {
            paste(paste(x[[1]][-c(length(x[[1]]), length(x[[1]]) - 1)],
              x[[1]][-c(1, length(x[[1]]))],
              sep = ""
            ), x[[1]][-c(1, 2)],
            sep = ""
            )
          })

        inter <- unlist(lapply(BSlict, function(X) {
          lapply(f, function(Y) {
            length(which(Y %in% X))
          })
        }), recursive = FALSE)

        s <-
          do.call(cbind, inter) %>%
          set_rownames(names(x))
        s <- t(apply(s, 1, function(x) (x - min(x)) / (max(x) - min(x))))
        return(s)
      }

      TC_Sm <- lapply(fastalist, function(x) p(x[[1]], BSlict))
      TC_Sm_df <-
        do.call(rbind, TC_Sm)

      TC_Sm_df <-
        TC_Sm_df %>%
        as.data.frame(.) %>%
        set_rownames(names(TC_Sm)) %>%
        rownames_to_column("identifier")

      return(TC_Sm_df)
    }

    #' Gold-standard Reference Set of Inter-Species PPIs
    #' @description This dataset consists of experimentally validated
    #' human-SARS-CoV-2 interactions (positive set) and
    #' non-interacting pairs (negative set).
    #' The following data consists of:
    #' \itemize{
    #'   \item PPI: SARS-CoV-2-human protein-protein interactions (PPIs)
    #'   \item Official Symbol Interactor A: SARS-CoV-2 gene names
    #'   \item official Symbol Interactor B: human host gene names
    #'   \item Pathogen_Protein: UniProt identifiers for SARS-CoV-2 virus
    #'   \item Host_Protein: UniProt identifiers for human proteins
    #'   \item class: labeled examples (both positive and negative)
    #' }
    #' @details
    #' To construct this dataset, validated interactions (positive set) were
    #' retrieved from BioGrid database and were further filtered to only include
    #' those interactions provided by (Samavarchi-Tehrani et al., 2020).
    #' In this study, the authors mapped interaction between 27 SARS-CoV-2 and
    #' human proteins via the proximity-dependent biotinylation (BioID)
    #' approach. 500 SARS-CoV-2-host interaction pairs then randomly
    #' selected from all pairs to serve as positive examples.
    #' To construct negative examples,negative sampling were used using
    #' \code{\link[HPiP]{get_negativePPI}}.
    #' @name Gold_ReferenceSet
    #' @usage data(Gold_ReferenceSet)
    #' @docType data
    #' @format a data.frame containing 500 validated pairs
    #' (i.e., positive set) and 500 non-interacting pairs (i.e., negative set).
    #' @source
    #' \url{https://www.biorxiv.org/content/10.1101/2020.09.03.282103v1}
    #' @references
    #' Samavarchi-Tehrani,P. et al.
    #' (2020) A SARS-CoV-2-host proximity interactome. BioRxiv.
    NULL

    #' enrichfindP
    #' @title Functional Enrichment Analysis for Pathogen Interactors in
    #' the High-Confidence Network.
    #' @param ppi A data.frame containing pathogen proteins in the
    #' first column and host proteins in the second column.
    #' @param threshold Custom p-value threshold for significance.
    #' @param sources A vector of data sources to use.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @param p.corrction.method The algorithm used for multiple testing
    #' correction;defaults to 'bonferroni'.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @param org An organism name;defaults to 'hsapiens'.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @return A data.frame with the enrichment analysis results.
    #' @seealso See \code{\link{enrichplot}} for plotting enrichment analysis.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom purrr map_df
    #' @importFrom stats aggregate
    #' @importFrom stringr str_split
    #' @description This function uses \code{\link[gprofiler2]{gost}} function
    #' in \code{gprofiler2} package to perfrom functional enrichment analysis
    #' for pathogen interactors in the high-confidence network.
    #' @export
    #' @examples
    #' data('predicted_PPIs')
    #' #perform enrichment
    #' enrich.df <- enrichfindP(predicted_PPIs,
    #' threshold = 0.05,
    #' sources = c("GO", "KEGG"),
    #' p.corrction.method = "bonferroni",
    #' org = "hsapiens")

    enrichfindP <-
      function(ppi, threshold = 0.05,
               sources = c("GO", "KEGG"),
               p.corrction.method = "bonferroni",
               org = "hsapiens") {
        p_value <- NULL
        ppi <- as.data.frame(ppi)

        agg.data <-
          aggregate(list(ppi[, 2]),
            by = list(ppi[, 1]),
            FUN = function(x) {
              paste(unique(x),
                collapse = ";"
              )
            }
          )
        colnames(agg.data)[2] <- "interactors"
        agg.data$interactors <-
          vapply(lapply(strsplit(agg.data$interactors, ";"), unique),
            paste, character(1L),
            collapse = ";"
          )


        indcpx <-
          str_split(agg.data$interactors, ";")
        names(indcpx) <- agg.data$Group.1

        annotCov <- lapply(indcpx, function(x) {
          gprofiler2::gost(x,
            significant = TRUE,
            exclude_iea = TRUE,
            evcodes = TRUE,
            user_threshold = threshold,
            sources = sources,
            correction_method = p.corrction.method,
            organism = org
          )
        })

        df <-
          lapply(annotCov, function(x) as.data.frame(x[[1]]))
        ans <-
          map_df(df, ~ as.data.frame(.x), .id = "id")

        ans <-
          arrange(ans, p_value)

        return(ans)
      }

    .gintersect <- function(x, y, gn = "G1") {
      g1 <- unlist(lapply(x, function(X) {
        lapply(y, function(Y) {
          ifelse(Y[[1]] %in% X, gn, NA)
        })
      }), recursive = FALSE)
    }


    .mapL <- function(x, y, z) {
      list.g1g2 <- Map(list, x, y)

      list.g1g2 <-
        lapply(list.g1g2, function(x) {
          ifelse(is.na(x[[1]]), x[[2]], x[[1]])
        })

      list.g1g2g3 <-
        Map(list, list.g1g2, z)

      list.g1g2g3 <-
        lapply(list.g1g2g3, function(x) {
          ifelse(is.na(x[[1]]), x[[2]], x[[1]])
        })

      return(list.g1g2g3)
    }


    #' calculateCTDT
    #' @title Calculate CTD Descriptors - Transition (T)
    #' @param x A data.frame containing gene/protein names and
    #' their fasta sequences.
    #' @return A length 21 named vector for the data input.
    #' @seealso See \code{\link{calculateCTDC}} and \code{\link{calculateCTDD}}
    #' for Composition and Distribution descriptors.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom dplyr full_join
    #' @importFrom dplyr select
    #' @description This function calculates Transition (T) descriptor
    #' for data input.
    #' @export
    #' @references
    #' Dubchak, I., Muchnik, I., Holbrook, S. R., and Kim, S.-H. (1995).
    #' Prediction of protein folding class using global description of
    #' amino acid sequence.
    #' \emph{Proc. Natl. Acad. Sci.} 92, 8700–8704.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateCTDT(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateCTDT <- function(x) {
      . <- NULL
      V1 <- NULL
      ID <- NULL
      group <- NULL
      Tr1221.prob <- NULL
      Tr1331.prop <- NULL
      Tr2332.prob <- NULL
      Tclass <- NULL
      count_CTDT <- NULL




      if (!is.data.frame(x)) {
        stop("Input data must be data.frame")
      }

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }


      # three-group classification of the 20 amino acids by each attribute

      group1 <- list(
        "hydrophobicity" = c("R", "K", "E", "D", "Q", "N"),
        "normwaalsvolume" = c("G", "A", "S", "T", "P", "D", "C"),
        "polarity" = c("L", "I", "F", "W", "C", "M", "V", "Y"),
        "polarizability" = c("G", "A", "S", "D", "T"),
        "charge" = c("K", "R"),
        "secondarystruct" = c("E", "A", "L", "M", "Q", "K", "R", "H"),
        "solventaccess" = c("A", "L", "F", "C", "G", "I", "V", "W")
      )

      group2 <- list(
        "hydrophobicity" = c("G", "A", "S", "T", "P", "H", "Y"),
        "normwaalsvolume" = c("N", "V", "E", "Q", "I", "L"),
        "polarity" = c("P", "A", "T", "G", "S"),
        "polarizability" = c("C", "P", "N", "V", "E", "Q", "I", "L"),
        "charge" = c(
          "A", "N", "C", "Q", "G", "H", "I", "L",
          "M", "F", "P", "S", "T", "W", "Y", "V"
        ),
        "secondarystruct" = c("V", "I", "Y", "C", "W", "F", "T"),
        "solventaccess" = c("R", "K", "Q", "E", "N", "D")
      )

      group3 <- list(
        "hydrophobicity" = c("C", "L", "V", "I", "M", "F", "W"),
        "normwaalsvolume" = c("M", "H", "K", "F", "R", "Y", "W"),
        "polarity" = c("H", "Q", "R", "K", "N", "E", "D"),
        "polarizability" = c("K", "M", "H", "F", "R", "Y", "W"),
        "charge" = c("D", "E"),
        "secondarystruct" = c("G", "N", "P", "S", "D"),
        "solventaccess" = c("M", "S", "P", "T", "H", "Y")
      )


      vect1 <- c("G1", "G2", "G3")
      vect2 <- c("G1", "G2", "G3")
      TClist <-
        apply(expand.grid(vect1, vect2), 1, paste, collapse = "")


      nchar_df <- .nchar(fastalist)


      fastaSplitted <-
        fastalist %>% lapply(., function(x) strsplit(x[[1]], ""))

      g1 <- .gintersect(group1, fastaSplitted, gn = "G1")
      g2 <- .gintersect(group2, fastaSplitted, gn = "G2")
      g3 <- .gintersect(group3, fastaSplitted, gn = "G3")
      list.g1g2g3 <- .mapL(g1, g2, g3)


      CTDT_calc <-
        list.g1g2g3 %>%
        lapply(., function(x) paste(x[-length(x)], x[-1], sep = "")) %>%
        lapply(., function(x) factor(x, levels = TClist)) %>%
        lapply(., function(x) table(x))

      G1G2.G2G1 <-
        CTDT_calc %>%
        lapply(., function(x) sum(x[c("G1G2", "G2G1")])) %>%
        do.call(rbind, .) %>%
        as.data.frame(.) %>%
        rename(G1G2.G2G1 = V1) %>%
        rownames_to_column("ID")

      G1G3.G3G1 <-
        CTDT_calc %>%
        lapply(., function(x) sum(x[c("G1G3", "G3G1")])) %>%
        do.call(rbind, .) %>%
        as.data.frame(.) %>%
        rename(G1G3.G3G1 = V1) %>%
        rownames_to_column("ID")


      G2G3.G3G2 <-
        CTDT_calc %>%
        lapply(., function(x) sum(x[c("G2G3", "G3G2")])) %>%
        do.call(rbind, .) %>%
        as.data.frame(.) %>%
        rename(G2G3.G3G2 = V1) %>%
        rownames_to_column("ID")

      dfOut <-
        full_join(G1G2.G2G1, G1G3.G3G1, by = "ID") %>%
        full_join(., G2G3.G3G2, by = "ID") %>%
        separate(ID, c("group", "identifier"), sep = "\\.") %>%
        left_join(., nchar_df, by = "identifier") %>%
        mutate(nchar = nchar - 1) %>%
        mutate(Tr1221.prob = G1G2.G2G1 / nchar) %>%
        mutate(Tr1331.prop = G1G3.G3G1 / nchar) %>%
        mutate(Tr2332.prob = G2G3.G3G2 / nchar) %>%
        select(2, 1, 7:9) %>%
        gather("Tclass", "count_CTDT", 3:ncol(.)) %>%
        mutate(Tclass = paste(group, Tclass, sep = ".")) %>%
        select(-2) %>%
        spread(Tclass, count_CTDT)

      return(dfOut)
    }

    #' calculateKSAAP
    #' @title Calculate k-spaced Amino Acid Pairs (KSAAP) Descriptor
    #' @param x A data.frame containing gene/protein names and their fasta
    #' sequences.
    #' @param spc A number of spaces separating two adjacent residues by a
    #' distance of spc, which can be any number up to two less than
    #' the length of the peptide; default to 3.
    #' @return A length 400 named vector for the data input.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description This function calculates k-spaced Amino Acid Pairs
    #' (KSAAP) Descriptor for data input.
    #' This function is adapted from the \code{\link[ftrCOOL]{CkSAApair}}
    #'  function in the ftrCOOL package.
    #' @export
    #' @references
    #' Kao, H.-J., Nguyen, V.-N., Huang, K.-Y., Chang, W.-C., and Lee, T.-Y.
    #' (2020).SuccSite: incorporating amino acid composition and informative
    #' k-spaced amino acid pairs to identify protein succinylation sites.
    #' \emph{Genomics. Proteomics Bioinformatics} 18, 208–219.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateKSAAP(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateKSAAP <- function(x, spc = 3) {
      if (!is.data.frame(x)) {
        stop("Input data must be data.frame")
      }

      . <- NULL
      V1 <- NULL

      AADict <-
        list(
          "A" = 1, "C" = 2, "D" = 3, "E" = 4, "F" = 5, "G" = 6,
          "H" = 7, "I" = 8, "K" = 9, "L" = 10, "M" = 11, "N" = 12,
          "P" = 13, "Q" = 14, "R" = 15, "S" = 16, "T" = 17, "V" = 18,
          "W" = 19, "Y" = 20
        )


      vect <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )


      DClist <-
        apply(expand.grid(vect, vect), 1, paste, collapse = " ")

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))

      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }

      seqs <- unlist(x[, 2])
      label <- names(fastalist)
      nSeqs <- length(fastalist)
      len <- 1

      attributesM <- matrix(0, ncol = 400, nrow = nSeqs)
      row.names(attributesM) <- label

      attributeName <- vector()
      for (i in seq_len(len)) {
        attributeName <- c(attributeName, gsub(
          " ", strrep("s", spc[i]),
          DClist
        ))
      }
      colnames(attributesM) <- attributeName


      for (n in seq_len(nSeqs)) {
        seq <- seqs[n]
        seqChars <- unlist(strsplit(seq, split = ""))
        lenSeq <- length(seqChars)

        for (i in seq_len(len)) {
          temp1 <- seqChars[seq_len(lenSeq - spc[i] - 1)]
          temp2 <- seqChars[((spc[i] + 1) + 1):(lenSeq)]
          kmers <- paste(temp1, temp2, sep = "")
          tbkmers <- table(kmers)
          nmtbkmers <- names(tbkmers)
          for (j in seq_along(tbkmers)) {
            tmp <- unlist(strsplit(nmtbkmers[j], split = ""))
            index <- (as.numeric(AADict[tmp[1]]) - 1) * 20 +
              as.numeric(AADict[tmp[2]])
            index <- index + ((i - 1) * 400)
            attributesM[n, index] <- tbkmers[j]
          }
        }
      }

      # normalize
      seqLen <- vapply(seqs, nchar, FUN.VALUE = numeric(1))
      attributesM <- attributesM / seqLen

      attributesdf <-
        attributesM %>%
        as.data.frame(.) %>%
        rownames_to_column("identifier")

      return(attributesdf)
    }

    .filter.corr <-
      function(features, class, cor.cutoff = 0.7) {
        . <- NULL
        x <- NULL
        if (!is.matrix(features)) features <- as.matrix(features)
        descrCorr <- cor(features)

        # find correlated features
        highCorr <-
          caret::findCorrelation(descrCorr, cor.cutoff, names = TRUE)

        # find the remining features
        clcol <- which(names(as.data.frame(features)) %in% highCorr)
        opt_var <- colnames(features[, -clcol])


        # remove correlated features
        reduced_Data <-
          features %>%
          as.data.frame(.) %>%
          select(-all_of(highCorr)) %>%
          rownames_to_column("PPI") %>%
          cbind(class, .)

        ret <- list()
        ret$corProfile <- descrCorr
        ret$corSelectedFeatures <- opt_var
        ret$cordf <- reduced_Data

        return(ret) # return RF profile
      }

    .rfeFS <- function(features, class, cor.cutoff = 0.7,
                       resampling.method = "cv", iter = 2, repeats = 2,
                       metric = "Accuracy", verbose = TRUE) {
      if (!is.data.frame(x)) x <- as.data.frame(x)

      . <- NULL

      caretFuncs <- rfFuncs
      if (metric == "ROC") {
        caretFuncs$summary <- twoClassSummary
      }

      sizeF <- ncol(features)
      #### recursive feature elimination-random forest
      rfProfile <- caret::rfe(
        x = features,
        y = as.factor(class),
        sizes = seq_len(sizeF), # all possible features to be tested
        metric = metric,
        rfeControl = rfeControl(
          functions = caretFuncs,
          method = resampling.method,
          repeats = repeats,
          number = iter,
          allowParallel = TRUE,
          verbose = verbose
        )
      )

      opt_var <- predictors(rfProfile)
      dataset_rfe <- features[, c(opt_var)]
      dataset_rfe <-
        dataset_rfe %>%
        as.data.frame(.) %>%
        tibble::rownames_to_column("PPI")
      dataset_rfe <- cbind(class, dataset_rfe)

      ret <- list()
      ret$rfProfile <- rfProfile
      ret$rfSelectedFeatures <- opt_var
      ret$rfdf <- dataset_rfe

      return(ret) # return RF profile
    }

    #' FSmethod
    #' @title Feature Selection via Matrix Correlation and
    #' Recursive Feature Elimination (RFE)
    #' @param x A data.frame containing protein-protein interactions,
    #' class labels and features.
    #' @param type The feature selection type, one or two of
    #' \code{filter.corr} and \code{rfeFS}.
    #' @param cor.cutoff Correlation coefficient cutoff used for filtering.
    #' See \code{filter.corr} for more details.
    #' @param resampling.method The resampling method for RFE :'boot',
    #' 'boot632', optimism_boot',boot_all', 'cv', 'repeatedcv', 'LOOCV',
    #' 'LGOCV';defaults to cv. See \code{rfeFS} and
    #' \code{\link[caret]{rfeControl}} for more details.
    #' @param iter Number of partitions for cross-validation;
    #' defaults to 2. See \code{rfeFS} and \code{\link[caret]{rfeControl}}
    #' for more details.
    #' @param repeats For repeated k-fold cross validation only;
    #'  defaults to 3.See \code{rfeFS} and \code{\link[caret]{rfeControl}}
    #'  for more details.
    #' @param metric  A string that specifies what summary metric will be used
    #' to select the optimal feature ; default to ROC.See \code{rfeFS} and
    #' \code{\link[caret]{rfe}} for more details.
    #' @param verbose Make the output verbose.See \code{rfeFS} and
    #' \code{\link[caret]{rfeControl}} for more details.

    #' @return
    #' If the type set to \code{filter.corr} , the output includes
    #' the following elements:
    #' \itemize{
    #' \item{corProfile} - A correlation matrix.
    #' \item{corSelectedFeatures} - Name of features that retained after
    #' the correlation analysis.
    #' \item{cordf} - A data.frame filtered.
    #' }
    #' If the type set to \code{rfeFS} , the output includes the following
    #' elements:
    #' \itemize{
    #' \item{rfProfile} - A list of elements. See \code{\link[caret]{rfe}} for
    #' more details.
    #' \item{rfSelectedFeatures} - Name of features that retained in the feature
    #' selection process.
    #' \item{rfdf} - A data.frame filtered.
    #' }
    #' If type set to \code{both} the output includes the following elements:
    #' \itemize{
    #' \item{rfdf} - The final data.frame that includes the selected features
    #' retained after both \code{filter.corr} and \code{rfeFS} analysis.
    #' }

    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}.
    #' @importFrom stats cor
    #' @importFrom caret twoClassSummary
    #' @importFrom dplyr select
    #' @importFrom dplyr all_of
    #' @importFrom tibble rownames_to_column
    #' @importFrom caret rfeControl
    #' @importFrom caret predictors
    #' @importFrom caret rfFuncs
    #' @description This function performs feature selections
    #' via two approaches
    #' \itemize{
    #' \item \code{filter.corr} - compute matrix correlation between features
    #' and filter using a threshold.
    #' \item \code{rfeFS} - perform recursive feature elimination (RFE) method
    #' wrapped with a Random Forest (RF) algorithm for feature importance
    #' evaluation.
    #' }
    #' @export
    #' @examples
    #' data('example_data')
    #' x <- na.omit(example_data)
    #' s <- FSmethod(x, type = 'both',
    #' cor.cutoff = 0.7, resampling.method = "repeatedcv",
    #' iter = 5, repeats = 3, metric = "ROC", verbose = TRUE)


    FSmethod <- function(x, type = c("cor", "rfe", "both"),
                         cor.cutoff = 0.7, resampling.method = "cv",
                         iter = 2, repeats = 3, metric = "Accuracy",
                         verbose = TRUE) {
      PPI <- NULL
      . <- NULL


      if (!is.data.frame(x)) x <- as.data.frame(x)

      if (all(colnames(x) != "class") == TRUE) {
        stop("class attribute is absent from the data.frame")
      }

      if (all(colnames(x) != "PPI") == TRUE) {
        stop("PPI attribute is absent from the data.frame")
      }

      if (missing(type)) stop("Must provide at least one feature selection
                          approach")

      if (!type %in% c("cor", "rfe", "both")) {
        stop("Feature selection type must be one of cor,rfe or both...")
      }
      features <-
        x %>%
        as.data.frame(.) %>%
        magrittr::set_rownames(.$PPI) %>%
        dplyr::select(-PPI, -class)

      class <-
        x$class

      if (type == "cor") {
        result <- .filter.corr(features, class, cor.cutoff)
      } else if (type == "rfe") {
        result <- .rfeFS(features, class,
          resampling.method = resampling.method,
          iter = iter, repeats = repeats,
          metric = metric, verbose = verbose
        )
      } else if (type == "both") {
        cor.result <-
          .filter.corr(features, class, cor.cutoff)
        df <-
          cor.result[["cordf"]]
        df.features <-
          df %>%
          as.data.frame(.) %>%
          magrittr::set_rownames(.$PPI) %>%
          dplyr::select(-PPI, -class)

        df.class <-
          df$class

        rf.result <-
          .rfeFS(df.features, df.class,
            resampling.method = resampling.method,
            iter = iter, repeats = repeats,
            metric = metric, verbose = verbose
          )

        result <- list()
        result$cor.result <- cor.result
        result$rf.result <- rf.result
      } else {
        stop('Feature selection type must be in "cor" and "rfe" or "both"')
      }

      return(result)
    }

    #' Enrichment Result
    #' @description Input data for \code{\link[HPiP]{enrichplot}}
    #' @details
    #' To construct this dataset, predicted interactions, generated
    #' from \code{\link[HPiP]{pred_ensembel}} was used as data input for
    #' \code{\link[HPiP]{enrichplot}}.
    #' @name enrich.df
    #' @usage data(enrich.df)
    #' @docType data
    NULL

    .checkFASTA <- function(x) {
      . <- NULL
      V1 <- NULL
      fastalist.check <-
        lapply(x, function(x) protr::protcheck(x))
      fastalist.check <-
        do.call(rbind, fastalist.check) %>%
        as.data.frame(.) %>%
        rownames_to_column("ID") %>%
        filter(V1 == FALSE)
    }

    #' calculateAAC
    #' @title Calculate Amino Acid Composition (AAC) Descriptor
    #' @param x A data.frame containing gene/protein names and their
    #' fasta sequences.
    #' @return A length 20 named vector for the data input.
    #' @seealso See \code{\link{calculateDC}} and
    #' \code{\link{calculateTC}} for Dipeptide Composition and
    #' Tripeptide Composition descriptors.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom dplyr filter
    #' @importFrom dplyr left_join
    #' @importFrom dplyr bind_rows
    #' @importFrom tidyr gather
    #' @importFrom tibble rownames_to_column
    #' @description This function calculates Amino Acid Composition
    #' (AAC) descriptor for the data input.
    #' @export
    #' @references
    #' Dey, L., Chakraborty, S., and Mukhopadhyay, A. (2020).
    #' Machine learning techniques for sequence-based prediction of
    #' viral–host interactions between SARS-CoV-2 and human proteins.
    #' \emph{Biomed. J.} 43, 438–450.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateAAC(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateAAC <- function(x) {
      . <- NULL
      V1 <- NULL
      AAC <- NULL
      AACfreq <- NULL
      count_AAC <- NULL



      if (!is.data.frame(x)) x <- is.data.frame(x)

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid

      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }


      # 20 Amino Acid Abbreviation
      AADict <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )


      # calculate amino acid composition descriptor
      AAC_calc <-
        fastalist %>%
        lapply(., function(x) strsplit(x[[1]], "")) %>%
        lapply(., function(x) {
          summary(factor(x[[1]], levels = AADict),
            maxsum = 20
          ) / lengths(x)
        })


      AAC_calc_df <-
        bind_rows(AAC_calc, .id = "identifier") %>%
        gather("AAC", "AACfreq", 2:ncol(.)) %>%
        spread(AAC, AACfreq) %>%
        as.data.frame(.)

      return(AAC_calc_df)
    }

    #' calculateDC
    #' @title Calculate Dipeptide Composition (DC) Descriptor
    #' @param x A data.frame containing gene/protein names and their fasta
    #' sequences.
    #' @return A length 400 named vector for the data input.
    #' @seealso See \code{\link{calculateAAC}} and \code{\link{calculateTC}}
    #' for Amino Acid Composition and Tripeptide Composition descriptors.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description This function calculates Dipeptide Composition (DC)
    #' descriptor for data input.
    #' @export
    #' @references
    #' Bhasin, M., and Raghava, G. P. S. (2004).
    #' Classification of nuclear receptors based on amino acid composition
    #' and dipeptide composition. \emph{J. Biol. Chem.} 279, 23262–23266.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateDC(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateDC <- function(x) {
      . <- NULL
      V1 <- NULL
      DCfreq <- NULL
      count_DC <- NULL
      DC <- NULL

      if (!is.data.frame(x)) x <- is.data.frame(x)

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }


      vect1 <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )
      vect2 <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )


      DClist <-
        apply(expand.grid(vect1, vect2), 1, paste, collapse = "")

      # calculate dipeptide composition descriptor
      DC_calc <-
        fastalist %>%
        lapply(., function(x) strsplit(x[[1]], "")) %>%
        lapply(., function(x) {
          summary(factor(paste(x[[1]][-length(x[[1]])], x[[1]][-1],
            sep = ""
          ), levels = DClist), maxsum = 400) / (lengths(x) - 1)
        })

      DC_calc_df <-
        bind_rows(DC_calc, .id = "identifier") %>%
        gather("DC", "DCfreq", 2:ncol(.)) %>%
        spread(., DC, DCfreq) %>%
        as.data.frame(.)

      return(DC_calc_df)
    }

    #' FreqInteractors
    #' @title Plot the Pathogen Proteins' frequency of Interactions with Host
    #' Proteins
    #' @param ppi A data.frame containing pathogen proteins in the first column
    #' and host proteins in the second column.
    #' @param cex.size Text size.
    #' @return A frequency plot.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom ggplot2 element_rect
    #' @importFrom ggplot2 coord_flip
    #' @importFrom ggplot2 element_rect
    #' @importFrom stats aggregate
    #' @description This function plots the pathogen proteins' Frequency of
    #' interactions with host proteins
    #' @export
    #' @examples
    #' ppi <- data.frame(
    #'     node1 = c("A", "A", "A", "B", "B", "B", "B"),
    #'     node2 = c("C", "E", "D", "F", "G", "H", "I")
    #' )
    #' FreqInteractors(ppi)
    FreqInteractors <- function(ppi, cex.size = 12) {
      Group.1 <- NULL
      sizeInt <- NULL

      agg.data <-
        aggregate(list(ppi[, 2]),
          by = list(ppi[, 1]),
          FUN = function(x) {
            paste(unique(x),
              collapse = ";"
            )
          }
        )
      colnames(agg.data)[2] <- "interactors"
      agg.data$interactors <-
        vapply(lapply(strsplit(agg.data$interactors, ";"), unique),
          paste, character(1L),
          collapse = ";"
        )

      agg.data$sizeInt <- lengths(strsplit(agg.data$interactors, ";"))
      agg.data <- agg.data[, c(1, 3)]

      print(ggplot2::ggplot(agg.data, aes(Group.1, y = sizeInt)) +
        geom_point(size = 3) +
        coord_flip() +
        ggthemes::theme_hc() +
        theme(axis.text.y = element_text(
          color = "black", # label color and size
          size = cex.size, lineheight = 0.9, colour = "black"
        )) +
        theme(axis.text.x = element_text(
          color = "black", # label color and size
          size = cex.size, lineheight = 0.9, colour = "black"
        )) +
        theme(axis.line = element_line(
          colour = "black",
          size = 0.5, linetype = "solid"
        )) +
        theme(axis.ticks = element_line(
          colour = "black",
          size = 0.5, linetype = "solid"
        )) +
        labs(title = "", x = "", y = "Number of Interactors") +
        theme(
          legend.title = element_blank(),
          legend.spacing.y = unit(0, "mm"),
          panel.border = element_rect(colour = "black", fill = NA),
          aspect.ratio = 1, axis.text = element_text(colour = 1, size = 12),
          legend.position = "top",
          legend.background = element_blank(),
          legend.box.background = element_rect(colour = "black")
        ) +
        theme(text = element_text(size = cex.size, colour = "black")) +
        theme(axis.ticks.length = unit(.25, "cm")))
    }

    .getKronecker <- function(A, B) {
      m <- matrix(0, nrow = nrow(A) * nrow(B), ncol = ncol(A) * ncol(B))

      for (i in seq_len(nrow(A))) {
        cur_rows <- seq_len(nrow(B)) + (i - 1) * nrow(B)
        for (j in seq_len(ncol(A))) {
          m[cur_rows, seq_len(ncol(B)) + (j - 1) * ncol(B)] <- A[i, j] * B
        }
      }
      m
    }

    #' getHPI
    #' @title Generating Host-Pathogen Protein-Protein Interaction (HP-PPI)
    #' Descriptors
    #' @param pathogenData The pathogen descriptor matrix.
    #' @param hostData The host descriptor matrix.
    #' @param type The interaction type, one or two of
    #' \code{"combine"} and \code{"kron.prod"}.
    #' @return A matrix containing the Host-Pathogen Protein-Protein Interaction
    #' (HP-PPI) descriptors.
    #' @author Matineh Rahmatbakhsh \email{matinerb.94@gmail.com}
    #' @description This function calculates Host-Pathogen Protein-Protein
    #' Interaction (HP-PPI) descriptors via two approaches
    #' \itemize{
    #' \item \code{combine} - combine the two descriptor matrix,
    #' result has \code{(p1 + p2)} columns
    #' \item \code{kron.prod} - if A has m x n matrix and B is q x p matrix,
    #' then the Kronecker product is the code{(pm × qn)} block matrix
    #' }
    #' @export
    #' @examples
    #' x <- matrix(c(1, 2, 3, 1), nrow = 2, ncol = 2, byrow = TRUE)
    #' y <- matrix(c(0, 3, 2, 1), nrow = 2, ncol = 2, byrow = TRUE)
    #' getHPI(x, y, "combine")
    #' getHPI(x, y, "kron.prod")
    getHPI <- function(pathogenData, hostData, type = c("combine", "kron.prod"))
      {
      if (!is.matrix(pathogenData)) pathogenData <- as.matrix(pathogenData)
      if (!is.matrix(hostData)) hostData <- as.matrix(hostData)

      pathogenRow <- nrow(pathogenData)
      hostRow <- nrow(hostData)

      if (pathogenRow != hostRow) stop("Matrix row count must match")
      if (missing(type)) stop("Must provide at least one interaction type")


      if (type == "combine") {
        result <- cbind(pathogenData, hostData)
      } else if (type == "kron.prod") {
        result <- .getKronecker(pathogenData, hostData)
      } else {
        stop("Interaction type must be  kron or combine")
      }


      return(result)
    }

test_getHPI = function() {
  x <- matrix(c(1, 2, 3, 1), nrow = 2, ncol = 2, byrow = TRUE)
  y <- matrix(c(0, 3, 2, 1), nrow = 2, ncol = 2, byrow = TRUE)
  RUnit::checkEquals(ncol(getHPI(x, y, 'combine')), 4L)
  RUnit::checkEquals(ncol(getHPI(x, y, 'kron.prod')), 4L)

}

## ----echo=FALSE,message=FALSE, warning=FALSE----------------------------------
library("knitr")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----echo=FALSE,message=FALSE, warning=FALSE----------------------------------
suppressPackageStartupMessages({
  library("readr")
  library("dplyr")
  library("Biostrings")
  library("SummarizedExperiment")
  library("stringr")
})

## -----------------------------------------------------------------------------
# Loading packages required for data handling & data manipulation
library(dplyr)
library(tibble)
library(stringr)
# Loading HPiP package
library(HPiP)
# Loading data 
data(Gold_ReferenceSet)
dim(Gold_ReferenceSet)

## ----eval=FALSE---------------------------------------------------------------
# local = tempdir()
# #Get positive interactions from BioGrid
# TP <- get_positivePPI(organism.taxID = 2697049,
#                       access.key = 'XXXX',
#                             filename = "PositiveInt.RData",
#                             path = local)

## ----echo=FALSE, warning=FALSE, message=FALSE, results='hide'-----------------
TP <- read_csv(
          system.file("extdata/TP_set.csv", package = "HPiP"),
          show_col_types = FALSE
        )

## -----------------------------------------------------------------------------
#pathogen proteins
prot1 <- unique(TP$`Official Symbol Interactor A`)
#host proteins
prot2 <- unique(TP$`Official Symbol Interactor B`)
#true positive PPIs 
TPset <- TP$PPI
TN <- get_negativePPI(prot1 , prot2, TPset)
dim(TN)

## -----------------------------------------------------------------------------
local = tempdir()
#retrieve FASTA sequences of SARS-CoV-2 virus 
id = unique(Gold_ReferenceSet$Pathogen_Protein)
fasta_list <- getFASTA(id, filename = 'FASTA.RData', path = local)

## ----results='hide'-----------------------------------------------------------
# Convert the list of sequences obtained in the previous section to data.frame 
fasta_df <- do.call(rbind, fasta_list) 
fasta_df <- data.frame(UniprotID = row.names(fasta_df), 
                       sequence = as.character(fasta_df))

#calculate AAC
acc_df <- calculateAAC(fasta_df)
#only print out the result for the first row 
acc_df[1,-1] 

## ----echo= FALSE--------------------------------------------------------------
ex <- acc_df[1,-1] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
dc_df <- calculateDC(fasta_df)
#only print out the first 30 elements for the first row 
dc_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <- dc_df[1, c(2:31)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
tc_df <- calculateTC(fasta_df)
#only print out the first 30 elements for the first row 
tc_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <-tc_df[1, c(2:31)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
TC_Sm_df <- calculateTC_Sm(fasta_df)
#only print out the first 30 elements for the first row 
TC_Sm_df[1, c(2:31)] 


## ----echo= FALSE--------------------------------------------------------------
ex <- TC_Sm_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
QD_df <- calculateQD_Sm(fasta_df)
#only print out the first 30 elements for the first row 
QD_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <- QD_df[1, c(2:31)]
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
F1_df <- calculateF(fasta_df, type = "F1")
#only print out the result the first row 
F1_df[1,-1] 

## ----echo= FALSE--------------------------------------------------------------
ex <- F1_df[1,-1] 
#convert df to character vector 
x_df <- structure(as.numeric(ex), names = colnames(ex)) 
x_df

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
F2_df <- calculateF(fasta_df, type = "F2")
#only print out the result the first row 
F2_df[1,-1] 

## ----echo= FALSE--------------------------------------------------------------
ex <- F2_df[1,-1] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----echo=FALSE---------------------------------------------------------------
df1 <- HPiP:::df1
df1[is.na(df1)] <- ""
knitr::kable(df1, align = "lccrr", caption = "Amino acid attributes and the division of amino acid into three-group.", longtable = TRUE)

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
CTDC_df <- calculateCTDC(fasta_df)
CTDC_df[1, c(-1)] 


## ----echo= FALSE--------------------------------------------------------------
ex <- CTDC_df[1, c(-1)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
CTDT_df <- calculateCTDT(fasta_df)
#only print out the result for the first row 
CTDT_df[1, -1] 


## ----echo= FALSE--------------------------------------------------------------
ex <- CTDT_df[1, -1] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
CTDD_df <- calculateCTDD(fasta_df)
#only print out the first 30 elements for the first row 
CTDD_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <- CTDD_df[1, c(2:31)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
CTriad_df <- calculateCTriad(fasta_df)
#only print out the first 30 elements for the first row 
CTriad_df[1, c(2:31)] 


## ----echo= FALSE--------------------------------------------------------------
ex <- CTriad_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----warning=FALSE, results='hide'--------------------------------------------
# using data.frame provided by getFASTA function as data input
moran_df <- calculateAutocor(fasta_df,type = "moran", nlag = 10)
#only print out the first 30 elements for the first row 
moran_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <- moran_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
mb_df <- calculateAutocor(fasta_df,type = "moreaubroto", nlag = 10)
#only print out the first 30 elements for the first row 
mb_df[1, c(2:31)] 


## ----echo= FALSE--------------------------------------------------------------
ex <- mb_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
geary_df <- calculateAutocor(fasta_df,type = "geary", nlag = 10)
#only print out the first 30 elements for the first row 
geary_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <- geary_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----echo=FALSE---------------------------------------------------------------
df2 <- HPiP:::df2
df2[is.na(df2)] <- ""
knitr::kable(df2, caption = "Composition of k-spaced amino acid pairs. Given 400 (20 × 20) amino acid pairs and four values for k (k=1–4), there are 1600 attributes generated for the KSAAP feature.", 
col.names = c("","","","","","","",""),row.names = NA,
             longtable = TRUE, align = "lccrr")

## ----results='hide'-----------------------------------------------------------
# using data.frame provided by getFASTA function as data input
KSAAP_df <- calculateKSAAP(fasta_df)
#only print out the first 30 elements for the first row 
KSAAP_df[1, c(2:31)] 

## ----echo= FALSE--------------------------------------------------------------
ex <- KSAAP_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex

## ----results='hide', warning=FALSE, eval = FALSE------------------------------
# # using data.frame provided by getFASTA function as data input
# BE_df <- calculateBE(fasta_df)
# #only print out the first 30 elements for the first row
# BE_df[1, c(2:31)]

## ----echo= FALSE, eval = FALSE------------------------------------------------
# ex <- BE_df[1, c(2:31)]
# #convert df to character vector
# ex <- structure(as.numeric(ex), names = colnames(ex))
# ex

## -----------------------------------------------------------------------------
#loading the package 
library(Biostrings)

#Read fasta sequences provided by HPiP package using Biostrings
fasta <- 
  readAAStringSet(system.file("extdata/UP000464024.fasta", package="HPiP"),
                  use.names=TRUE)
#Convert to df
fasta_bios = data.frame(ID=names(fasta),sequences=as.character(fasta))
#Extract the UniProt identifier
fasta_bios$ID <- sub(".*[|]([^.]+)[|].*", "\\1", fasta_bios$ID)
# for example, run ACC
acc_bios <- calculateAAC(fasta_bios)

## -----------------------------------------------------------------------------
#loading viral_se object
data(viral_se)
viral_se

## -----------------------------------------------------------------------------
#loading host_se object
data(host_se)
host_se

## -----------------------------------------------------------------------------
#generate descriptors
CTDC_df <- calculateCTDC(fasta_df)
CTDC_m <- as.matrix(CTDC_df[, -1])
row.names(CTDC_m) <- CTDC_df$identifier

CTDT_df <- calculateCTDT(fasta_df)
CTDT_m <- as.matrix(CTDT_df[, -1])
row.names(CTDT_m) <- CTDT_df$identifier

CTDD_df <- calculateCTDD(fasta_df)
CTDD_m <- as.matrix(CTDD_df[, -1])
row.names(CTDD_m) <- CTDD_df$identifier

## -----------------------------------------------------------------------------
#convert matrix to se object
ctdc_se <- SummarizedExperiment(assays = list(counts = CTDC_m),
                                colData = paste0(colnames(CTDC_df[,-1]),
                                                 "CTDC"))
ctdt_se <- SummarizedExperiment(assays = list(counts = CTDT_m),
                                colData = paste0(colnames(CTDT_df[,-1]),
                                                 "CTDT"))
ctdd_se <- SummarizedExperiment(assays = list(counts = CTDD_m),
                                colData = paste0(colnames(CTDD_df[,-1]),
                                                 "CTDD"))
#combine all se objects to one 
viral_se <- cbind(ctdc_se,ctdd_se,ctdt_se)

## ----echo=FALSE---------------------------------------------------------------
df3 <- HPiP:::df3
df3[is.na(df3)] <- ""
knitr::kable(df3, caption = "List of commonly used descriptors in HPiP.",
             align = "lccrr")


## -----------------------------------------------------------------------------
#extract features from viral_se
counts_v <- assays(viral_se)$counts
#extract row.names from viral_Se
rnames_v <- row.names(counts_v)

## -----------------------------------------------------------------------------
#extract features from host_se
counts_h <- assays(host_se)$counts
#extract row.names from viral_Se
rnames_h <- row.names(counts_h)

## -----------------------------------------------------------------------------
#Loading gold-standard data
gd <- Gold_ReferenceSet

x1_viral <- matrix(NA, nrow = nrow(gd), ncol = ncol(counts_v))
for (i in 1:nrow(gd)) 
  x1_viral[i, ] <- counts_v[which(gd$Pathogen_Protein[i] == rnames_v), ]

x1_host <- matrix(NA, nrow = nrow(gd), ncol = ncol(counts_h))
for (i in 1:nrow(gd)) 
  x1_host[i, ] <- counts_h[which(gd$Host_Protein[i] == rnames_h), ]

## -----------------------------------------------------------------------------
x <- getHPI(x1_viral,x1_host, type = "combine")
x <- as.data.frame(x)
x <- cbind(gd$PPI, gd$class, x)
colnames(x)[1:2] <- c("PPI", "class")

## ----message=FALSE------------------------------------------------------------
#to use correlation analysis, make sure to drop the columns with sd zero
xx <- Filter(function(x) sd(x) != 0, x[,-c(1,2)])
xx <- cbind(x$PPI, x$class, xx)
colnames(xx)[1:2] <- c("PPI", "class")

#perform feature selection using both correlation analysis and RFE approach
set.seed(101)
x_FS <- FSmethod(xx, type = c("both"),
                 cor.cutoff = 0.8,resampling.method = "cv",
                 iter = 2,repeats =NULL, metric = "Accuracy", 
                 verbose = FALSE)

## ----message=FALSE,fig.width = 8, fig.height = 8------------------------------
#cor plot
corr_plot(x_FS$cor.result$corProfile, method = 'square' , cex = 0.1)

## ----fig.width = 8, fig.height = 4--------------------------------------------
#var importance
var_imp(x_FS$rf.result$rfProfile, cex.x = 8, cex.y = 8)

## -----------------------------------------------------------------------------
#load the unlabeled HP-PPIs
data('unlabel_data')
#Constructed labeled HP-PPIs
labeled_dat <- x_FS$rf.result$rfdf
labeled_dat <- labeled_dat[,-1] 
#select important features
unlabel_data <- 
  unlabel_data[names(unlabel_data) %in% names(x_FS$rf.result$rfdf)]

#merge them 
ind_data <- rbind(unlabel_data,labeled_dat)

## -----------------------------------------------------------------------------
# Get class labels
gd <-  x_FS$rf.result$rfdf
gd <-  gd[, c(2,1)]

## ----warning=FALSE,results='hide', message=FALSE------------------------------
set.seed(102)
ppi <- pred_ensembel(ind_data,
                     gd,
                     classifier = c("svmRadial", "glm", "ranger"),
                     resampling.method = "cv",
                     ncross = 5,
                     verboseIter = TRUE,
                     plots = FALSE,
                     filename=file.path(tempdir(), "plots.pdf"))

## -----------------------------------------------------------------------------
pred_interactions <- ppi[["predicted_interactions"]]
head(pred_interactions)

## -----------------------------------------------------------------------------
pred_interactions <- filter(pred_interactions, ensemble >= 0.7)
dim(pred_interactions)

## ----message=FALSE, results='hide',fig.width = 6, fig.height = 6--------------
S_interc <- filter(pred_interactions, 
                           str_detect(Pathogen_protein, "^ORF8:"))
#drop the first column
ppi <- S_interc[,-1]

plotPPI(ppi, edge.name = "ensemble",
            node.color ="red",
            edge.color = "grey",
            cex.node = 10,
            node.label.dist= 2)

## ----fig.width = 10, fig.height = 5-------------------------------------------
ppi <- pred_interactions[,-1]
FreqInteractors(ppi,cex.size = 12)

## ----warning=FALSE, message=FALSE, results='hide', eval=FALSE-----------------
# enrich_result <-
#   enrichfindP(ppi,threshold = 0.05,
#             sources = "GO",
#             p.corrction.method = "bonferroni",
#             org = "hsapiens")

## ----message=FALSE------------------------------------------------------------
ppi <- pred_interactions[,-1]
set.seed(103)
predcpx <- run_clustering(ppi, method = "FC")

## ----message=FALSE------------------------------------------------------------
enrichcpx <- enrichfind_cpx(predcpx,
             threshold = 0.05,
             sources = "GO",
             p.corrction.method = "bonferroni",
             org = "hsapiens")

## -----------------------------------------------------------------------------
sessionInfo()

    #' impute_missing_data
    #' @title Impute missing Values per Features (i.e., Columns)
    #' @param x A numeric matrix as input.
    #' @param method Imputation method for missing values
    #' (mean, median or zero).
    #' @return Imputed matrix.
    #' @author Matineh Rahmatbakhsh \email{matinerb.94@gmail.com}
    #' @importFrom stats median
    #' @description Given an input matrix, impute the missing values
    #' via three approaches including mean, median or zero.
    #' @export
    #' @examples
    #' x <- matrix(1:10, ncol = 2)
    #' x[1:3, 2] <- NA
    #' row.names(x) <- c("A", "B", "C", "D", "E")
    #' colnames(x) <- c("col1", "col2")
    #' impute_missing_data(x, method = "mean")
    #' impute_missing_data(x, method = "median")
    #' impute_missing_data(x, method = "zero")
    impute_missing_data <-
      function(x, method = c("mean", "median", "zero")) {
        if (!is.matrix(x)) x <- as.matrix(x)


        if (length(rownames(x)) == 0) {
          stop("Row names are missing...")
        }
        if (length(colnames(x)) == 0) {
          stop("Column names are missing...")
        }


        impute.f <- function(vec, method = c("mean", "median", "zero")) {
          if (missing(method)) {
            stop("Must provide at least one imputation approach")
          }

          if (!method %in% c("mean", "median", "zero")) {
            stop("Imputation method must be one of mean or median...")
          }

          if (!is.numeric(vec)) vec <- as.numeric(vec)

          if (method == "mean") {
            vec[is.na(vec)] <- mean(vec, na.rm = TRUE)
          }
          if (method == "median") {
            vec[is.na(vec)] <- median(vec, na.rm = TRUE)
          }
          if (method == "zero") {
            vec[is.na(vec)] <- 0
          }
          return(vec)
        }
        names <- rownames(x)
        x.imputed <-
          apply(x, 2, impute.f, method = method)
        row.names(x.imputed) <- names
        return(x.imputed)
      }

test_enrich.df <- function() {
  data('predicted_PPIs')
  enrich.df <- enrichfindP(predicted_PPIs,
                           threshold = 0.05,
                           sources = c("GO", "KEGG"),
                           p.corrction.method = "bonferroni",
                           org = "hsapiens")

  checkTrue(is.data.frame(enrich.df) == TRUE)
}

    #' calculateCTDD
    #' @title Calculate CTD Descriptors - Distribution (D)
    #' @param x A data.frame containing gene/protein names and their fasta
    #' sequences.
    #' @return A length 105 named vector for the data input.
    #' @seealso See \code{\link{calculateCTDC}} and \code{\link{calculateCTDT}}
    #' for Composition and Transition descriptors.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom dplyr rename
    #' @importFrom dplyr arrange
    #' @importFrom dplyr full_join
    #' @importFrom dplyr row_number
    #' @importFrom dplyr across
    #' @importFrom dplyr everything
    #' @description This function calculates Distribution (D) descriptor
    #' for data input.
    #' @export
    #' @references
    #' Dubchak, I., Muchnik, I., Holbrook, S. R., and Kim, S.-H. (1995).
    #' Prediction of protein folding class using global description of amino
    #' acid sequence.\emph{Proc. Natl. Acad. Sci.} 92, 8700–8704.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateCTDD(UP000464024_df)
    #' head(x_df, n = 1L)
    calculateCTDD <- function(x) {
      . <- NULL
      Freq <- NULL
      ID <- NULL
      V1 <- NULL
      V2 <- NULL
      V3 <- NULL
      V4 <- NULL
      V5 <- NULL
      identifier <- NULL
      category <- NULL
      d <- NULL
      key <- NULL
      value <- NULL

      if (!is.data.frame(x)) {
        stop("Input data must be data.frame")
      }

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }


      # three-group classification of the 20 amino acids by each attribute
      # retrieved from "https://CRAN.R-project.org/package=protr"

      group1 <- list(
        "hydrophobicity" = c("R", "K", "E", "D", "Q", "N"),
        "normwaalsvolume" = c("G", "A", "S", "T", "P", "D", "C"),
        "polarity" = c("L", "I", "F", "W", "C", "M", "V", "Y"),
        "polarizability" = c("G", "A", "S", "D", "T"),
        "charge" = c("K", "R"),
        "secondarystruct" = c("E", "A", "L", "M", "Q", "K", "R", "H"),
        "solventaccess" = c("A", "L", "F", "C", "G", "I", "V", "W")
      )

      group2 <- list(
        "hydrophobicity" = c("G", "A", "S", "T", "P", "H", "Y"),
        "normwaalsvolume" = c("N", "V", "E", "Q", "I", "L"),
        "polarity" = c("P", "A", "T", "G", "S"),
        "polarizability" = c("C", "P", "N", "V", "E", "Q", "I", "L"),
        "charge" = c(
          "A", "N", "C", "Q", "G", "H", "I", "L",
          "M", "F", "P", "S", "T", "W", "Y", "V"
        ),
        "secondarystruct" = c("V", "I", "Y", "C", "W", "F", "T"),
        "solventaccess" = c("R", "K", "Q", "E", "N", "D")
      )

      group3 <- list(
        "hydrophobicity" = c("C", "L", "V", "I", "M", "F", "W"),
        "normwaalsvolume" = c("M", "H", "K", "F", "R", "Y", "W"),
        "polarity" = c("H", "Q", "R", "K", "N", "E", "D"),
        "polarizability" = c("K", "M", "H", "F", "R", "Y", "W"),
        "charge" = c("D", "E"),
        "secondarystruct" = c("G", "N", "P", "S", "D"),
        "solventaccess" = c("M", "S", "P", "T", "H", "Y")
      )


      nchar_df <- .nchar(fastalist)

      fastaSplitted <-
        fastalist %>%
        lapply(., function(x) strsplit(x[[1]], ""))


      g1 <- .gintersect(group1, fastaSplitted, gn = "G1")
      g2 <- .gintersect(group2, fastaSplitted, gn = "G2")
      g3 <- .gintersect(group3, fastaSplitted, gn = "G3")
      list.g1g2g3 <- .mapL(g1, g2, g3)


      D <- vector("list", length(list.g1g2g3))
      names(D) <-
        names(list.g1g2g3)
      for (i in seq_along(list.g1g2g3)) D[[i]] <- matrix(ncol = 5, nrow = 3)

      for (i in seq_along(list.g1g2g3)) {
        for (j in seq_len(3)) {
          inds <- which(list.g1g2g3[[i]] == paste0("G", j))
          quartiles <- floor(length(inds) * c(0.25, 0.5, 0.75))
          quartiles[which(quartiles <= 0)] <- 1
          D[[i]][j, ] <- if (length(inds) > 0) {
            (inds[c(1, quartiles, length(inds))]) * 100
          } else {
            0
          }
        }
      }
      tbl2 <- do.call(rbind, D)

      names.tbl2 <-
        as.data.frame(matrix(unlist(names(D)), ncol = 1, byrow = TRUE))

      names.tbl3 <-
        names.tbl2 %>%
        mutate(g1 = "g1") %>%
        mutate(g2 = "g2") %>%
        mutate(g3 = "g3") %>%
        gather("key", "value", -V1) %>%
        mutate(names = paste(V1, value, sep = ".")) %>%
        separate(V1, c("group", "identifier"), sep = "\\.") %>%
        select(1, 2, 5)


      # sort based on the attribute
      d1 <-
        names(fastalist)
      Final_df <-
        names.tbl3[order(match(names.tbl3$identifier, d1)), ]
      d2 <- c(
        "hydrophobicity", "normwaalsvolume", "polarity", "polarizability",
        "charge", "secondarystruct", "solventaccess"
      )
      Final_df <-
        Final_df[order(match(Final_df$group, d2)), ]




      row.names(tbl2) <-
        Final_df$names


      dfOut <-
        tbl2 %>%
        as.data.frame(.) %>%
        rownames_to_column("ID") %>%
        separate(ID, c("class", "identifier", "g"), "\\.", remove = FALSE) %>%
        rename(
          residue0 = V1, residue25 = V2, residue50 = V3, residue75 = V4,
          residue100 = V5
        ) %>%
        select(-c(2, 4)) %>%
        gather("key", "value", 3:7) %>%
        left_join(., nchar_df, by = "identifier") %>%
        mutate(Freq = value / nchar) %>%
        separate(ID, c("c", "p", "d"), sep = "\\.") %>%
        mutate(category = paste(c, d, key, sep = ".")) %>%
        arrange(d) %>%
        select(4, 9, 8) %>%
        group_by(identifier) %>%
        mutate(row = row_number()) %>%
        spread(category, Freq) %>%
        select(-row) %>%
        group_by(identifier) %>%
        replace(is.na(.), 0) %>%
        summarise(across(everything(), ~ max(.)))


      return(dfOut)
    }

    #' run_clustering
    #' @title Module Detection
    #' @param ppi A data.frame containing pathogen proteins in the
    #' first column,host proteins in the second column, and edge weight in the
    #' third column.
    #' @param method Module detection algorithms including:
    #' \itemize{
    #' \item \code{FC} - fast-greedy algorithm.
    #' \item \code{RW} - walktrap algorithm.
    #' \item \code{ML} - multi-level community algorithm.
    #' \item \code{clp} - label propagation algorithm.
    #' \item \code{MCL} - markov clustering.
    #' }
    #' @param expan Numeric value > 1 for the expansion parameter.
    #' See \code{\link[MCL]{mcl}} for more details.
    #' @param infla Numeric value > 0 for the inflation power coefficient.
    #' See \code{\link[MCL]{mcl}} for more details.
    #' @param iter An integer, the maximum number of iterations for the MCL.
    #' See \code{\link[MCL]{mcl}} for more details.
    #' @return A data.frame with the enrichment analysis results.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom igraph get.adjacency
    #' @importFrom igraph graph.adjacency
    #' @importFrom igraph fastgreedy.community
    #' @importFrom igraph cluster_walktrap
    #' @importFrom igraph cluster_louvain
    #' @importFrom igraph cluster_label_prop
    #' @importFrom igraph simplify
    #' @importFrom igraph membership
    #' @importFrom igraph E
    #' @importFrom MCL mcl
    #' @description This function contains five module detection algorithms
    #' including fast-greedy algorithm
    #' (\code{FC}), walktrap algorithm (\code{RW}), multi-level community
    #' algorithm
    #' (\code{ML}), label propagation algorithm (\code{clp}), and markov
    #' clustering (\code{MCL}).
    #' @export

    run_clustering <- function(ppi, method = c("FC",
                                               "RW",
                                               "ML",
                                               "clp",
                                               "MCL"),
                               expan = 2,
                               infla = 5,
                               iter = 50) {

        method <- match.arg(method)

        attr <- colnames(ppi[,3])
        #create adjacency matrix
        m <-
            adjacency_matrix <- #create adjacency matrix
            get.adjacency(
                graph_from_data_frame(ppi, directed = FALSE),
                attr = attr,
                sparse = FALSE
            )
        #igraph
        g <-
            graph.adjacency(m,
                            mode = "undirected", weighted = TRUE, diag = TRUE)
        graph<-
            simplify(g)

        if(method == "FC") {
            fc <-
                fastgreedy.community(graph,merges=TRUE,modularity=TRUE)
            mem<-
                igraph::membership(fc)
        }else if(method == "RW") {
        rw <-
            cluster_walktrap(graph,weights = E(graph)$weight,
                             merges=TRUE,modularity=TRUE)
        mem<-
            igraph::membership(rw)


        }else if(method == "ML") {
        ml <-
            cluster_louvain(graph,weights = E(graph)$weight)
        mem<-
            igraph::membership(ml)

        }else if(method == "clp") {
        clp <-
            cluster_label_prop(graph, weights = E(graph)$weight)
        mem<-
            igraph::membership(clp)

        } else if(method == "MCL") {

        mcl.clust <-
            mcl(m, addLoops = TRUE,
                expansion = expan,
                inflation = infla,
                allow1 = FALSE,
                max.iter = iter, ESM = FALSE)
        mem<-
            mcl.clust$Cluster
        names(mem) <- colnames(m)

    }
        dfout <-
            cbind(names(mem),mem)
        colnames(dfout)[1] <-
            "member"
        return(dfout)
    }

    #' enrichfind_hp
    #' @title Functional Enrichment Analysis of all Host Proteins
    #' @param ppi A data.frame containing pathogen proteins in the
    #' first column and host proteins in the second column.
    #' @param threshold Custom p-value threshold for significance.
    #' @param sources A vector of data sources to use.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @param p.corrction.method The algorithm used for multiple testing
    #' correction;defaults to 'bonferroni'.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @param org An organism name;defaults to 'hsapiens'.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @return A data.frame with the enrichment analysis results.
    #' @seealso See \code{\link{enrichplot}} for plotting enrichment analysis.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description This function uses \code{\link[gprofiler2]{gost}} function
    #' in \code{gprofiler2} package to perfrom functional enrichment analysis
    #' for all predicted host proteins in the high-confidence network.
    #' @export
    #' @examples
    #' data('predicted_PPIs')
    #' #perform enrichment
    #' enrich.df <- enrichfind_hp(predicted_PPIs,
    #' threshold = 0.05,
    #' sources = c("GO", "KEGG"),
    #' p.corrction.method = "bonferroni",
    #' org = "hsapiens")
    
    
    enrichfind_hp <-
        function(ppi, threshold = 0.05,
                 sources = c("GO", "KEGG"),
                 p.corrction.method = "bonferroni",
                 org = "hsapiens") {
            p_value <- NULL
            ppi <- as.data.frame(ppi)
            
            prot <- unique(ppi[,2])
    
            annotCov <- 
                gprofiler2::gost(prot,
                                 significant = TRUE,
                                 exclude_iea = TRUE,
                                 evcodes = TRUE,
                                 user_threshold = threshold,
                                 sources = sources,
                                 correction_method = p.corrction.method,
                                 organism = org
                )
            
            df <-
                annotCov[["result"]]
            ans <-
                arrange(df, p_value)
            
            return(ans)
        }

    #' calculateBE
    #' @title Tranform a Seqeuence into Binary Encoding (BE)
    #' @param x A data.frame containing gene/protein names and their
    #' fasta sequences.
    #' @return A length 400 named vector for the data input.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom tibble add_column
    #' @description This function transform each residue in a peptide
    #' into 20 coding values.
    #' @export
    #' @references
    #' Al-Barakati, H. J., Saigo, H., and Newman, R. H. (2019).
    #' RF-GlutarySite: a random forest based predictor for glutarylation sites.
    #' \emph{Mol. Omi.} 15, 189–204.
    calculateBE <- function(x) {
      . <- NULL
      V1 <- NULL


      if (!is.data.frame(x)) {
        stop("Input data must be data.frame")
      }

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }

      dict <- list(
        "A" = 1, "C" = 2, "D" = 3, "E" = 4, "F" = 5, "G" = 6, "H" = 7, "I" = 8,
        "K" = 9, "L" = 10, "M" = 11, "N" = 12, "P" = 13, "Q" = 14, "R" = 15,
        "S" = 16, "T" = 17, "V" = 18, "W" = 19, "Y" = 20
      )


      bin_m <- function(x) {
        fp <- matrix(0L, nrow = length(x), ncol = 400)
        vect <- c(
          "A", "C", "D", "E", "F", "G", "H", "I",
          "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y"
        )
        n <- rep(vect, 20)
        colnames(fp) <- n

        charSeq <- unlist(strsplit(x[[1]], split = ""))
        pos <- as.numeric(dict[unique(charSeq)])
        lenSeqs <- length(pos)
        rng <- (0:(lenSeqs[1] - 1)) * 20
        pos1 <- rng + pos
        for (i in 1) fp[i, pos1] <- 1L
        return(fp)
      }


      BM_list <- lapply(fastalist, function(x) bin_m(x[[1]]))


      # generate df from list
      tbl <- do.call(rbind, BM_list)
      rownames(tbl) <- names(BM_list)

      # ensure unique names (again, just in case) and convert to tibble safely
      colnames(tbl) <- make.unique(colnames(tbl), sep = "_")
      tbl <- tibble::as_tibble(tbl, .name_repair = "unique")
      
      # avoid duplicate 'identifier' then add it from rownames
      if ("identifier" %in% names(tbl)) tbl$identifier <- NULL
      tbl <- tibble::rownames_to_column(tbl, var = "identifier")



      return(tbl)
    }

    #' filter_missing_values
    #' @title Drop the Missing Values Above a Certain Threshold
    #' @param x A numeric matrix as input.
    #' @param max_miss_rate Maximal missing rate allowed for a feature;default
    #' is 20.
    #' @return A dataframe with features with missingness rate of more than
    #' user-defined threshold.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description Given an input matrix, compute the missingness rate for
    #' each features and keep only features with missing rate more than
    #' user-defined percentage.
    #' @export
    #' @examples
    #' x <- matrix(1:10, ncol = 2)
    #' x[, 2] <- NA
    #' filter_missing_values(x, 30)
    filter_missing_values <-
      function(x, max_miss_rate = 20) {
        if (is.matrix(x)) x <- as.data.frame(x)
        if (missing(max_miss_rate)) {
          stop("Must provide percentage value for features removal")
        }
        if (length(colnames(x)) == 0) {
          stop("Column (feature) names are missing...")
        }

        # remove column with more than 20% missing values
        miss_prec <-
          colSums(is.na(x)) / nrow(x) * 100
        print(miss_prec[miss_prec > 20])
        col_miss <-
          names(miss_prec[miss_prec > 20])

        if (length(col_miss) == 0) {
          x
        } else {

          # Removing the columns with more than 20% missing value
          x[, c(col_miss)] <- NULL
        }
        return(x)
      }

    #' plotPPI
    #' @title Plot the Predicted PPI
    #' @param ppi A data.frame containing protein-protein interactions with edge
    #' score.
    #' @param edge.name A character string giving an edge attribute name.
    #' @param node.color The fill color of the node.
    #' @param edge.color The color of the edge.
    #' @param cex.node The size of the node.
    #' @param node.label.dist The distance of the label from the center of the
    #' node.
    #' @return A PPI plot.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom igraph get.adjacency
    #' @importFrom graphics par
    #' @importFrom igraph graph_from_data_frame
    #' @description Plot the predicted PPIs. This function uses the plot
    #' function of the \code{igraph}.
    #' @rdname plotPPI
    #' @export
    #' @examples
    #' df <- data.frame(
    #'     node1 = c("A", "B", "C", "D", "E"),
    #'     node2 = c("C", "E", "E", "E", "A"),
    #'     edge.scores = c(0.5, 0.4, 0.3, 0.2, 0.7)
    #' )
    #' plotPPI(df, edge.name = "edge.scores")
    plotPPI <- function(ppi, edge.name = "ensemble_score",
                        node.color = "grey",
                        edge.color = "orange",
                        cex.node = 4,
                        node.label.dist = 1.5) {
      m <- adjacency_matrix <- # create adjacency matrix
        get.adjacency(
          graph_from_data_frame(ppi, directed = FALSE),
          attr = edge.name,
          sparse = FALSE
        )

      # igraph
      g <-
        igraph::graph.adjacency(m,
          mode = "undirected", weighted = TRUE, diag = TRUE
        )
      graph <-
        igraph::simplify(g)

      par(mar = c(0, 0, 0, 0))
      print(plot(graph,
        edge.color = edge.color, vertex.color = node.color,
        vertex.label.color = "black", vertex.label.dist = node.label.dist,
        vertex.size = cex.node
      ))
    }

    #' HP-PPIs with Unknown Class Labels
    #' @description This dataset consists of interactions between
    #' SARS-CoV-2 and human proteins, achieved by AP-MS
    #' (affinity purification mass spectrometry).
    #' @details
    #' To construct this dataset, data (supplementary table 1) containing
    #' SARS-CoV-2-human PPIs was retrieved from (Gordon et al., 2020) and 700
    #' pairs were randomly selected from total pairs, followed by converting
    #' protein sequences of host or viral proteins to numerical features and
    #' finally concatenating the computed features in order to construct
    #' host-pathogen PPIs.
    #' @name unlabel_data
    #' @usage data(unlabel_data)
    #' @docType data
    #' @format A data.frame containing 700 SARS-CoV-2-Human protein-protein
    #' interactions (PPIs) with pre-computed numerical features using
    #' CTD (composition/transition/distribution) descriptors.
    #' @source \url{https://www.nature.com/articles/s41586-020-2286-9#Sec36}
    #' @references
    #' Gordon,D.E. et al. (2020) A SARS-CoV-2 protein interaction map reveals
    #' targets for drug repurposing. Nature, 583, 459–468.
    NULL

    #' Viral SummarizedExperiment object
    #' @description SummarizedExperiment object of numerical features for
    #' SARS-CoV-2 proteins.
    #' @details
    #' To construct this object, first protein sequences were converted to
    #' numerical features using (CTD) descriptors provided in the HPiP package,
    #' followed by converting each numerical features matrix to
    #' SummarizedExperiment object.
    #' Each object is then merged into one object using `cbind()`.
    #' @name viral_se
    #' @usage data(viral_se)
    #' @docType data
    NULL

    .Moran_Autocor <- function(x, AADict, nlag, n.props, Pr) {
      fastaSplitted <- strsplit(x, split = "")[[1]]

      if (nchar(x) <= nlag) {
        warning("length of the amino acid sequence is <= nlag;
                    NAs will be generated")
      }

      list <- fastaSplitted
      L <- vector("list", n.props)
      for (i in seq_len(n.props)) L[[i]] <- list

      for (i in seq_len(n.props)) {
        for (j in AADict) {
          L[[i]][which(L[[i]] == j)] <- Pr[i, j]
        }
      }
      L <- lapply(L, as.numeric)

      Moran <- vector("list", n.props)
      N <- length(list)
      Pbar <- vapply(L, mean, FUN.VALUE = numeric(1))

      for (i in seq_len(n.props)) {
        for (j in seq_len(nlag)) {
          Moran[[i]][j] <- ifelse(
            N - j > 0,
            (N / (N - j)) * ((sum((L[[i]][seq_len(N - j)] - Pbar[i]) *
              (L[[i]][(seq_len(N - j)) + j] - Pbar[i]))) /
              (sum((L[[i]] - Pbar[i])^2))),
            NA
          )
          cMoran <- Moran
          cMoran <- unlist(cMoran)
        }
      }
      return(cMoran)
    }


    .MB_Autocor <- function(x, AADict, nlag, n.props, Pr) {
      fastaSplitted <- strsplit(x, split = "")[[1]]
      if (nchar(x) <= nlag) {
        warning("length of the amino acid sequence is <= nlag;
                    NAs will be generated")
      }

      list <- fastaSplitted
      L <- vector("list", n.props)
      for (i in seq_len(n.props)) L[[i]] <- list

      for (i in seq_len(n.props)) {
        for (j in AADict) {
          L[[i]][which(L[[i]] == j)] <- Pr[i, j]
        }
      }
      L <- lapply(L, as.numeric)

      MB <- vector("list", n.props)
      N <- length(list)

      for (i in seq_len(n.props)) {
        for (j in seq_len(nlag)) {
          MB[[i]][j] <- ifelse(
            N - j > 0,
            sum(L[[i]][seq_len(N - j)] * L[[i]][(seq_len(N - j)) + j]) /
              (N - j), NA
          )
          cMB <- MB
          cMB <- unlist(cMB)
        }
      }
      return(cMB)
    }


    .Geary_Autocor <- function(x, AADict, nlag, n.props, Pr) {
      fastaSplitted <- strsplit(x, split = "")[[1]]
      if (nchar(x) <= nlag) {
        warning("length of the amino acid sequence is <= nlag;
                    NAs will be generated")
      }


      list <- fastaSplitted
      L <- vector("list", n.props)
      for (i in seq_len(n.props)) L[[i]] <- list

      for (i in seq_len(n.props)) {
        for (j in AADict) {
          L[[i]][which(L[[i]] == j)] <- Pr[i, j]
        }
      }
      L <- lapply(L, as.numeric)

      Geary <- vector("list", n.props)
      N <- length(list)
      Pbar <- vapply(L, mean, FUN.VALUE = numeric(1))


      for (i in seq_len(n.props)) {
        for (j in seq_len(nlag)) {
          Geary[[i]][j] <- ifelse(
            N - j > 0,
            ((N - 1) / (2 * (N - j))) *
              ((sum((L[[i]][seq_len(N - j)] - L[[i]][(seq_len(N - j)) + j])^2))
              / (sum((L[[i]] - Pbar[i])^2))),
            NA
          )
          cGeary <- Geary
          cGeary <- unlist(cGeary)
        }
      }
      return(cGeary)
    }



    #' calculateAutocor
    #' @title Calculate Autocorrelation Descriptors
    #' @param x A data.frame containing gene/protein names and their
    #' fasta sequences.
    #' @param target.props A character vector, specifying the
    #' accession number of the target properties.
    #' 8 properties are used by default, as listed below:
    #' \describe{
    #' \item{AccNo. CIDH920105}{Normalized average hydrophobicity scales
    #'  (Cid et al., 1992)}
    #' \item{AccNo. BHAR880101}{Average flexibility indices
    #' (Bhaskaran-Ponnuswamy, 1988)}
    #' \item{AccNo. CHAM820101}{Polarizability parameter
    #' (Charton-Charton, 1982)}
    #' \item{AccNo. CHAM820102}{Free energy of solution in water, kcal/mole
    #' (Charton-Charton, 1982)}
    #' \item{AccNo. CHOC760101}{Residue accessible surface area in
    #'  tripeptide (Chothia, 1976)}
    #' \item{AccNo. BIGC670101}{Residue volume (Bigelow, 1967)}
    #' \item{AccNo. CHAM810101}{Steric parameter (Charton, 1981)}
    #' \item{AccNo. DAYM780201}{Relative mutability (Dayhoff et al., 1978b)}}
    #' @param nlag Maximum value of the lag parameter. Default is \code{30}.
    #' @return A length \code{nlag} named vector for data input.
    #' @param type The autocorrelation type:
    #' \code{moran}, \code{geary}, or \code{moreaubroto}.
    #'
    #' @author Matineh Rahmatbakhsh <\email{matinerb.94@gmail.com}>,
    #'         Nan Xiao
    #' @importFrom stats sd
    #' @importFrom utils read.csv
    #' @importFrom dplyr select
    #' @description This function calculates autocorrelation descriptors:
    #' \itemize{
    #' \item \code{moran} - moran autocorrelation,
    #' (Dim: \code{length(target.props) * nlag}).
    #' \item \code{geary} - geary autocorrelation,
    #' (Dim: \code{length(target.props) * nlag}).
    #' \item \code{moreaubroto} - moreau-broto autocorrelation,
    #' (Dim: \code{length(target.props) * nlag}).
    #' }
    #' @export
    #' @references
    #' AAindex: Amino acid index database.
    #' \url{http://www.genome.ad.jp/dbget/aaindex.html}
    #'
    #' Feng, Z.P. and Zhang, C.T. (2000)
    #' Prediction of membrane protein types based on the hydrophobic
    #' index of amino acids.
    #' \emph{Journal of Protein Chemistry}, 19, 269-275.
    #'
    #' Horne, D.S. (1988)
    #' Prediction of protein helix content from
    #' an autocorrelation analysis of sequence hydrophobicities.
    #' \emph{Biopolymers}, 27, 451-477.
    #'
    #' Sokal, R.R. and Thomson, B.A. (2006)
    #' Population structure inferred by local spatial autocorrelation:
    #' an Usage from an Amerindian tribal population.
    #' \emph{American Journal of Physical Anthropology}, 129, 121-131.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateAutocor(UP000464024_df,type = 'moran')
    #' head(x_df, n = 2L)




    calculateAutocor <-
      function(x,
               target.props =
                 c(
                   "CIDH920105", "BHAR880101", "CHAM820101",
                   "CHAM820102", "CHOC760101", "BIGC670101",
                   "CHAM810101", "DAYM780201"
                 ),
               nlag = 30L, type = c("moran", "geary", "moreaubroto")) {
        . <- NULL
        Pr <- NULL
        V1 <- NULL
        n.props <- NULL


        if (!is.data.frame(x)) {
          stop("Input data must be data.frame")
        }

        # convert data frame to list
        fastalist <-
          as.list(unlist(x[, 2]))
        names(fastalist) <-
          unlist(x[, 1])

        # check if there is any unrecognized amino acid
        f <- .checkFASTA(fastalist)
        if (nrow(f) > 0) {
          stop("Fastalist has unrecognized amino acid type")
        }

        ## this has to change later
        AAidx <- read.csv(
          system.file("extdata/AAidx.csv", package = "HPiP"),
          header = TRUE
        )


        # Compute Pr for each type of property
        aaidx <-
          AAidx %>%
          magrittr::set_rownames(.$AccNo) %>%
          select(-1)

        n.props <-
          length(target.props)

        pmean <-
          rowMeans(aaidx[target.props, ])

        psd <-
          apply(aaidx[target.props, ], 1, sd) * sqrt((20 - 1) / 20)

        Pr <- data.frame(matrix(ncol = 20, nrow = n.props))
        for (i in seq_len(n.props)) {
          Pr[i, ] <- (aaidx[target.props[i], ] - pmean[i]) / psd[i]
        }


        AADict <- c(
          "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
          "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
        )
        names(Pr) <- AADict


        if (type == "moran") {

          # apply the function to multiple lists
          autocor_lists <-
            lapply(fastalist, function(x) {
              .Moran_Autocor(x[[1]],
                AADict = AADict,
                nlag = nlag,
                Pr = Pr, n.props = n.props
              )
            })
        }

        if (type == "geary") {

          # apply the function to multiple lists
          autocor_lists <-
            lapply(fastalist, function(x) {
              .Geary_Autocor(x[[1]],
                AADict = AADict,
                nlag = nlag,
                Pr = Pr, n.props = n.props
              )
            })
        }

        if (type == "moreaubroto") {

          # apply the function to multiple lists
          autocor_lists <-
            lapply(fastalist, function(x) {
              .MB_Autocor(x[[1]],
                AADict = AADict,
                nlag = nlag,
                Pr = Pr, n.props = n.props
              )
            })
        }

        # generate df from list
        tbl <- do.call(rbind, autocor_lists)

        # change the colnames
        colnames(tbl) <- as.vector(t(outer(target.props,
          paste(".lag", seq_len(nlag), sep = ""),
          paste,
          sep = ""
        )))

        tbl <-
          tbl %>%
          as.data.frame(.) %>%
          rownames_to_column("identifier")


        return(tbl)
      }

test_pred_ensembel <- function() {
  data('example_data')
  features <- example_data[, -2]
  gd <- example_data[, c(1,2)]
  gd <- na.omit(gd)
  ppi <-pred_ensembel(features,gd,
                      classifier = c("avNNet", "svmRadial", "ranger"),
                      resampling.method = "cv",ncross = 5,
                      verboseIter = FALSE,plots = FALSE,
                      filename = "plots.pdf")
  #extract predicted interactions
  pred_interaction <- ppi[["predicted_interactions"]]
  checkTrue(is.data.frame(pred_interaction) == TRUE)
}

    #' corr_plot
    #' @title Plot Correlation Matrix between Input Features
    #' @param cormat A correlation matrix.
    #' @param method The visualization method of correlation matrix;
    #' defaults to number.
    #' See \code{\link[corrplot]{corrplot}} for more details.
    #' @param cex The size of x/y axis label.
    #' @return A correlation plot.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}.
    #' @importFrom corrplot corrplot
    #' @description A graphical display of a correlation matrix.
    #' @export
    #' @examples
    #' data('example_data')
    #' x <- na.omit(example_data)
    #' #perform feature selection
    #' s <- FSmethod(x, type = 'both',
    #' cor.cutoff = 0.7, resampling.method = "repeatedcv",
    #' iter = 5, repeats = 3, metric = "ROC", verbose = TRUE)
    #' corr_plot(s$cor.result$corProfile, method = 'square' , cex = 0.5)
    corr_plot <- function(cormat, method = "number", cex = 0.9) {
      if (!is.matrix(cormat)) cormat <- as.matrix(cormat)

      # plotting corr matrix
      plot.cor <- corrplot(cormat,
        order = "hclust",
        tl.col = "black",
        method = method,
        tl.cex = cex,
        cl.cex = 1.0
      ) # Make plot
      return(invisible(plot.cor))
    }

    #' calculateCTriad
    #' @title Calculate Conjoint Triad Descriptor
    #' @param x A data.frame containing gene/protein names and their
    #' fasta sequences.
    #' @return A length 343  named vector for the data input.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description This function calculates Conjoint Triad descriptor
    #' for data input.
    #' @export
    #' @references
    #' Shen, J., Zhang, J., Luo, X., Zhu, W., Yu, K., Chen, K., et al.
    #' (2007). Predicting protein–protein interactions based only on
    #' sequences information. \emph{Proc. Natl. Acad. Sci.} 104, 4337–4341.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateCTriad(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateCTriad <- function(x) {
      if (!is.data.frame(x)) x <- is.data.frame(x)

      . <- NULL
      V1 <- NULL

      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }


      classes <- vector("list", 7)
      classes[[1]] <- c("A", "G", "V")
      classes[[2]] <- c("I", "L", "F", "P")
      classes[[3]] <- c("Y", "M", "T", "S")
      classes[[4]] <- c("H", "N", "Q", "W")
      classes[[5]] <- c("R", "K")
      classes[[6]] <- c("D", "E")
      classes[[7]] <- c("C")


      vspace <- expand.grid(seq_len(7), seq_len(7), seq_len(7))

      CTDict <- vector("list", 343)

      for (i in seq_len(343)) {
        tmp <- as.vector(outer(classes[[vspace[i, 1]]],
          classes[[vspace[i, 2]]], paste,
          sep = ""
        ))
        CTDict[[i]] <- as.vector(outer(tmp,
          classes[[vspace[i, 3]]], paste,
          sep = ""
        ))
      }
      names(CTDict) <- paste(seq_along(CTDict))



      # compute TC_Sm
      p <- function(x, CTDict) {
        f <-
          x %>%
          lapply(., function(x) strsplit(x[[1]], "")) %>%
          lapply(., function(x) {
            paste(paste(x[[1]][-c(length(x[[1]]), length(x[[1]]) - 1)],
              x[[1]][-c(1, length(x[[1]]))],
              sep = ""
            ), x[[1]][-c(1, 2)],
            sep = ""
            )
          })

        inter <- unlist(lapply(CTDict, function(X) {
          lapply(f, function(Y) {
            length(which(Y %in% X))
          })
        }), recursive = FALSE)

        s <-
          do.call(cbind, inter) %>%
          set_rownames(names(x))
        s <- t(apply(s, 1, function(x) (x - min(x)) / max(x)))
        return(s)
      }

      CT <- lapply(fastalist, function(x) p(x[[1]], CTDict))
      CT_df <-
        do.call(rbind, CT)

      CT_df <-
        CT_df %>%
        as.data.frame(.) %>%
        set_rownames(names(CT)) %>%
        rownames_to_column("identifier")

      return(CT_df)
    }

    #' calculateTC
    #' @title Calculate Tripeptide Composition (TC) Descriptor
    #' @param x A data.frame containing gene/protein names and their fasta
    #' sequences.
    #' @return A length 8,000  named vector for the data input.
    #' @seealso See \code{\link{calculateAAC}},\code{\link{calculateDC}} and
    #' \code{\link{calculateTC_Sm}} for
    #' Amino Acid Composition, Dipeptide Composition and Tripeptide
    #' Composition (TC) Descriptor from Biochemical Similarity Classes.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description This function calculates Tripeptide Composition (TC)
    #' descriptor for data input.
    #' @export
    #' @references
    #' Liao, B., Jiang, J.-B., Zeng, Q.-G., and Zhu, W. (2011).
    #' Predicting apoptosis protein subcellular location with
    #' PseAAC by incorporating tripeptide composition.
    #' \emph{Protein Pept. Lett.} 18, 1086–1092
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateTC(UP000464024_df)
    #' head(x_df, n = 2L)
    calculateTC <- function(x) {
      . <- NULL
      V1 <- NULL
      AAC <- NULL
      TCfreq <- NULL
      count_TC <- NULL
      TC <- NULL

      if (!is.data.frame(x)) x <- is.data.frame(x)


      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }


      vect1 <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )
      vect2 <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )
      vect3 <- c(
        "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
        "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
      )



      TClist <-
        apply(expand.grid(vect1, vect2, vect3), 1, paste, collapse = "")



      # calculate dipeptide composition descriptor
      TC_calc <-
        fastalist %>%
        lapply(., function(x) strsplit(x[[1]], "")) %>%
        lapply(., function(x) {
          summary(factor(paste(paste(x[[1]][-c(
            length(x[[1]]),
            length(x[[1]]) - 1
          )],
          x[[1]][-c(1, length(x[[1]]))],
          sep = ""
          ), x[[1]][-c(1, 2)],
          sep = ""
          ), levels = TClist), maxsum = 8000) / (lengths(x) - 2)
        })


      TC_calc_df <-
        bind_rows(TC_calc, .id = "identifier") %>%
        gather("TC", "TCfreq", 2:ncol(.)) %>%
        spread(., TC, TCfreq) %>%
        as.data.frame(.) %>%
        replace(is.na(.), 0)


      return(TC_calc_df)
    }

    #' Data.frame Containing SARS-CoV-2 FASTA Sequences
    #' @description This data includes one protein sequence per SARS-CoV-2 gene,
    #' retrieved directly from UniProt database using
    #' \code{\link[HPiP]{getFASTA}}.
    #' @name UP000464024_df
    #' @usage data(UP000464024_df)
    #' @docType data
    #' @format A data.frame with two columns:(1) UniprotKBID, UniProt
    #' identifier.(2) FASTASEQ, sequences per SARS-CoV-2 gene.
    #' @source \url{https://www.uniprot.org/uniprot/?query=proteome:UP000464024}
    NULL

    #' var_imp
    #' @title Variable Importance Plot
    #' @param x A list of elements returned from RFE analysis.
    #' See \code{\link[caret]{rfe}} for more details
    #' @param cex.x The size of x axis label.
    #' @param cex.y The size of y axis label.
    #' @return Variable Importance Plot.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}.
    #' @importFrom caret varImp
    #' @importFrom ggplot2 ggplot
    #' @importFrom ggplot2 aes
    #' @importFrom ggplot2 geom_point
    #' @importFrom ggplot2 labs
    #' @importFrom ggplot2 theme_classic
    #' @importFrom ggplot2 theme
    #' @importFrom ggplot2 geom_text
    #' @importFrom ggplot2 ggtitle
    #' @importFrom ggplot2 element_text
    #' @importFrom stats reorder
    #' @importFrom ggplot2 unit
    #' @description A graphical display of variable importance of selected
    #' features.
    #' @export
    #' @examples
    #' data('example_data')
    #' x <- na.omit(example_data)
    #' #perform feature selection
    #' s <- FSmethod(x, type = 'both',
    #' cor.cutoff = 0.7, resampling.method = "repeatedcv",
    #' iter = 5, repeats = 3, metric = "ROC", verbose = TRUE)
    #' var_imp(s$rf.result$rfProfile, cex.x = 10, cex.y = 10)
    var_imp <- function(x, cex.x = 1, cex.y = 2) {
      feature <- NULL
      importance <- NULL

      # examine variable importance for the selected features
      varimp_data <-
        data.frame(
          feature = row.names(varImp(x))[seq_len(length(predictors(x)))],
          importance = varImp(x)[seq_len(length(predictors(x))), 1]
        )

      ggplot(
        data = varimp_data,
        aes(x = reorder(feature, -importance), y = importance, color = feature)
      ) +
        geom_point(size = 2) +
        labs(x = "Features", y = "Variable Importance") +
        theme_classic() +
        theme(legend.position = "none") +
        geom_text(aes(label = round(importance, 2)),
          vjust = 1.6,
          color = "white", size = 4
        ) +
        ggtitle("Variable importance for the selected features") +
        theme(plot.title = element_text(
          color = "red", size = 14,
          face = "bold.italic"
        )) +
        theme(
          text = element_text(size = 12, color = "black"),
          axis.text.x = element_text(size = cex.x, colour = "black"),
          axis.text.y = element_text(size = cex.y, colour = "black")
        ) +
        theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1)) +
        theme(plot.margin = unit(c(1, 1, 1, 1), "cm")) +
        theme(axis.ticks.length = unit(.2, "cm"))
    }

    .MajVot <- function(x) {
      tabulatedOutcomes <- table(x)
      sortedOutcomes <- sort(tabulatedOutcomes, decreasing = TRUE)
      mostCommonLabel <- names(sortedOutcomes)[1]
      mostCommonLabel
    }

    #' pred_ensembel
    #' @title Predict Interactions via Ensemble Learning Method
    #' @param features A data frame with host-pathogen protein-protein
    #' interactions (HP-PPIs) in the first column, and features to be passed to
    #' the classifier in the remaining columns.
    #' @param gold_standard A data frame with gold_standard HP-PPIs and class
    #' label indicating if such PPIs are positive or negative.
    #' @param classifier The type of classifier to use. See \code{caret}
    #' for the availbale classifiers.
    #' @param resampling.method The resampling method:'boot', 'boot632',
    #' 'optimism_boot', boot_all', 'cv', 'repeatedcv', 'LOOCV', 'LGOCV';
    #' defaults to cv. See \code{\link[caret]{trainControl}} for more details.
    #' @param ncross  Number of partitions for cross-validation;
    #' defaults to 5.See \code{\link[caret]{trainControl}} for more details.
    #' @param repeats for repeated k-fold cross validation only;
    #'  defaults to 3.See \code{\link[caret]{rfeControl}}
    #'  for more details.
    #' @param verboseIter Logical value, indicating whether to check the status
    #' of training process;defaults to FALSE.
    #' @param plots Logical value, indicating whether to plot the performance
    #' of ensemble learning algorithm as compared to individual classifiers;
    #' defaults to TRUE.If the argument set to TRUE, plots will be saved in
    #' the current working directory.
    #' These plots are :
    #' \itemize{
    #' \item{pr_plot} - Precision-recall plot of ensemble classifier vs
    #' selected individual classifiers.
    #' \item{roc_plot} - ROC plot of ensemble classifier vs selected individual
    #' classifiers.
    #' \item{points_plot} - Plot accuracy, F1-score ,positive predictive value
    #' (PPV),sensitivity (SE), and Matthews correlation coefficient (MCC) of
    #' ensemble classifier vs selected individual classifiers.
    #' }
    #' @param filename A character string, indicating the output filename as an
    #' pdf object.
    #' @return
    #' Ensemble_training_output
    #' \itemize{
    #' \item{prediction score} - Prediction scores for whole dataset from each
    #' individual classifier.
    #' \item{Best} - Selected hyper parameters.
    #' \item{Parameter range} - Tested hyper parameters.
    #' \item{prediction_score_test} - Scores probabilities for test data from
    #' each individual classifier.
    #' \item{class_label} -  Class probabilities for test data from each
    #' individual classifier.
    #' }
    #' classifier_performance
    #' \itemize{
    #' \item{cm} - A confusion matrix.
    #' \item{ACC} - Accuracy.
    #' \item{SE} - Sensitivity.
    #' \item{SP} - Specificity.
    #' \item{PPV} - Positive Predictive Value.
    #' \item{F1} - F1-score.
    #' \item{MCC} - Matthews correlation coefficient.
    #' \item{Roc_Object} - A list of elements.
    #' See \code{\link[pROC]{roc}} for more details.
    #' \item{PR_Object} - A list of elements.
    #' See \code{\link[PRROC]{pr.curve}} for more details.
    #' }
    #' predicted_interactions - The input data frame of pairwise interactions,
    #' including classifier scores averaged across all models.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom caret trainControl
    #' @importFrom caret train
    #' @importFrom dplyr inner_join
    #' @importFrom pROC roc
    #' @importFrom stats predict
    #' @importFrom pROC plot.roc
    #' @importFrom graphics legend
    #' @importFrom PRROC pr.curve
    #' @importFrom pROC ggroc
    #' @importFrom ggplot2 geom_jitter
    #' @importFrom ggplot2 ylab
    #' @importFrom ggplot2 xlab
    #' @importFrom ggplot2 element_line
    #' @importFrom ggplot2 scale_y_continuous
    #' @importFrom ggplot2 geom_line
    #' @importFrom ggplot2 element_blank
    #' @importFrom ggplot2 annotation_custom
    #' @importFrom ggplot2 theme_bw
    #' @importFrom ggplot2 coord_equal
    #' @importFrom ggplot2 geom_abline
    #' @importFrom grDevices dev.off
    #' @importFrom grDevices pdf
    #' @description This function uses an ensemble of classifiers to
    #' predict interactions from the sequence-based dataset. This ensemble
    #' algorithm combines different results generated from individual
    #' classifiers within the ensemble via average to enhance prediction.
    #' @export
    #' @examples
    #' data('example_data')
    #' features <- example_data[, -2]
    #' gd <- example_data[, c(1,2)]
    #' gd <- na.omit(gd)
    #' ppi <-pred_ensembel(features,gd,
    #' classifier = c("avNNet", "svmRadial", "ranger"),
    #' resampling.method = "cv",ncross = 2,verboseIter = FALSE,plots = FALSE,
    #' filename = "plots.pdf")
    #' #extract predicted interactions
    #' pred_interaction <- ppi[["predicted_interactions"]]
    pred_ensembel <-
      function(features,
               gold_standard,
               classifier = c("avNNet", "svmRadial", "ranger"),
               resampling.method = "cv",
               ncross = 2,
               repeats = 2,
               verboseIter = TRUE,
               plots = FALSE,
               filename = "plots.pdf") {
        PPI <- NULL
        . <- NULL
        model <- NULL
        x <- NULL
        y <- NULL
        X1 <- NULL
        X2 <- NULL

        t.data <-
          features

        `%nin%` <- Negate(`%in%`)
        p.data <-
          features %>%
          filter(PPI %nin% gold_standard$PPI)


        if (!is.data.frame(features)) {
          stop("Input data should be data.frame")
        }
        if (all(colnames(features) != "PPI") == TRUE) {
          stop("PPI is missing from feature...")
        }
        if (length(colnames(features)) == 0) {
          stop("Column names are missing...")
        }

        if (all(colnames(gold_standard) != "class") == TRUE) {
          stop("class attribute is absent from the gold_standard data")
        }

        if (all(colnames(gold_standard) != "PPI") == TRUE) {
          stop("PPI is missing from gold_standard...")
        }
        if (!is.character(gold_standard$class)) {
          stop("Class column must include character vector")
        }


        # training data preparation

        x_train <-
          inner_join(t.data, gold_standard, by = "PPI") %>%
          select(-PPI)

        # change charactor to factor and reverse the factor level
        x_train$class <- # chanage charactor to factor
          as.factor(as.character(x_train$class))
        x_train$class <-
          factor(x_train$class, levels = rev(levels(x_train$class)))

        # check tghe factor level
        x <- unique(x_train$class)
        x <- data.frame(x, as.integer(x))
        x <-
          x %>%
          filter(x == "Positive") %>%
          .$as.integer.x.
        if (x == 2) {
          x_train$class <- factor(x_train$class,
                                  levels = rev(levels(x_train$class))
          )
        }

        # train control
        trControl <- trainControl(
          method = resampling.method,
          number = ncross, # number of resampling iterations
          repeats = repeats,
          verboseIter = verboseIter,
          classProbs = TRUE,
          savePredictions = "final",
          allowParallel = TRUE
        )

        Ensemble_output <-
          list()
        for (ml in classifier) {
          message(ml)
          models <-
            train(class ~ .,
                  data = x_train,
                  method = ml,
                  metric = "Accuracy",
                  trControl = trControl
            )

          clcol <- which(names(p.data) == "PPI")
          preds <- # predict on new data
            predict(models, newdata = p.data[, -clcol], type = "prob")
          Ensemble_output[[ml]][["prediction_score"]] <-
            preds[, "Positive"]

          # get the best parameter
          Ensemble_output[[ml]][["Best"]] <-
            models$bestTune
          # list of tested parameter
          Ensemble_output[[ml]][["Parameter range"]] <-
            models[["results"]]
          # prediction score on test set during cv
          Ensemble_output[[ml]][["prediction_score_test"]] <-
            models[["pred"]]$Positive

          # class label
          Ensemble_output[[ml]][["class_label"]] <-
            models[["pred"]]$obs
        }

        # get prediction interactions data
        Pred_interactions <-
          list()
        for (ml in classifier) {
          Pred_interactions[[ml]] <-
            Ensemble_output[[ml]][["prediction_score"]]
        }

        Pred_interactions <-
          do.call(cbind, Pred_interactions)
        ensemble <-
          rowMeans(Pred_interactions)
        datScore <-
          as.data.frame(cbind(p.data$PPI, ensemble))
        colnames(datScore)[1] <- "PPI"
        datScore$ensemble <- 
          as.numeric(as.character(datScore$ensemble))
        datScore <-
          separate(datScore, PPI, c(
            "Pathogen_protein",
            "Host_protein"
          ), sep = "~", remove = FALSE)

        ## class label output
        label_output <-
          list()
        for (ml in classifier) {
          label_output[[ml]] <-
            Ensemble_output[[ml]]$class_label
        }
        ss <-
          do.call(cbind, label_output)

        ss <- apply(ss, 2, function(x) {
          as.factor(ifelse(x == 2, "Negative", "Positive"))
        })

        ensemble <- apply(ss, 1, function(x) .MajVot(x))
        ss <- cbind(ss, ensemble)
        ss <- as.data.frame(ss)


        # classifier prediction output
        classifier_output <-
          list()
        for (ml in classifier) {
          classifier_output[[ml]] <-
            Ensemble_output[[ml]]$prediction_score_test
        }

        classifier_output <-
          do.call(cbind, classifier_output)

        ensemble <-
          rowMeans(classifier_output)

        df_perf <-
          cbind(classifier_output, ensemble)
        df_perf <- as.data.frame(df_perf)

        output <- list()
        for (i in names(df_perf)) {
          for (j in names(ss)) {
            df_perf.response <- df_perf
            df_perf.response[, i] <-
              as.factor(ifelse(df_perf.response[, i] > 0.5,
                               "Positive", "Negative"
              ))
            x <- unique(df_perf.response[, i])
            x <- data.frame(x, as.integer(x))
            x <-
              x %>%
              filter(x == "Positive") %>%
              .$as.integer.x.
            if (x == 2) {
              df_perf.response[, i] <-
                factor(df_perf.response[, i],
                       levels = rev(levels(df_perf.response[, i]))
                )
            }

            ss1 <- ss
            ss1[, j] <- as.factor(as.character(ss1[, j]))
            x <- unique(ss1[, j])
            x <- data.frame(x, as.integer(x))
            x <-
              x %>%
              filter(x == "Positive") %>%
              .$as.integer.x.
            if (x == 2) {
              ss1[, j] <- factor(ss1[, j],
                                 levels = rev(levels(ss1[, j]))
              )
            }
            cm <-
              table(df_perf.response[, i], ss1[, j])

            TP <- as.double(cm[1, 1])
            TN <- as.double(cm[2, 2])
            FP <- as.double(cm[1, 2])
            FN <- as.double(cm[2, 1])

            ACC <- (TP + TN) / (TP + TN + FP + FN)
            SE <- TP / (TP + FN)
            SP <- TN / (FP + TN)
            PPV <- TP / (TP + FP)
            F1 <- 2 * TP / (2 * TP + TP + FN)
            MCC <- (TP * TN - FP * FN) /
              sqrt((TP + FP) * (TP + FN) * (TN + FP) * (TN + FN))

            output[[i]][["cm"]] <- cm
            output[[i]][["ACC"]] <- ACC
            output[[i]][["SE"]] <- SE
            output[[i]][["SP"]] <- SP
            output[[i]][["PPV"]] <- PPV
            output[[i]][["F1"]] <- F1
            output[[i]][["MCC"]] <- MCC



            output[[i]][["Roc_Object"]] <- roc(ss1[, j], df_perf[, i])

            output[[i]][["PR_Object"]] <- pr.curve(
              scores.class0 =
                df_perf[, i][ss1[, j] == "Positive"],
              scores.class1 =
                df_perf[, i][ss1[, j] == "Negative"],
              curve = TRUE
            )
          }
        }


        if (plots) {

          ################## plot pr.curve curve

          pr.auc.list <- list()
          for (i in names(output)) {
            pr.auc.list[[i]] <- output[[i]][["PR_Object"]][["auc.integral"]]
          }
          pr.auc.list <-
            do.call(rbind, pr.auc.list) %>%
            as.data.frame(.) %>%
            rownames_to_column("model")
          colnames(pr.auc.list)[2] <- "auc"
          pr.auc.list$auc <- round(pr.auc.list$auc, 2)   # was $au
          


          mytheme <- gridExtra::ttheme_default(
            core = list(fg_params = list(cex = 0.6)),
            colhead = list(fg_params = list(cex = 0.6)),
            rowhead = list(fg_params = list(cex = 0.6))
          )


          PR_Object <- list()
          for (i in names(output)) {
            PR_Object[[i]] <- output[[i]][["PR_Object"]]$curve
          }

          PR_m <-
            do.call(rbind, lapply(PR_Object, data.frame)) %>%
            # as.data.frame(.) %>%
            rownames_to_column("model") %>%
            mutate(model = str_remove_all(model, "\\.\\d+"))

          for (i in seq_along(PR_m)) {
            PR_plot <-
              ggplot(PR_m, aes(x = X1, y = X2, color = model)) +
              geom_line() +
              theme(plot.title = element_text(hjust = 0.5)) +
              theme_bw() +
              scale_y_continuous(limits = c(0, 1)) +
              ggtitle("pr_plot") +
              theme(plot.title = element_text(hjust = 0.5)) +
              theme(
                axis.line = element_line(colour = "black"),
                panel.grid.major = element_blank(),
                panel.grid.minor = element_blank()
              ) +
              theme(axis.ticks.length = unit(.25, "cm")) +
              theme(legend.position = "top", legend.box = "horizontal") +
              theme(legend.title = element_blank()) +
              theme(text = element_text(size = 14, color = "black")) +
              theme(axis.text = element_text(size = 12, color = "black")) +
              xlab("Recall") +
              ylab("Percision") +
              annotation_custom(
                gridExtra::tableGrob(pr.auc.list, rows = NULL, theme = mytheme),
                xmin = 0.90, xmax = 0.90,   # was unit(0.9, "npc")
                ymin = 0.22, ymax = 0.22
              )
          }


          ######## plot the result from confusion matrix
          cm_output <- list()
          for (i in names(output)) {
            cm_output[[i]][["ACC"]] <- output[[i]][["ACC"]]
            cm_output[[i]][["SE"]] <- output[[i]][["SE"]]
            cm_output[[i]][["PPV"]] <- output[[i]][["PPV"]]
            cm_output[[i]][["F1"]] <- output[[i]][["F1"]]
            cm_output[[i]][["MCC"]] <- output[[i]][["MCC"]]
          }

          cm_output <-
            do.call(rbind, cm_output) %>%
            as.data.frame(.) %>%
            rownames_to_column("model")

          point_plot <-
            cm_output %>%
            gather(x, y, 2:6) %>%
            ggplot(aes(x = x, y = as.numeric(y), color = model)) +
            geom_jitter(width = 0.2, size = 2) +
            theme_bw() +
            ggtitle("points_plot") +
            theme(legend.position = "top", legend.box = "horizontal") +
            theme(plot.margin = unit(c(1, 1, 1, 1), "cm")) +
            labs(
              x = "Performance metrics", y = "value", title = "",
              colour = ""
            ) +
            theme(text = element_text(size = 14, color = "black")) +
            theme(axis.text = element_text(
              color = "black", # label color and size
              size = 12
            )) +
            theme(axis.ticks.length = unit(.25, "cm")) +
            theme(plot.title = element_text(hjust = 0.5))



          ################### plot ROC curve
          auc.list <- list()
          for (i in names(output)) {
            auc.list[[i]] <- output[[i]][["Roc_Object"]][["auc"]]
          }
          auc.list <-
            do.call(rbind, auc.list) %>%
            as.data.frame(.) %>%
            rownames_to_column("model")
          colnames(auc.list)[2] <- "auc"
          auc.list$auc <- round(auc.list$auc, 2)         # was $au
          


          Roc_Object <- list()
          for (i in names(output)) {
            Roc_Object[[i]] <- output[[i]][["Roc_Object"]]
          }

          ROC_plot <-
            ggroc(Roc_Object, legacy.axes = TRUE) +
            theme(plot.title = element_text(hjust = 0.5)) +
            geom_abline(
              slope = 1, intercept = 0,
              linetype = "dashed", alpha = 0.7, color = "black"
            ) +
            coord_equal() +
            theme_bw() +
            ggtitle("roc_plot") +
            theme(plot.title = element_text(hjust = 0.5)) +
            theme(
              axis.line = element_line(colour = "black"),
              panel.grid.major = element_blank(),
              panel.grid.minor = element_blank()
            ) +
            theme(axis.ticks.length = unit(.25, "cm")) +
            theme(legend.position = "top", legend.box = "horizontal") +
            theme(legend.title = element_blank()) +
            theme(text = element_text(size = 14, color = "black")) +
            theme(axis.text = element_text(size = 12, color = "black")) +
            xlab("False Positive Rate (1-Specificity)") +
            ylab("True Positive Rate (Sensitivity)") +
            annotation_custom(
              gridExtra::tableGrob(pr.auc.list, rows = NULL, theme = mytheme),
              xmin = 0.90, xmax = 0.90,   # was unit(0.9, "npc")
              ymin = 0.22, ymax = 0.22
            )


          # Make plots.
          plot_list <- list()
          plot_list$roc <- ROC_plot
          plot_list$pr <- PR_plot
          plot_list$point <- point_plot

          pdf(filename)
          for (i in seq_along(plot_list)) {
            print(plot_list[[i]])
          }
          dev.off()
        }

        final_output <- list()
        final_output[["Ensemble_training_output"]] <- Ensemble_output
        final_output[["classifier_performance"]] <- output
        final_output[["predicted_interactions"]] <- datScore

        return(final_output)
      }

    #' enrichfind_cpx
    #' @title Functional Enrichment Analysis for Predicted Modules
    #' @param predcpx Predicted modules resulted from
    #' \code{\link[HPiP]{run_clustering}}.
    #' @param threshold Custom p-value threshold for significance.
    #' @param sources A vector of data sources to use.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @param p.corrction.method The algorithm used for multiple testing
    #' correction;defaults to 'bonferroni'.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @param org An organism name;defaults to 'hsapiens'.
    #' See \code{\link[gprofiler2]{gost}} for more details.
    #' @return A data.frame with the enrichment analysis results.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @description This function uses \code{\link[gprofiler2]{gost}} function
    #' in \code{gprofiler2} package to perform functional enrichment analysis
    #' for predicted modules.
    #' @export

    enrichfind_cpx <-
        function(predcpx,
                 threshold = 0.05,
                 sources = c("GO", "KEGG"),
                 p.corrction.method = "bonferroni",
                 org = "hsapiens") {

            p_value <- NULL
            predcpx <-
                as.data.frame(predcpx)

            agg.data <-
                aggregate(list(predcpx[, 1]),
                          by = list(predcpx[, 2]),
                          FUN = function(x) {
                              paste(unique(x),
                                    collapse = ";"
                              )
                          }
                )
            colnames(agg.data)[2] <- "members"
            agg.data$members <-
                vapply(lapply(strsplit(agg.data$members, ";"), unique),
                       paste, character(1L),
                       collapse = ";"
                )


            indcpx <-
                str_split(agg.data$members, ";")
            names(indcpx) <- agg.data$Group.1

            annotCov <- lapply(indcpx, function(x) {
                gprofiler2::gost(x,
                                 significant = TRUE,
                                 exclude_iea = TRUE,
                                 evcodes = TRUE,
                                 user_threshold = threshold,
                                 sources = sources,
                                 correction_method = p.corrction.method,
                                 organism = org
                )
            })

            df <-
                lapply(annotCov, function(x) as.data.frame(x[[1]]))
            ans <-
                map_df(df, ~ as.data.frame(.x), .id = "id")

            ans <-
                arrange(ans, p_value)

            return(ans)
        }

    .F1_calc <-
      function(x) {
        . <- NULL
        V1 <- NULL
        V2 <- NULL
        AAcid <- NULL

        string <- strsplit(x, "")[[1]]
        rl <- rle(string)
        lst <- lapply(split(
          seq_along(string),
          rep(seq_along(rl$values), rl$lengths)
        ), range)
        names(lst) <- rl$values
        rl <-
          do.call(rbind, lst) %>%
          as.data.frame(.) %>%
          rownames_to_column("AAcid") %>%
          mutate(AAcid = str_remove_all(AAcid, ".\\d+")) %>%
          mutate(length = ((V2 - V1) + 1)^2) %>%
          group_by(AAcid) %>%
          summarise(Freq = sum(length))
        return(rl)
      }


    .F2_calc <-
      function(x) {
        . <- NULL
        V1 <- NULL
        V2 <- NULL
        AAcid <- NULL

        f <-
          x %>%
          lapply(., function(x) strsplit(x[[1]], "")) %>%
          lapply(., function(x) {
            paste(paste(x[[1]][-c(length(x[[1]]), length(x[[1]]) - 1)],
              x[[1]][-c(1, length(x[[1]]))], x[[1]][-c(
                seq_len(2),
                length(x[[1]])
              )],
              x[[1]][-c(seq_len(3), length(x[[1]]))],
              x[[1]][-c(seq_len(4), length(x[[1]]))],
              x[[1]][-c(seq_len(5), length(x[[1]]))],
              sep = ""
            ), x[[1]][-c(1, 2, 3, 4, 5, 6)],
            sep = ""
            )
          })


        x <- unlist(f)

        x <- paste(x, collapse = " ")
        x <- unlist(strsplit(x, split = ""))


        rl <- rle(x)
        lst <- lapply(split(
          seq_along(x),
          rep(seq_along(rl$values), rl$lengths)
        ), range)

        names(lst) <- rl$values
        rl <-
          do.call(rbind, lst) %>%
          as.data.frame(.) %>%
          tibble::rownames_to_column("AAcid") %>%
          mutate(AAcid = str_remove_all(AAcid, ".\\d+")) %>%
          mutate(length = ((V2 - V1) + 1)^2) %>%
          group_by(AAcid) %>%
          summarise(Freq = max(length)) %>%
          filter(AAcid != "X.")
        return(rl)
      }


    #' calculateF
    #' @title Calculate F1 or F2 Descriptors
    #' @param x A data.frame containing gene/protein names and their
    #' fasta sequences.
    #' @param type The descriptor type:
    #' \code{F1} or \code{F2}.
    #' @return A length 20 named vector for the data input.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom dplyr mutate
    #' @importFrom magrittr %>%
    #' @importFrom dplyr group_by
    #' @importFrom dplyr summarise
    #' @importFrom tibble rownames_to_column
    #' @importFrom tidyr separate
    #' @importFrom stringr str_remove_all
    #' @importFrom tidyr spread
    #' @description This function calculates F1 or F2 descriptors:
    #' \itemize{
    #' \item \code{F1} - sum of squared length of Single Amino Acid Repeats
    #' (SARs) in the entire protein sequence.
    #' \item \code{F2} - maximum of the sum of Single Amino Acid Repeats (SARs)
    #' in a window of 6 residues.
    #' }
    #' @export calculateF
    #' @references
    #' Alguwaizani, S., Park, B., Zhou, X., Huang, D.-S., and Han, K. (2018).
    #' Predicting interactions between virus and host proteins using repeat
    #' patterns and composition of amino acids.
    #' \emph{J. Healthc. Eng.} 2018.
    #' @examples
    #' data(UP000464024_df)
    #' x_df <- calculateF(UP000464024_df, type = "F1")
    #' head(x_df, n = 2L)
    calculateF <- function(x, type = c("F1", "F2")) {
      if (!is.data.frame(x)) {
        stop("Input data must be data.frame")
      }

      . <- NULL
      V1 <- NULL
      AAcid <- NULL
      Freq <- NULL


      # convert data frame to list
      fastalist <-
        as.list(unlist(x[, 2]))
      names(fastalist) <-
        unlist(x[, 1])

      # check if there is any unrecognized amino acid
      f <- .checkFASTA(fastalist)
      if (nrow(f) > 0) {
        stop("Fastalist has unrecognized amino acid type")
      }

      if (type == "F1") {
        p <-
          lapply(fastalist, function(x) .F1_calc(x[[1]]))
      }

      if (type == "F2") {
        p <-
          lapply(fastalist, function(x) .F2_calc(x[[1]]))
      }

      F_calc_df <-
        bind_rows(p, .id = "identifier") %>%
        spread(AAcid, Freq) %>%
        replace(is.na(.), 0)

      return(F_calc_df)
    }

    #' Host SummarizedExperiment object
    #' @description SummarizedExperiment object of numerical features for host
    #' proteins.
    #' @details
    #' To construct this object, first protein sequences were converted to
    #' numerical features using (CTD) descriptors provided in the HPiP package,
    #' followed by converting each numerical features matrix to
    #' SummarizedExperiment object.Each object is then merged into one object
    #' using `cbind()`.
    #' @name host_se
    #' @usage data(host_se)
    #' @docType data
    NULL

    #' enrichplot
    #' @title Plot the Enrichment Reuslt
    #' @param x A data.frame with the enrichment analysis results.
    #' @param low Colours for low.
    #' @param high Colours for high.
    #' @param cex.size Text size.
    #' @return An enrichment plot.
    #' @seealso See \code{\link{enrichfindP}} for functional enrichment
    #' analysis.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom ggplot2 scale_colour_gradient2
    #' @importFrom ggplot2 scale_x_continuous
    #' @importFrom ggplot2 element_rect
    #' @description This function plots the enrichment result.
    #' @export
    #' @examples
    #' data('enrich.df')
    #' #select enrichment for one of the example (e.g., E protein)
    #' enrich.df <-
    #' enrich.df[enrich.df$id == "E:P0DTC4", ]
    #' enrichplot(enrich.df, low = "blue", high = "red", cex.size = 10)

    enrichplot <-
      function(x,
               low = "blue",
               high = "red",
               cex.size = 15) {
        xaxis <- NULL
        term_name <- NULL
        intersection_size <- NULL
        p_value <- NULL

        x$xaxis <- 1

        print(ggplot(x, aes(xaxis, term_name, color = -log10(p_value))) +
          geom_point(aes(xaxis, term_name, size = intersection_size),
            stroke = 1, show.legend = TRUE
          ) +
          scale_colour_gradient2(low = low, high = high) +
          theme_bw() +
          scale_x_continuous(expand = c(0, 0), limits = c(1, 1)) +
          theme(text = element_text(size = cex.size, color = "#000000")) +
          theme(axis.text.y = element_text(color = "#000000")) +
          theme(axis.ticks.x = element_blank()) +
          theme(axis.text.x = element_blank()) +
          labs(title = "", x = "", y = "term_name"))
      }

    #' getFASTA
    #' @title Fetch FASTA Sequence from the UniProt Database
    #' @param uniprot.id A character vector of UniProt identifiers.
    #' @param filename A character string, indicating the output filename as an
    #' RData object to store the retrieved sequences.
    #' @param path A character string indicating the path to the project
    #' directory that contains the interaction data.
    #' If the directory is missing, it will be stored in the current directory.
    #' Default is FASTASeq.
    #' @return A list containing protein FASTA sequences.
    #' @author Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
    #' @importFrom stats na.omit
    #' @importFrom utils read.delim
    #' @importFrom protr readFASTA
    #' @description This function retrieves protein sequences in FASTA format
    #' directly from the UniProt database via UniProt protein IDs. This function
    #' also checks if the amino-acid composition of protein sequences
    #' is in the 20 default types.
    #' @export
    #' @examples
    #' # get fasta sequences for three proteins of SARS-Cov-2
    #' local = tempdir()
    #' uniprot.id <- c("P0DTC4", "P0DTC5", "P0DTC9")
    #' fasta_df <- getFASTA(uniprot.id, filename = 'FASTA.RData', path = local)
    #' head(fasta_df)

    getFASTA <- function(uniprot.id, filename = "FASTA.RData",
                         path = "FASTASeq") {
      . <- NULL
      V1 <- NULL

      `%nin%` <- Negate(`%in%`)

      names <- unique(uniprot.id)
      if (!is.character(names)) names <- as.character(names)
      len <- length(names)
      fastalist.query <- vector("list", len)

      for (i in seq_len(len)) {
        fastalist.query[[i]] <-
          tryCatch(protr::readFASTA(paste(
            "https://www.uniprot.org/uniprot/", names[i], ".fasta",
            sep = ""
          ))[[1]], error = function(err) {
            cat("Error detected --> NA substituted\n")
            return(NA)
          })
      }

      names(fastalist.query) <- names
      na.omit.list <-
        function(y)
        { return(y[!sapply(y, function(x) all(is.na(x)))]) }

      fastalist.query <-
        na.omit.list(fastalist.query)

      # check if protein sequence's amino acid types re in the 20 default types
      s1.check <-
        lapply(fastalist.query, function(x) protr::protcheck(x))
      s1.check <-
        do.call(rbind, s1.check) %>%
        as.data.frame(.) %>%
        rownames_to_column("Uniprot") %>%
        filter(V1 == FALSE) %>%
        .$Uniprot
      fastalist.query <-
        fastalist.query[names(fastalist.query) %nin% s1.check]


      fastalist.query.out <-
        do.call(rbind, fastalist.query) %>%
        as.data.frame(.) %>%
        rownames_to_column("UniprotKBID") %>%
        rename(FASTASEQ = V1) %>%
        na.omit(.)

      if (!file.exists(path)) {
        dir.create(path, recursive = TRUE)
        localPath <- tools::file_path_as_absolute(path)
        fname <- file.path(localPath, filename)
        save(fastalist.query.out, file = fname)
      } else {
        fname <- file.path(path, filename)
        save(fastalist.query.out, file = fname)
      }

      return(fastalist.query)
    }

test_get_negativePPI = function() {

  prot1 <- c("P0DTC4", "P0DTC5", "P0DTC9")
  prot2 <- c("Q9Y679", "Q9NW15", "Q9NXF8")
  TPset <- c("P0DTC4~P31948", "P0DTC8~Q13438")
  TN_PPI <- get_negativePPI(prot1, prot2, TPset)
  checkEquals(nrow(TN_PPI), 9L)

}

#!/usr/bin/env bash
set -euo pipefail

REPO="salvo-rs/hpip"
BINARY="hpip"

# --- Helpers ---

info() { printf '%s\n' "$@"; }
error() { printf 'Error: %s\n' "$@" >&2; exit 1; }

need_cmd() {
    if ! command -v "$1" > /dev/null 2>&1; then
        error "need '$1' (command not found)"
    fi
}

# --- Detect platform ---

detect_target() {
    local os arch target

    os="$(uname -s)"
    arch="$(uname -m)"

    case "$os" in
        Linux)  os="unknown-linux-gnu" ;;
        Darwin) os="apple-darwin" ;;
        *)      error "unsupported OS: $os" ;;
    esac

    case "$arch" in
        x86_64|amd64)  arch="x86_64" ;;
        aarch64|arm64) arch="aarch64" ;;
        *)             error "unsupported architecture: $arch" ;;
    esac

    target="${arch}-${os}"
    printf '%s' "$target"
}

# --- Resolve install directory ---

resolve_install_dir() {
    if [ -n "${INSTALL_DIR:-}" ]; then
        printf '%s' "$INSTALL_DIR"
        return
    fi

    if [ -w /usr/local/bin ]; then
        printf '%s' "/usr/local/bin"
    else
        local dir="${HOME}/.local/bin"
        mkdir -p "$dir"
        printf '%s' "$dir"
    fi
}

# --- Fetch latest version tag ---

get_latest_version() {
    local url="https://api.github.com/repos/${REPO}/releases/latest"
    local version

    if command -v curl > /dev/null 2>&1; then
        version="$(curl -fsSL "$url" | grep '"tag_name"' | sed -E 's/.*"tag_name":\s*"([^"]+)".*/\1/')"
    elif command -v wget > /dev/null 2>&1; then
        version="$(wget -qO- "$url" | grep '"tag_name"' | sed -E 's/.*"tag_name":\s*"([^"]+)".*/\1/')"
    else
        error "need 'curl' or 'wget' to download"
    fi

    if [ -z "$version" ]; then
        error "could not determine latest version"
    fi

    printf '%s' "$version"
}

# --- Download and install ---

download_and_install() {
    local version="$1"
    local target="$2"
    local install_dir="$3"

    local archive_name="${BINARY}-${target}.tar.gz"
    local url="https://github.com/${REPO}/releases/download/${version}/${archive_name}"

    local tmpdir
    tmpdir="$(mktemp -d)"
    # shellcheck disable=SC2064
    trap "rm -rf '$tmpdir'" EXIT

    info "Downloading ${BINARY} ${version} for ${target}..."

    if command -v curl > /dev/null 2>&1; then
        curl -fsSL "$url" -o "${tmpdir}/${archive_name}"
    elif command -v wget > /dev/null 2>&1; then
        wget -q "$url" -O "${tmpdir}/${archive_name}"
    fi

    info "Extracting to ${install_dir}..."

    tar xzf "${tmpdir}/${archive_name}" -C "$tmpdir"
    install -m 755 "${tmpdir}/${BINARY}" "${install_dir}/${BINARY}"
}

# --- Main ---

main() {
    local version="${1:-}"
    local target install_dir

    need_cmd uname
    need_cmd tar
    need_cmd install

    target="$(detect_target)"
    install_dir="$(resolve_install_dir)"

    if [ -z "$version" ]; then
        version="$(get_latest_version)"
    fi

    download_and_install "$version" "$target" "$install_dir"

    info ""
    info "Successfully installed ${BINARY} ${version} to ${install_dir}/${BINARY}"

    if ! echo "$PATH" | tr ':' '\n' | grep -qx "$install_dir"; then
        info ""
        info "NOTE: ${install_dir} is not in your PATH."
        info "Add it with:"
        info "  export PATH=\"${install_dir}:\$PATH\""
    fi
}

main "$@"

#Requires -Version 5.1
<#
.SYNOPSIS
    Install hpip on Windows.
.DESCRIPTION
    Downloads the latest hpip release from GitHub and installs it.
.PARAMETER Version
    Specific version to install (e.g., "v0.1.2"). Defaults to latest.
.PARAMETER InstallDir
    Installation directory. Defaults to "$env:USERPROFILE\.hpip\bin".
.EXAMPLE
    irm https://raw.githubusercontent.com/salvo-rs/hpip/main/install.ps1 | iex
.EXAMPLE
    .\install.ps1 -Version v0.1.2
#>

param(
    [string]$Version = "",
    [string]$InstallDir = ""
)

$ErrorActionPreference = "Stop"

$Repo = "salvo-rs/hpip"
$Binary = "hpip"

function Get-LatestVersion {
    $url = "https://api.github.com/repos/$Repo/releases/latest"
    try {
        $release = Invoke-RestMethod -Uri $url -Headers @{ "User-Agent" = "hpip-installer" }
        return $release.tag_name
    }
    catch {
        throw "Failed to fetch latest version: $_"
    }
}

function Get-InstallDirectory {
    if ($InstallDir -ne "") {
        return $InstallDir
    }
    $dir = Join-Path $env:USERPROFILE ".hpip\bin"
    return $dir
}

function Install-Hpip {
    $target = "x86_64-pc-windows-msvc"

    if ([string]::IsNullOrEmpty($Version)) {
        $Version = Get-LatestVersion
    }

    $installPath = Get-InstallDirectory

    if (-not (Test-Path $installPath)) {
        New-Item -ItemType Directory -Path $installPath -Force | Out-Null
    }

    $archiveName = "$Binary-$target.zip"
    $url = "https://github.com/$Repo/releases/download/$Version/$archiveName"

    $tmpDir = Join-Path ([System.IO.Path]::GetTempPath()) ([System.Guid]::NewGuid().ToString())
    New-Item -ItemType Directory -Path $tmpDir -Force | Out-Null

    try {
        $archivePath = Join-Path $tmpDir $archiveName

        Write-Host "Downloading $Binary $Version for $target..."
        Invoke-WebRequest -Uri $url -OutFile $archivePath -UseBasicParsing

        Write-Host "Extracting to $installPath..."
        Expand-Archive -Path $archivePath -DestinationPath $tmpDir -Force

        $exePath = Join-Path $tmpDir "$Binary.exe"
        Copy-Item -Path $exePath -Destination (Join-Path $installPath "$Binary.exe") -Force
    }
    finally {
        Remove-Item -Path $tmpDir -Recurse -Force -ErrorAction SilentlyContinue
    }

    # Add to user PATH if not already present
    $userPath = [Environment]::GetEnvironmentVariable("Path", "User")
    if ($userPath -notlike "*$installPath*") {
        [Environment]::SetEnvironmentVariable("Path", "$installPath;$userPath", "User")
        $env:Path = "$installPath;$env:Path"
        Write-Host ""
        Write-Host "Added $installPath to your user PATH."
        Write-Host "Restart your terminal for the PATH change to take effect."
    }

    Write-Host ""
    Write-Host "Successfully installed $Binary $Version to $installPath\$Binary.exe"
}

Install-Hpip

---
title: "Introduction to HPiP"
author: 
- name: Matineh Rahmatbakhsh
  affiliation: Babu Lab, University of Regina, Regina, Saskatchwan
  email: mmr668@uregina.ca
package: HPiP

output: 
    BiocStyle::html_document:
    toc_float: true
    toc_depth: 4
    number_sections: true
    highlight: "textmate"
vignette: >
  %\VignetteIndexEntry{Introduction to HPiP}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: HPiP.bib
csl: bioinformatics.csl  
---

```{r echo=FALSE,message=FALSE, warning=FALSE}
library("knitr")
```

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)
```


```{r, echo=FALSE,message=FALSE, warning=FALSE}
suppressPackageStartupMessages({
  library("readr")
  library("dplyr")
  library("Biostrings")
  library("SummarizedExperiment")
  library("stringr")
})
```


\pagebreak


# Introduction
Infectious disease imposes a significant threat to human health and pose
substantial healthcare costs.  Infectious diseases result from the cross-talks 
between hosts and pathogens, which are mainly mediated by protein-protein 
interactions between hosts and pathogen proteins (HP-PPIs). The potential 
(HP-PPIs) represents the crucial elements of the infection mechanism as they 
decide the outcome, leading to either pathogen clearance or spread of the 
pathogen in the host due to evading the host immune response 
[@rahmatbakhsh2021bioinformatic]. Therefore, the study of the host-pathogen 
interactome is increasingly vital to uncover the molecular attributes of 
infectious diseases and potentially discover novel pharmacological targets or 
laying a strong foundation for repurposing of existing drugs.

In the past decades, several high throughput experimental approaches have been
developed to chart HP-PPIs on a large scale (e.g., yeast two-hybrid (Y2H) 
system [@ito2001comprehensive] or affinity purification (AP) coupled to mass
spectrometry (MS) [@puig2001tandem]). However, such high-throughput experimental
screens are typically laborious, time-consuming, and challenging to capture the
complete interactome, resulting in limited number of experimentally validated 
interactome in a database of HP-PPIs [@hart2006complete]. In-silico prediction 
of HP-PPIs can complement wet-lab experiments by suggesting candidate 
interaction partners for all the host proteins and excluding partners with low 
interacting probability scores to reduce the range of possible PPI candidates 
that need to be validated via wet-lab experiments. Specialized computational 
approaches to predict HP-PPIs are therefore of significant importance. 
While many computational tools have been developed to predict intra-species 
PPIs (i.e., PPIs within the same species)
[@wu2006prediction; @shen2007predicting; @nourani2015computational],
the availability of computational tools to predict inter-species PPIs 
such as HP-PPIs are rare.

For this purpose, we describe HPiP (host-pathogen interaction prediction), 
an R package for automated prediction of HP-PPIs using structural and 
physicochemical descriptors computed from amino acid-composition of host and 
pathogen proteins. Briefly, HPiP extracts gold-standard of experimentally 
verified HP-PPIs (i.e., positive interactions) from public repository, 
construct negative interactions via negative sampling, retrieve and convert 
protein sequences to numerical representation via various descriptors, applies 
multivariate feature selection based on correlation and recursive feature 
elimination (RFE)-embedded, and finally applies ensemble averaging to predict 
interactions. Taken together, we hope that the HPiP package not only 
contributes a useful predictor to accelerate the exploration of host-pathogen 
PPIs, but also provides some meaningful insights into host-pathogen 
relationships.  

# Overview of HPiP
Briefly, HPiP downloads the gold-standard data sets of experimentally verified 
host-pathogen PPIs from the BioGRID database [@stark2006biogrid]. 
Such interactions serve as a positive set. In the absence of ground truth 
negative examples, HPiP uses negative sampling to construct a negative set. 
Following the construction of gold-standard data, HPiP retrieves the FASTA 
sequences of associated proteins. HPiP then represents protein sequences into
a fixed-length feature vector using a series of structural and physicochemical 
descriptors. Host-pathogen feature vectors and the accompanying gold standard 
reference set also called the training set, are fed into the hybrid 
filter-wrapper feature selection method to select the most relevant features 
in inferring the target variable. In the following step, HPiP uses a training 
set to train a series of individual machine learning models (base learners) 
provided in the caret package [@kuhn2020package]. For each applied model, 
hyperparameters are tweaked throughout training via resampling techniques 
(e.g., k-fold cross-validation), and the best set of hyperparameters are 
selected based on the accuracy performance measure. The optimized models 
will then be applied to host-pathogen feature vectors with an unknown class 
label to return a classification result for each pair. The HPiP then uses 
ensemble averaging to average classification results over an ensemble of 
classifiers for each possible interaction. Finally, HPiP compares the 
algorithmic performance of the ensemble model with individual base learners
through resampling technique (e.g., k-fold cross-validation) and various 
performance metrics (e.g., accuracy).



# An Example of Predicting HP-PPIs
In the following sections, we explain the main components of the HPiP package, 
including dataset preparation (i.e., construction of the gold-standard set, 
FASTA sequence retrieval), feature extraction, data processing steps (i.e., 
imputation of missing values, feature selection), ensemble model generation 
and evaluation, prediction of HP-PPIs, network visualization and external 
validation of the predicted network using functional enrichment analysis. 
Furthermore, we guide users through each step by applying the HPiP to 
sample data derived from public databases. 


## Data Set Preparation
  
### Gold Standard Reference Dataset of Host-Pathogen PPIs
In this tutorial, we use the data provided by @samavarchi2020sars as our 
benchmark dataset. In this study, the authors mapped interaction between 27 
SARS-CoV-2 and host proteins via the proximity-dependent biotinylation (BioID)
approach. We then randomly selected 500  SARS-CoV-2-host interaction pairs from
all pairs as the positive samples. Since ground truth negatives are not 
available, here negative examples are generated from the positive set using
negative sampling [@eid2016denovo]. In this approach, negative instances are 
sampled from all the possible pairwise combinations of host and viral proteins, 
as long as the possible pairs do not occur in the positive reference set. 
To prevent statistical differences, the same scale is assumed for the negative 
and positive instances (i.e., the ratio of positive to negative 1:1) 
[@zhou2018generalized]. The `gold-reference` data set can be loaded with the 
following command: 
    
```{r}
# Loading packages required for data handling & data manipulation
library(dplyr)
library(tibble)
library(stringr)
# Loading HPiP package
library(HPiP)
# Loading data 
data(Gold_ReferenceSet)
dim(Gold_ReferenceSet)
```
As stated by `dim()` the gold reference set includes 1000 HP-PPIs interaction 
between 27 SARS-CoV-2 and 784 host proteins. 
  
In addition, users can use `get_positivePPI` in the HPiP package to construct 
`positive set` from the BioGRID database [@stark2006biogrid]. 

This function takes the following parameters:

- `organism.taxID` Taxonomy identifier for the pathogen.
- `access.key` Access key for using BioGRID webpage. To retrieve interactions 
from the BioGRID database, the users are first required to register for 
[access key](https://webservice.thebiogrid.org/).
- `filename` A character string, indicating the output filename as an RData 
object to store the retrieved interactions.
- `path` A character string indicating the path to the project directory 
that contains the interaction data. If the directory is missing, it will 
be stored in the current directory.

```{r, eval=FALSE}
local = tempdir()
#Get positive interactions from BioGrid 
TP <- get_positivePPI(organism.taxID = 2697049,
                      access.key = 'XXXX',
                            filename = "PositiveInt.RData",
                            path = local)
```

```{r, echo=FALSE, warning=FALSE, message=FALSE, results='hide'}
TP <- read_csv(
          system.file("extdata/TP_set.csv", package = "HPiP"),
          show_col_types = FALSE
        )
```
Subsequently, we can construct negative set via negative sampling using the 
following command:
```{r}
#pathogen proteins
prot1 <- unique(TP$`Official Symbol Interactor A`)
#host proteins
prot2 <- unique(TP$`Official Symbol Interactor B`)
#true positive PPIs 
TPset <- TP$PPI
TN <- get_negativePPI(prot1 , prot2, TPset)
dim(TN)
```

### FASTA Sequence extraction
To compute different features from protein sequences, we must first extract 
their sequences (in FASTA format). The `getFASTA` function in the HPiP package
can retrieve the sequences for any organism from the UniProt database. 

```{r}
local = tempdir()
#retrieve FASTA sequences of SARS-CoV-2 virus 
id = unique(Gold_ReferenceSet$Pathogen_Protein)
fasta_list <- getFASTA(id, filename = 'FASTA.RData', path = local)
```
## Sequence-based Features Extraction
To apply a learning algorithm on a host or pathogen protein sequence, it is 
needed to encode sequence information as numerical features. However, one of 
the critical challenges in inferring protein-protein interactions from the
protein sequences is finding an appropriate way to encode protein sequences' 
important information fully. Also, the amino-acid sequences of different lengths
should be converted to fixed-length feature vectors, which is crucial in 
classifying training data using machine-learning techniques as such techniques
require fixed-length patterns. The HPiP offers multiple functions for 
generating various numerical features from protein sequences.

These feature coding schemes listed in HPiP include amino acid composition (AAC)
, dipeptide composition (DC), tripeptide composition (TC), 
tripeptide composition (TC) from biochemical similarity classes, quadruplets 
composition (QC), F1, F2, CTD (composition/transition/distribution), 
conjoint triad, autocorrelation, *k*-spaced amino acid pairs, and binary 
encoding. 

### Amino acid Composition (AAC) Descriptor
The amino acid composition (AAC) has low complexity and has been widely
used to predict protein-protein interactions (PPIs) 
[@beltran2019predicting; @dey2020machine].The AAC explains the fraction of a 
type of amino acid found within a protein sequence [@dey2020machine]. This 
gives 20-dimensional feature vectors. For example, the fraction of all 20 
natural amino acids is computed as follow:

\[
f_{(i)}=\frac{n_i}{L} \text{ }\  (i = 1,2,3,....,20)
\]

where *n*~i~ is the number of amino acid type and *L* is the sequence length.
The ACC descriptor from the  protein sequences can be loaded
with the following command:

```{r, results='hide'}
# Convert the list of sequences obtained in the previous section to data.frame 
fasta_df <- do.call(rbind, fasta_list) 
fasta_df <- data.frame(UniprotID = row.names(fasta_df), 
                       sequence = as.character(fasta_df))

#calculate AAC
acc_df <- calculateAAC(fasta_df)
#only print out the result for the first row 
acc_df[1,-1] 
```

```{r, echo= FALSE}
ex <- acc_df[1,-1] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```


### Dipeptide Composition (DC) Descriptor
The dipeptide composition (DC) is simply the fraction of the different adjacent 
pairs of amino acids within a protein sequence [@bhasin2004classification]. 
Also, this descriptor encapsulates the properties of neighboring amino acids. 
Dipeptide composition converts a protein sequence to a vector of 400 dimensions.
The composition of all 400 natural amino acids can be calculated using the 
following equation: 

\[
  f_{(m,k)}=\frac{n_{m,k}}{L-1} \text{ }\  (m,k = 1,2,3,....,20)
\]

where *n*~m,k~ corresponds to the number of dipeptide compositions characterized
by amino acid type *m* and type *k*, while *L* is the sequence length.The DC 
descriptor from the protein sequences can be loaded with the following command:
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
dc_df <- calculateDC(fasta_df)
#only print out the first 30 elements for the first row 
dc_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <- dc_df[1, c(2:31)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
### Tripeptide Composition (TC) Descriptor
The tripeptide composition explains the occurrence of adjacent triune residues 
in a protein sequence [@liao2011predicting]. Tripeptide composition converts a 
protein sequence to a vector of 8,000 dimensions. The composition of all 
8,000-dimensional descriptor can be calculated using the following equation: 
\[
  f_{(m,k,j)}=\frac{n_{m,k,j}}{L-2} \text{ }\  (m,k,j = 1,2,3,....,20)
\]
where *n*~m,k,j~ corresponds to the number of tripeptide compositions 
characterized by amino acid type *m*, *k* and *j*, while *L* is the sequence 
length.The TC descriptor from the protein sequences can be loaded with 
the following command:
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
tc_df <- calculateTC(fasta_df)
#only print out the first 30 elements for the first row 
tc_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <-tc_df[1, c(2:31)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```

### Tripeptide Composition (TC) from Biochemical Similarity Classes Descriptor
In order to reduce the dimension of length-8,000 TC descriptor, the sequence 
alphabet is reduced from 20 amino acids to six classes based on biochemical 
similarity. The classes are [{IVLM}, {FYW}, {HKR}, {DE}, {QNTP}, and {ACGS} 
[@ahmed2018prediction]]. This classification of amino acids  converts a protein
sequence to a vector of 216 (i.e., 6 * 6 * 6) different combinations of possible
substrings of length 3. The frequency of triplet for each encoded class in the 
protein sequence can be calculated as follows:
  
\[
  q_{(i)}=\frac{f_i - M_0}{M_1-M_0}
\]
\[
  M_0 = min(f_1,f_2,...,f_{216})\text { and}\ M_1 = max(f_1,f_2,...,f_{216})
\]

Here *f*~i~ is the frequency of *i*^th^ triplet in the sequence *i*=1,2,...,216.
To get 216-dimensional descriptor from the protein sequences,
the following command can be used:
  
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
TC_Sm_df <- calculateTC_Sm(fasta_df)
#only print out the first 30 elements for the first row 
TC_Sm_df[1, c(2:31)] 

```

```{r, echo= FALSE}
ex <- TC_Sm_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```

### Quadruplets Composition from Biochemical Similarity Classes Descriptor
To compute these features, the sequence alphabet is first reduced to six classes
reported above (section `3.3.2.4`). This reduction converts a protein sequence
to a vector of 1296 (i.e., 6 * 6 * 6 * 6) different combinations of possible 
substrings of length 4 [@ahmed2018prediction]. The frequency of quadruplets for
each encoded class in the protein sequence can be calculated similarly to the
equation mentioned above:
  
\[
  q_{(i)}=\frac{f_i - M_0}{M_1-M_0}
\]
\[
  M_0 = min(f_1,f_2,...,f_{1296})\text { and}\ M_1 = max(f_1,f_2,...,f_{1296})
\]
To get 1296-dimensional descriptor from the protein sequences, the following 
command can be used:
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
QD_df <- calculateQD_Sm(fasta_df)
#only print out the first 30 elements for the first row 
QD_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <- QD_df[1, c(2:31)]
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```

### F1/F2 Composition Descriptor
F1 composition gives 20-dimensional description, defined as: 
  
\[
  F1(SAR)=\sum_{SAR\text{ }\epsilon\text{ } sequence}length(SAR)^2 
\]

Where *SAR* is the sum of squared length of single amino acid repeats (SARs) in 
the entire protein sequence. Since F1 includes SAR of length 1, the F1 
descriptor reveals global composition of amino acids and amino acid repeats 
[@alguwaizani2018predicting]. 


<img src="Figures/F-1.png" width="100%" class="img-responsive" align="center">
  <p align="right"> 
**Figure 1: Example of calculating F1 (repeats of S) in the protein sequence.** 
  
  
While, to calculate feature F2, the sequence alphabet is first split into 
substrings of length 6 residues [@alguwaizani2018predicting]. There are two main
differences between feature F2 and F1: 
  
- For F2, sum of square length of single amino acid repeats (SARS) is computed 
for every window of 6 residues. 
- The maximum  of the sum of squared length of SARs is selected for F2.

F2 composition gives 20-dimensional description, defined as: 
  
\[
  F1(SAR)=max_{windows\text{ }\epsilon\text{ }sequence} \
  sum_{SAR\text{ }\epsilon\text{ } sequence}length(SAR)^2 
\]

Where SAR is the sum of squared length of single amino acid repeats (SARs) 
in the entire protein sequence.


- **Get feature F1 from the protein sequences:**
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
F1_df <- calculateF(fasta_df, type = "F1")
#only print out the result the first row 
F1_df[1,-1] 
```

```{r, echo= FALSE}
ex <- F1_df[1,-1] 
#convert df to character vector 
x_df <- structure(as.numeric(ex), names = colnames(ex)) 
x_df
```
- **Get feature F2 from the protein sequences:**
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
F2_df <- calculateF(fasta_df, type = "F2")
#only print out the result the first row 
F2_df[1,-1] 
```

```{r, echo= FALSE}
ex <- F2_df[1,-1] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
### Composition/Transition/Distribution (CTD) Descriptors
To calculate CTD descriptors developed by 
[@dubchak1995prediction; @dubchak1999recognition], the 20 standard amino acids 
is first clustered into three classes according to its attribute. 
Then, each amino acid in the protein sequence is encoded by one of the indices 
1,2,3 depending on its grouping.  The corresponding divisions for each group are
shown in **Table 1**. According to Hydrophobicity grouping mentioned in
**Table 1**, the protein sequence `CLVIMFWGASTPHYRKEDQN` is replaced by 
`11111112222222333333`. Next, for a given attribute, three types of descriptors, 
composition (C), transition (T), and distribution (D) can be calculated, 
which will be explained in the following sections.

```{r, echo=FALSE}
df1 <- HPiP:::df1
df1[is.na(df1)] <- ""
knitr::kable(df1, align = "lccrr", caption = "Amino acid attributes and the division of amino acid into three-group.", longtable = TRUE)
```

#### Composition (C) Descriptor
The composition represents the number of amino acids of a particular property 
(e.g., hydrophobicity)  for each encoded class divided by the protein sequence 
length [@you2014prediction]. In the above example using the hydrophobicity 
attribute, the number for encoded classes `1`, `2`, `3` are 7,7,6 respectively. 
Therefore, the compositions for each class are **7/20 =35%**, **7/20 =35%**, 
and **6/20 =30%**, respectively. Composition descriptor can be defined as:
  
\[
  C_{(i)}=\frac{n_i}{L} \text{ }\  (i = 1,2,3)
\]

where *n*~i~ is the number of amino acid type i and *L* is the sequence length.
The C descriptor from the  protein sequences can be loaded with the following 
command:
  
  
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
CTDC_df <- calculateCTDC(fasta_df)
CTDC_df[1, c(-1)] 

```

```{r, echo= FALSE}
ex <- CTDC_df[1, c(-1)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```


#### Transition (T) Descriptor
Transition (T) characterizes the percent frequency from a type of amino acids 
to another type (Wang et al., 2017). For instance, a transition from class `1` 
to `2`  or `2` to `1` is the percent frequency with which class `1` is followed 
by class `2` or vice versa [@xiao2015protr]. The frequency of these transitions 
can be computed as follow: 
  
\[
  T_{(rs)}=\frac{n_{rs} + n_{sr}}{L-1} \text{ }\  (rs = 12,13,23)
\]

where *n*~rs~,*n*~sr~ are the number of dipeptide encoded as `rs` and `sr` in 
the sequence and and *L* is the sequence length.The T descriptor from the  
protein sequences can be calculated with the following command:
  
  
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
CTDT_df <- calculateCTDT(fasta_df)
#only print out the result for the first row 
CTDT_df[1, -1] 

```

```{r, echo= FALSE}
ex <- CTDT_df[1, -1] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
#### Distribution (D) Descriptor
The distribution measures the chain length within which the first, 25%, 50%, 
75%, and 100% of the amino acids of a particular property (e.g., hydrophobicity)
for a certain encoded class are located, respectively [@dubchak1995prediction]. 
For example, as shown in **Figure 3**,  there are 8 residues as `1`, the 
position for the first residue `1` , the 2nd residue `1` (25% * 8 = 2), 
the 5th `1`  residue (50% * 8 = 4), the 7th `1` (75% * 8= 6) and 
the 10th residue `2` (100% * 8 =8) in the encoded sequence are 1, 2, 13, 17, 22 
respectively, so that the distribution descriptors for residue `1`  are : 
(1/22) ×100% = 4.55%, (2/22)×100% = 9.09%, (13/22) ×100% = 59.09%, 
(17/22)×100% = 77.27%, (22/22)×100% = 100%, respectively. Likewise, the 
distribution descriptor for `2` and `3` is 
(18.18%, 18.18%, 27.27%, 63.64%, 95.45%) and (13.64%, 31.82%, 45.45%, 54.55%, 
86.36%), respectively.

<img src="Figures/ctd.png" width="100%" class="img-responsive" align="center" 
alt="CTD descriptors">
  <p align="left">
**Figure 2:The sequence of hypothetical protein showing the construction of CTD 
descriptors of a protein**. The index 1, 2 and 3 indicates the position of 
amino acid for each encoded class. 1-2 transitions indicated the position
of `12` or `21` pairs in the sequence. 
Similarly, 1-3 and 2-3 transitions are defined in the same way.

The D descriptor from the  protein sequences can be calculated with the 
following command:
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
CTDD_df <- calculateCTDD(fasta_df)
#only print out the first 30 elements for the first row 
CTDD_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <- CTDD_df[1, c(2:31)] 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```  
### Conjoint Triad Descriptor
The conjoint triad is one of the popular sequence-based approaches for 
protein-protein interactions prediction [@shen2007predicting]. This method 
encodes a protein sequence as a feature vector by calculating the frequency of 
amino acid triplets as follows (Figure 2) :
  
  
- Similar to section `3.3.2.4`, it encodes 20 amino acids to seven classes 
based on their dipoles and volumes of the side chains. These seven classes are 
[{AGV}, {DE}, {FILP}, {KR}, {MSTY}, and {C} (Shen et al., 2007)]


- A given protein sequence is then represented using three consecutive amino 
acids (i.e., amino acid triple).


- It uses 343-dimensional feature vectors to represent a given protein sequence,
where  then each feature vector *v* is then mapped to frequency 
vector *d*~i~ (i= 1,2,…343), which is defined as follow:  
  
  
\[
  d_i = \frac{f_i - \min\{\,f_1, f_2 , \ldots, f_{343}\,\}}{\max\{\,f_1, f_2, \
  ldots, f_{343}\,\}}
\]

Where *f*~i~ is the frequency of *i*-th triplet type in the protein sequence. 
The numerical value of *d*~i~ of each protein ranges between 0 to 1,
which therefore allows the comparison between proteins. 


<img src="Figures/CTriad.png" width="100%" class="img-responsive" 
align="center" alt="CTD descriptors">
  <p align="left">
**Figure 3: Schematic diagram for constructing conjoint triad method**. V is the
vector space of feature vectors that includes a fixed number of features; each 
feature (v~i~) includes amino acid triplet; F represents the frequency vector 
corresponding to V, and the value of *i*-th dimension of F(*f*~i~) corresponds
to the frequency of that *v*~i~-triad observed in the sequence.


The conjoint triad Descriptor descriptor from the  protein sequences can be 
calculated with the following command:
  
  
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
CTriad_df <- calculateCTriad(fasta_df)
#only print out the first 30 elements for the first row 
CTriad_df[1, c(2:31)] 

```

```{r, echo= FALSE}
ex <- CTriad_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
### Autocorrelation (Auto) Descriptors
Autocorrelation descriptors, also known as molecular connectivity indices, 
explain the magnitude of the correlation between protein or peptide sequences 
based on their particular structural or physiochemical information, which are 
defined according to the distribution of amino acid properties along the 
protein sequence [@ong2007efficacy]. Eight default properties [@xiao2015protr] 
are used here for deriving the autocorrelation descriptors: normalized average
hydrophobicity scales (AccNo. CIDH920105), average flexibility indices 
(AccNo. BHAR88010), polarizability parameter (AccNo. CHAM820101), free energy 
of solution in water(AccNo. CHAM820102), residue accessible surface area in 
tripeptide (AccNo. CHOC760101), residue volume (AccNo. BIGC670101), steric 
parameter (AccNo. CHAM810101), and relative mutability (AccNo. DAYM780201). 
Autocorrelation descriptors includes three types of descriptors 
(Morean-Broto, Moran, and Geary) which are described below. Prior to 
integrating any of the physiochemical attributes into the autocorrelation 
formula, these attributes need to be normalized by the following equation:
  
  
\[
  P_r = \frac{P_r - \bar{P}}{\sigma}
\]
where $\bar{P}$ is the mean value of the eight physiochemical attributes, 
and sigma represents the standard deviation, in which both can be defined as: 
  
\[
  \bar{P} = \frac{\sum_{r=1}^{20} P_r}{20} \quad \textrm{and} 
  \quad \sigma = \sqrt{\frac{1}{2} \sum_{r=1}^{20} (P_r - \bar{P})^2}
\]

The first type of autocorrelation is known as Moreau-Broto autocorrelation 
[@broto1984molecular]. Application  of Moreau-Broto autocorrelation to 
protein sequence is calculated by the following equation:  

\[
  AC(d) = \sum_{i=1}^{L-d} P_i P_{i + d} \quad d = 1, 2, \ldots, \textrm{nlag}
\]

where $P_i$ and $P_{i+d}$ represent the amino acid property at 
position $i$ and $i+d$ and $d$ is termed the lag of the autocorrelation 
along the protein sequence; 
$P_i$ and $P_{i+d}$. While, $\textrm{nlag}$ is the maximum value of the lag. 
This equation can be normalized based on peptide length to get normalized 
Moreau-Broto autocorrelation: 
  
\[
  ATS(d) = \frac{AC(d)}{L-d} \quad d = 1, 2, \ldots, \textrm{nlag}
\]

The second type of autocorrelation, named the Moran autocorrelation 
(Moran, 1950), can be defined as:
  
\[
I(d) = \frac{\frac{1}{L-d} \sum_{i=1}^{L-d} (P_i - \bar{P}') 
(P_{i+d} - \bar{P}')}{\frac{1}{L} \sum_{i=1}^{L} (P_i - \bar{P}')^2} 
\quad d = 1, 2, \ldots, 30
\]

where $d$, $P_i$, and $P_{i+d}$ are described in the same fashion as that 
for Moreau-Broto autocorrelation; $\bar{P}'$ is the mean of the given amino 
acid property $P$ across the protein sequence, i.e.,

\[
\bar{P}' = \frac{\sum_{i=1}^L P_i}{L}
\]

$d$, $P$, $P_i$ and $P_{i+d}$, $\textrm{nlag}$ are defined as above. The main 
difference between Moran and Moreau-Broto autocorrelation is that, unlike 
Moreau-Broto, the Moran autocorrelation utilizes the mean value of the given
amino acid property instead of the actual value of the property [@al2019rf]. 

The last type of autocorrelation , known as the Geary autocorrelation, 
can be calculated as: 
\[
C(d) = \frac{\frac{1}{2(L-d)} \sum_{i=1}^{L-d} (P_i - P_{i+d})^2}{\frac{1}{L-1}
\sum_{i=1}^{L} (P_i - \bar{P}')^2} \quad d = 1, 2, \ldots, 30
\]

where $d$, $P$, $P_i$, $P_{i+d}$, and $\textrm{nlag}$ are defined above. 
The key difference between Geary  and the other two autocorrelations is that 
the Geary autocorrelation utilizes the square difference of the property values
[@al2019rf]. 


Computing autocorrelation with HPiP is simple as the following commands: 
  
- **Get Moran autocorrelation:**
```{r, warning=FALSE, results='hide'}
# using data.frame provided by getFASTA function as data input
moran_df <- calculateAutocor(fasta_df,type = "moran", nlag = 10)
#only print out the first 30 elements for the first row 
moran_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <- moran_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
- **Get Normalized Moreau-Broto autocorrelation:**
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
mb_df <- calculateAutocor(fasta_df,type = "moreaubroto", nlag = 10)
#only print out the first 30 elements for the first row 
mb_df[1, c(2:31)] 

```

```{r, echo= FALSE}
ex <- mb_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```

- **Get Geary autocorrelation:**
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
geary_df <- calculateAutocor(fasta_df,type = "geary", nlag = 10)
#only print out the first 30 elements for the first row 
geary_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <- geary_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```


### k-Spaced Amino Acid Pairs
The k-spaced amino acid pairs (KSAAP) feature describes the number of 
occurrences of all possible amino acid pairs by a distance of k, which can be 
any number of residues up to two less than the protein length [@al2019rf]. 
For instance, given 400 (20 x 20) amino acid pairs and four values for k 
(k = 1-4), there would be 1600 attributes resulted from the KSAAP feature,
and the frequency of each amino acid pair with k spaces is calculated by
sliding through protein sequence one by once. Table 2 illustrates the outputs 
of using KSAAP features with various values of k for protein sequence 
`ARAQRTAAADARAKAAKAGCAARRAAATANYN`.  


```{r, echo=FALSE}
df2 <- HPiP:::df2
df2[is.na(df2)] <- ""
knitr::kable(df2, caption = "Composition of k-spaced amino acid pairs. Given 400 (20 × 20) amino acid pairs and four values for k (k=1–4), there are 1600 attributes generated for the KSAAP feature.", 
col.names = c("","","","","","","",""),row.names = NA,
             longtable = TRUE, align = "lccrr")
```
The KSAAP descriptor from the  protein sequences can be calculated with the
following command:
```{r, results='hide'}
# using data.frame provided by getFASTA function as data input
KSAAP_df <- calculateKSAAP(fasta_df)
#only print out the first 30 elements for the first row 
KSAAP_df[1, c(2:31)] 
```

```{r, echo= FALSE}
ex <- KSAAP_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
### Binary encoding
Binary encoding (BE) can be used to transform each residue in a protein 
sequence into 20 coding values [@al2019rf]. For example, ALa is described as 
(10000000000000000000) while Cys is defined as (01000000000000000000), etc. 
Thus, the total length of this feature is 400(20 * 20) vectors.

```{r, results='hide', warning=FALSE, eval = FALSE}
# using data.frame provided by getFASTA function as data input
BE_df <- calculateBE(fasta_df)
#only print out the first 30 elements for the first row 
BE_df[1, c(2:31)] 
```

```{r, echo= FALSE, eval = FALSE}
ex <- BE_df[1, c(2:31)] 
#convert df to character vector 
ex <- structure(as.numeric(ex), names = colnames(ex)) 
ex
```
### BString Object as Data Input
Alternatively, we can directly read the FASTA sequences into R using 
[Biostrings](https://bioconductor.org/packages/Biostrings)  
package [@pages2019biostrings], followed by converting the protein sequences 
into numerical features.
```{r}
#loading the package 
library(Biostrings)

#Read fasta sequences provided by HPiP package using Biostrings
fasta <- 
  readAAStringSet(system.file("extdata/UP000464024.fasta", package="HPiP"),
                  use.names=TRUE)
#Convert to df
fasta_bios = data.frame(ID=names(fasta),sequences=as.character(fasta))
#Extract the UniProt identifier
fasta_bios$ID <- sub(".*[|]([^.]+)[|].*", "\\1", fasta_bios$ID)
# for example, run ACC
acc_bios <- calculateAAC(fasta_bios)
```
- ***Note that `fasta_bios` can be used as data input for all the descriptors 
listed in section `3.3.2`***. 



### Generate a SummarizedExperiment Objects
[SummerizedExperiment](https://bioconductor.org/packages/SummarizedExperiment)
objects can be used to store and merge rectangular matrices of different 
outputs, as long as they have similar `rownames` or `colnames`. As illustrated 
in section `3.3.2`, all the computed data.frames have the same `rownames` but 
different features; therefore, we can easily use the `cbind` functions to merge
multiple `SummerizedExperiment` objects to one object. The HPiP package
provides two example SummarizedExperiment objects: `viral_se` and `host_se`.
`viral_se` includes pre-computed **(CTD)** numerical features per viral 
proteins present in the `Gold_ReferenceSet`. 
Similarly,`host_se` includes **(CTD)** pre-computed numerical features per 
host proteins in the `Gold_ReferenceSet`. 
```{r}
#loading viral_se object
data(viral_se)
viral_se
```

```{r}
#loading host_se object
data(host_se)
host_se
```
The numerical features from each SummarizedExperiment object can be easily 
retrieved using the `assays()$counts`, where each row represent the viral or
host proteins and each column represents one of the numerical features. 

As an example, construction of ` SummarizedExperiment` for viral proteins 
using **CTD** descriptors is as follows:
```{r}
#generate descriptors
CTDC_df <- calculateCTDC(fasta_df)
CTDC_m <- as.matrix(CTDC_df[, -1])
row.names(CTDC_m) <- CTDC_df$identifier

CTDT_df <- calculateCTDT(fasta_df)
CTDT_m <- as.matrix(CTDT_df[, -1])
row.names(CTDT_m) <- CTDT_df$identifier

CTDD_df <- calculateCTDD(fasta_df)
CTDD_m <- as.matrix(CTDD_df[, -1])
row.names(CTDD_m) <- CTDD_df$identifier
```

```{r}
#convert matrix to se object
ctdc_se <- SummarizedExperiment(assays = list(counts = CTDC_m),
                                colData = paste0(colnames(CTDC_df[,-1]),
                                                 "CTDC"))
ctdt_se <- SummarizedExperiment(assays = list(counts = CTDT_m),
                                colData = paste0(colnames(CTDT_df[,-1]),
                                                 "CTDT"))
ctdd_se <- SummarizedExperiment(assays = list(counts = CTDD_m),
                                colData = paste0(colnames(CTDD_df[,-1]),
                                                 "CTDD"))
#combine all se objects to one 
viral_se <- cbind(ctdc_se,ctdd_se,ctdt_se)
```

### Table of Summary Descriptors

```{r, echo=FALSE}
df3 <- HPiP:::df3
df3[is.na(df3)] <- ""
knitr::kable(df3, caption = "List of commonly used descriptors in HPiP.",
             align = "lccrr")

```
- ***Note that we can calculate protein sequence descriptors for the host 
proteins using the functions described above (i.e., section `3.2.1-3.2.14`)***.


## Combine Host-Pathogen Interaction Descriptors
To generate host-pathogen protein-protein interaction descriptors, 
sequence-based descriptors can be combined into one vector space 
using `getHPI()`, which provides two types of interactions, controlled by 
argument `type`. To illustrate the usage of `getHPI`, we will continue our 
example from section `3.2.16`

**1.Extraction of numerical features from `viral_se` and `host_se` objects**
```{r}
#extract features from viral_se
counts_v <- assays(viral_se)$counts
#extract row.names from viral_Se
rnames_v <- row.names(counts_v)
```

```{r}
#extract features from host_se
counts_h <- assays(host_se)$counts
#extract row.names from viral_Se
rnames_h <- row.names(counts_h)
```

**2.Map the features to the gold-standard data:**
```{r}
#Loading gold-standard data
gd <- Gold_ReferenceSet

x1_viral <- matrix(NA, nrow = nrow(gd), ncol = ncol(counts_v))
for (i in 1:nrow(gd)) 
  x1_viral[i, ] <- counts_v[which(gd$Pathogen_Protein[i] == rnames_v), ]

x1_host <- matrix(NA, nrow = nrow(gd), ncol = ncol(counts_h))
for (i in 1:nrow(gd)) 
  x1_host[i, ] <- counts_h[which(gd$Host_Protein[i] == rnames_h), ]
```

**3.Generate host-pathogen interaction descriptors using `getHPI`:**
```{r}
x <- getHPI(x1_viral,x1_host, type = "combine")
x <- as.data.frame(x)
x <- cbind(gd$PPI, gd$class, x)
colnames(x)[1:2] <- c("PPI", "class")
```

## Data Processing
It is crucial to pre-process the data (i.e., remove the noise) before feeding 
it into the machine learning model as the quality of data and valuable 
information that can be extracted from it directly affect the model's 
performance. The pre-processing steps are as follow:

- ***Handling missing values***: in any real-world data set, there are always
missing values. The easiest option is to remove rows or columns including 
missing values; however, such an approach results in losing valuable 
information. The alternative method is to impute missing values using 
neighboring information (e.g., average or median) or replace the missing 
values with zeros. HPiP package provides two functions to deal with the 
missing values. The `filter_missing_values` allows the user to drop the 
missing values above a certain threshold, controlled by argument 
`max_miss_rate`, while the `impute_missing_data` function replaces the null 
values with mean/median or zero, controlled by argument `method`. 

- ***Feature selection***: some of the sequence-based features are high 
dimensional, including hundreds to thousands of features. Unfortunately, 
such high-dimensional data includes many redundant features that reduce the 
predictive model's accuracy and increase the training time. 
The `FSmethod`function in the HPiP package combines two feature selection 
(FS) methods, controlled by `type()` argument, to eliminate redundant features. 
The first FS method is based on correlation analysis that computes the 
correlation between features using Pearson correlation measure and removes 
highly correlated features above the user-defined threshold. The second FS
method uses the Recursive Feature Elimination (RFE) algorithm  (wrapped up with
a random forest (rf) machine learning algorithm) to perform feature selection. 
RFE works by fitting the *rf* algorithm with all features in the training data
set, ranking features by importance, removing the least important features, 
and re-fitting the model until the desired number of features remains. 
The feature importance can be computed using *rf* model-independent metric 
(e.g.,  ROC curve analysis or accuracy), which is controlled by 
argument `metric()`. 
    
The complete set of arguments for `FSmethod` function are:
      
- `x` A data.frame containing protein-protein interactions, class labels and 
features.
- `type` The feature selection type 
- `cor.cutoff` Correlation coefficient cutoff used for filtering.
- `resampling.method` The re-sampling method (e.g., *k*-fold cross-validation) 
for RFE.
- `iter` Number of partitions for cross-validation.
- `repeats` For repeated *k*-fold cross validation only.
- `metric`  A string that specifies what summary metric will be used to select
the optimal feature.
- `verbose` Make the output verbose.
    
Continuing our example from section **3.3**, feature selection using both 
correlation analysis and RFE approach can be performed using the following 
command: 

```{r,message=FALSE}
#to use correlation analysis, make sure to drop the columns with sd zero
xx <- Filter(function(x) sd(x) != 0, x[,-c(1,2)])
xx <- cbind(x$PPI, x$class, xx)
colnames(xx)[1:2] <- c("PPI", "class")

#perform feature selection using both correlation analysis and RFE approach
set.seed(101)
x_FS <- FSmethod(xx, type = c("both"),
                 cor.cutoff = 0.8,resampling.method = "cv",
                 iter = 2,repeats =NULL, metric = "Accuracy", 
                 verbose = FALSE)
```

We can also visualize the results from the `FSmethod` analysis. For instance, 
the correlation matrix of unfiltered data can be visualized using the 
`corr_plot`. This will present us with a heatmap showing the correlation between
all the features prior to filtration.  
```{r,message=FALSE,fig.width = 8, fig.height = 8}
#cor plot
corr_plot(x_FS$cor.result$corProfile, method = 'square' , cex = 0.1)
```
In addition, the variable importance of retained features after the RFE feature 
selection approach can also be plotted using the `var_imp` function. 
```{r, fig.width = 8, fig.height = 4}
#var importance
var_imp(x_FS$rf.result$rfProfile, cex.x = 8, cex.y = 8)
```

## Prediction Algorithm
Sequence features and a list of gold-standard HP-PPIs can be fed into an 
ensemble classifier to rank the potential HP-PPIs interaction. This is 
accomplished via the `pred_ensmebel` function. This function uses the 
ensemble averaging approach, to combine any base classifiers provided in 
the `caret` package to predict HP-PPIs. To score interactions, the 
`pred_ensmebel` function uses the the training data 
(i.e., labelled HP-PPIs with sequence features) as well 
as unlabeled HP-PPIs data set to learn features that allow reliable prediction
of HP-PPIs, utilizing multiple ML algorithms. Following training, the trained 
models will be used to make predictions on unlabeled data. Then,ensemble 
averaging will be applied to average predictions over an ensemble of 
classifiers, each with different cross-validation splits. Finally, through 
resamplign techniques (e.g., *k*-fold cross-validation), this function also 
compares and evaluates the performance of ensemble model with individual 
machine learning models via commonly used measurements such as Recall 
(Sensitivity), Specificity, Accuracy , Precision, F1-score, and Matthews 
correlation coefficient (MCC). The corresponding formulae are as follows: 
  
\[
  Recall=Sensitivity=TPR=\frac{TP}{TP+FN}
\]

\[
  Specificity=1-FPR=\frac{TN}{TN+FP}
\]

\[
  Accuracy=\frac{TP+TN}{TP+TN+FP+FN}
\]

\[
  Precision=\frac{TP}{TP+FP}
\]


\[
  F1=2 \text{ × } \frac{Precision \text{ × } Recall}{Precision + Recall}
\]

\[
  MCC=\frac{TP \text{ × } TN - FP  \text{ × } FN}{\sqrt{(TP+FP)\text{ × } 
  (TP+FN)\text{ × } (TN+FP)\text{ × } (TN+FN)}}
\]

The `pred_ensemble` takes the following parameters: 
  
- `features` A data frame with host-pathogen protein-protein interactions 
(HP-PPIs) in the first column, and features to be passed to the classifier 
in the remaining columns. 
- `gold_standard` A data frame with gold_standard HP-PPIs and class label 
indicating if such PPIs are positive or negative.
- `classifier` The type of classifier to use. See `caret` package for all 
the available classifiers. 
- `resampling.method` The re-sampling technique 
(i.e., *k*-fold cross-validation).
- `ncross`  Number of partitions for cross-validation.
- `plots` Logical value, indicating whether to plot the performance of ensemble
learning algorithm as compared to individual classifiers.If the argument set
to TRUE, plots will be saved in current working directory.
- `verboseIter` Make the output verbose.
- `filename` A character string, indicating the output filename as an pdf
object.


As an example, we will use three base learners , support vector machine 
(`svmRadial`), Fitting Generalized Linear Models (`glm`), and random 
forest (`ranger`), controlled by argument `classifier`, to rank potential 
interaction. For the sake of time, we use only five-fold cross-validation 
(`ncross = 5`). In order to perform prediction, we will use `unlabel_data`, 
retrieved from Supplementary Table 1 presented in [@gordon2020sars] , which 
includes unlabeled HP-PPIs along with pre-computed `CTD` features, as well as 
constructed data containing labeled HP-PPIs from section `3.4`.
```{r}
#load the unlabeled HP-PPIs
data('unlabel_data')
#Constructed labeled HP-PPIs
labeled_dat <- x_FS$rf.result$rfdf
labeled_dat <- labeled_dat[,-1] 
#select important features
unlabel_data <- 
  unlabel_data[names(unlabel_data) %in% names(x_FS$rf.result$rfdf)]

#merge them 
ind_data <- rbind(unlabel_data,labeled_dat)
```

```{r}
# Get class labels
gd <-  x_FS$rf.result$rfdf
gd <-  gd[, c(2,1)]
```
Now we can predict interactions using `pred_ensembel`:
```{r,warning=FALSE,results='hide', message=FALSE}
set.seed(102)
ppi <- pred_ensembel(ind_data,
                     gd,
                     classifier = c("svmRadial", "glm", "ranger"),
                     resampling.method = "cv",
                     ncross = 5,
                     verboseIter = TRUE,
                     plots = FALSE,
                     filename=file.path(tempdir(), "plots.pdf"))
```
To retrieve predicted interactions from the result generated by `pred_ensembel`
function, we can just type:
```{r}
pred_interactions <- ppi[["predicted_interactions"]]
head(pred_interactions)
```
Finally, users can subset their list of high-confidence interactions for further
analysis, using a stringent classifier confidence ensemble score cutoff of 0.7:
```{r}
pred_interactions <- filter(pred_interactions, ensemble >= 0.7)
dim(pred_interactions)
```
When the `plots` argument set to TRUE, the `pred_ensembel` function generates 
one pdf file indicating the performance of the ensemble classier as compared 
to individual base learners



- The first plot shows the Receiver Operating Characteristic (ROC) curve.
<img src="Figures/ROC_Curve_new.png" width="100%" 
class="img-responsive" align="center">
  <p align="right"> **Figure 4: ROC_Curve curve.**
  
- The second plot shows the Precision-Recall (PR) curve 
<img src="Figures/pr-curve_new.png" width="100%" 
class="img-responsive" align="center">
  <p align="right"> **Figure 5: Precision-Recall (PR) curve.**
  
- The third plot shows the accuracy (ACC), F1-score ,positive predictive value 
(PPV),sensitivity (SE),and Matthews correlation coefficient (MCC)
of ensemble classifier vs selected individual classifiers.
<img src="Figures/point_plot_new.png" width="100%" 
class="img-responsive" align="center">
  <p align="right"> **Figure 6: Point plot.**

## Nework Visualization
Following PPI prediction, users can visualize the predicted PPI network using 
`plotPPI` and `FreqInteractors` functions.
    
- The `plotPPI` function, which is based on the *igraph* plotting function 
[@csardi2013package], provide visualization on interacting host protein 
partners of pathogen proteins. For instance, to get the PPI network of 
SARS-CoV2-ORF8-human, we can run the following command:
      
```{r,message=FALSE, results='hide',fig.width = 6, fig.height = 6}
S_interc <- filter(pred_interactions, 
                           str_detect(Pathogen_protein, "^ORF8:"))
#drop the first column
ppi <- S_interc[,-1]

plotPPI(ppi, edge.name = "ensemble",
            node.color ="red",
            edge.color = "grey",
            cex.node = 10,
            node.label.dist= 2)
```


- The `FreqInteractors` function, shows the degree distribution of pathogen 
 proteins in the HP-PPI network:
      
```{r, fig.width = 10, fig.height = 5}
ppi <- pred_interactions[,-1]
FreqInteractors(ppi,cex.size = 12)
```


## GO Enrichment Analysis
To identify significantly enriched annotation terms in predicted interacting 
host protein partners of each pathogen protein, we can use the `enrichfindP` 
function based on the *g:Profiler* tool [@kolberg2020gprofiler2]. Additionally,
we can use the `erichfind_hp` function to analyze functional characteristics of 
all predicted human proteins in the predicted network.

For instance, the following command can be used to performs functional 
enrichment analysis of host protein partners of each pathogen protein:

```{r, warning=FALSE, message=FALSE, results='hide', eval=FALSE}
enrich_result <- 
  enrichfindP(ppi,threshold = 0.05,
            sources = "GO",
            p.corrction.method = "bonferroni",
            org = "hsapiens")
```

## Complex Prediction
for this analysis, we utilizes the predicted HP-PPIS network generated in 
section `3.5` as input data for complex prediction. For the community detection 
analysis, `run_clustering` function provided in the HPiP package  includes the
five most popular complex detection algorithms in including fast-greedy, 
walktrap, label propagation, multi-level community, and markov clustering. For
example, to detect complexes using fast-greedy algorithm, we can run the 
following command:

```{r,message=FALSE}
ppi <- pred_interactions[,-1]
set.seed(103)
predcpx <- run_clustering(ppi, method = "FC")
```
Additionally, we can analyze the functional characteristics of 
these predicted modules via `enrichfind_cpx` function.   

```{r,message=FALSE}
enrichcpx <- enrichfind_cpx(predcpx,
             threshold = 0.05,
             sources = "GO",
             p.corrction.method = "bonferroni",
             org = "hsapiens")
```


    
# Session info

```{r}
sessionInfo()
```

# References
  

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_missing_data.R
\name{impute_missing_data}
\alias{impute_missing_data}
\title{Impute missing Values per Features (i.e., Columns)}
\usage{
impute_missing_data(x, method = c("mean", "median", "zero"))
}
\arguments{
\item{x}{A numeric matrix as input.}

\item{method}{Imputation method for missing values
(mean, median or zero).}
}
\value{
Imputed matrix.
}
\description{
Given an input matrix, impute the missing values
via three approaches including mean, median or zero.
}
\details{
impute_missing_data
}
\examples{
x <- matrix(1:10, ncol = 2)
x[1:3, 2] <- NA
row.names(x) <- c("A", "B", "C", "D", "E")
colnames(x) <- c("col1", "col2")
impute_missing_data(x, method = "mean")
impute_missing_data(x, method = "median")
impute_missing_data(x, method = "zero")
}
\author{
Matineh Rahmatbakhsh \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateTC.R
\name{calculateTC}
\alias{calculateTC}
\title{Calculate Tripeptide Composition (TC) Descriptor}
\usage{
calculateTC(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their fasta
sequences.}
}
\value{
A length 8,000  named vector for the data input.
}
\description{
This function calculates Tripeptide Composition (TC)
descriptor for data input.
}
\details{
calculateTC
}
\examples{
data(UP000464024_df)
x_df <- calculateTC(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Liao, B., Jiang, J.-B., Zeng, Q.-G., and Zhu, W. (2011).
Predicting apoptosis protein subcellular location with
PseAAC by incorporating tripeptide composition.
\emph{Protein Pept. Lett.} 18, 1086–1092
}
\seealso{
See \code{\link{calculateAAC}},\code{\link{calculateDC}} and
\code{\link{calculateTC_Sm}} for
Amino Acid Composition, Dipeptide Composition and Tripeptide
Composition (TC) Descriptor from Biochemical Similarity Classes.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/viral_se.R
\docType{data}
\name{viral_se}
\alias{viral_se}
\title{Viral SummarizedExperiment object}
\usage{
data(viral_se)
}
\description{
SummarizedExperiment object of numerical features for
SARS-CoV-2 proteins.
}
\details{
To construct this object, first protein sequences were converted to
numerical features using (CTD) descriptors provided in the HPiP package,
followed by converting each numerical features matrix to
SummarizedExperiment object.
Each object is then merged into one object using `cbind()`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateDC.R
\name{calculateDC}
\alias{calculateDC}
\title{Calculate Dipeptide Composition (DC) Descriptor}
\usage{
calculateDC(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their fasta
sequences.}
}
\value{
A length 400 named vector for the data input.
}
\description{
This function calculates Dipeptide Composition (DC)
descriptor for data input.
}
\details{
calculateDC
}
\examples{
data(UP000464024_df)
x_df <- calculateDC(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Bhasin, M., and Raghava, G. P. S. (2004).
Classification of nuclear receptors based on amino acid composition
and dipeptide composition. \emph{J. Biol. Chem.} 279, 23262–23266.
}
\seealso{
See \code{\link{calculateAAC}} and \code{\link{calculateTC}}
for Amino Acid Composition and Tripeptide Composition descriptors.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateAutocor.R
\name{calculateAutocor}
\alias{calculateAutocor}
\title{Calculate Autocorrelation Descriptors}
\usage{
calculateAutocor(
  x,
  target.props = c("CIDH920105", "BHAR880101", "CHAM820101", "CHAM820102",
    "CHOC760101", "BIGC670101", "CHAM810101", "DAYM780201"),
  nlag = 30L,
  type = c("moran", "geary", "moreaubroto")
)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their
fasta sequences.}

\item{target.props}{A character vector, specifying the
accession number of the target properties.
8 properties are used by default, as listed below:
\describe{
\item{AccNo. CIDH920105}{Normalized average hydrophobicity scales
 (Cid et al., 1992)}
\item{AccNo. BHAR880101}{Average flexibility indices
(Bhaskaran-Ponnuswamy, 1988)}
\item{AccNo. CHAM820101}{Polarizability parameter
(Charton-Charton, 1982)}
\item{AccNo. CHAM820102}{Free energy of solution in water, kcal/mole
(Charton-Charton, 1982)}
\item{AccNo. CHOC760101}{Residue accessible surface area in
 tripeptide (Chothia, 1976)}
\item{AccNo. BIGC670101}{Residue volume (Bigelow, 1967)}
\item{AccNo. CHAM810101}{Steric parameter (Charton, 1981)}
\item{AccNo. DAYM780201}{Relative mutability (Dayhoff et al., 1978b)}}}

\item{nlag}{Maximum value of the lag parameter. Default is \code{30}.}

\item{type}{The autocorrelation type:
\code{moran}, \code{geary}, or \code{moreaubroto}.}
}
\value{
A length \code{nlag} named vector for data input.
}
\description{
This function calculates autocorrelation descriptors:
\itemize{
\item \code{moran} - moran autocorrelation,
(Dim: \code{length(target.props) * nlag}).
\item \code{geary} - geary autocorrelation,
(Dim: \code{length(target.props) * nlag}).
\item \code{moreaubroto} - moreau-broto autocorrelation,
(Dim: \code{length(target.props) * nlag}).
}
}
\details{
calculateAutocor
}
\examples{
data(UP000464024_df)
x_df <- calculateAutocor(UP000464024_df,type = 'moran')
head(x_df, n = 2L)
}
\references{
AAindex: Amino acid index database.
\url{http://www.genome.ad.jp/dbget/aaindex.html}

Feng, Z.P. and Zhang, C.T. (2000)
Prediction of membrane protein types based on the hydrophobic
index of amino acids.
\emph{Journal of Protein Chemistry}, 19, 269-275.

Horne, D.S. (1988)
Prediction of protein helix content from
an autocorrelation analysis of sequence hydrophobicities.
\emph{Biopolymers}, 27, 451-477.

Sokal, R.R. and Thomson, B.A. (2006)
Population structure inferred by local spatial autocorrelation:
an Usage from an Amerindian tribal population.
\emph{American Journal of Physical Anthropology}, 129, 121-131.
}
\author{
Matineh Rahmatbakhsh <\email{matinerb.94@gmail.com}>,
        Nan Xiao
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_data.R
\docType{data}
\name{example_data}
\alias{example_data}
\title{Input Data for Prediction Algorithm}
\format{
a data.frame containing unlabeled or labeled HP-PPIs and pre-computed
numerical features.
}
\usage{
data(example_data)
}
\description{
Input data for \code{\link[HPiP]{pred_ensembel}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_imp.R
\name{var_imp}
\alias{var_imp}
\title{Variable Importance Plot}
\usage{
var_imp(x, cex.x = 1, cex.y = 2)
}
\arguments{
\item{x}{A list of elements returned from RFE analysis.
See \code{\link[caret]{rfe}} for more details}

\item{cex.x}{The size of x axis label.}

\item{cex.y}{The size of y axis label.}
}
\value{
Variable Importance Plot.
}
\description{
A graphical display of variable importance of selected
features.
}
\details{
var_imp
}
\examples{
data('example_data')
x <- na.omit(example_data)
#perform feature selection
s <- FSmethod(x, type = 'both',
cor.cutoff = 0.7, resampling.method = "repeatedcv",
iter = 5, repeats = 3, metric = "ROC", verbose = TRUE)
var_imp(s$rf.result$rfProfile, cex.x = 10, cex.y = 10)
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UP000464024_df.R
\docType{data}
\name{UP000464024_df}
\alias{UP000464024_df}
\title{Data.frame Containing SARS-CoV-2 FASTA Sequences}
\format{
A data.frame with two columns:(1) UniprotKBID, UniProt
identifier.(2) FASTASEQ, sequences per SARS-CoV-2 gene.
}
\source{
\url{https://www.uniprot.org/uniprot/?query=proteome:UP000464024}
}
\usage{
data(UP000464024_df)
}
\description{
This data includes one protein sequence per SARS-CoV-2 gene,
retrieved directly from UniProt database using
\code{\link[HPiP]{getFASTA}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getHPI.R
\name{getHPI}
\alias{getHPI}
\title{Generating Host-Pathogen Protein-Protein Interaction (HP-PPI)
Descriptors}
\usage{
getHPI(pathogenData, hostData, type = c("combine", "kron.prod"))
}
\arguments{
\item{pathogenData}{The pathogen descriptor matrix.}

\item{hostData}{The host descriptor matrix.}

\item{type}{The interaction type, one or two of
\code{"combine"} and \code{"kron.prod"}.}
}
\value{
A matrix containing the Host-Pathogen Protein-Protein Interaction
(HP-PPI) descriptors.
}
\description{
This function calculates Host-Pathogen Protein-Protein
Interaction (HP-PPI) descriptors via two approaches
\itemize{
\item \code{combine} - combine the two descriptor matrix,
result has \code{(p1 + p2)} columns
\item \code{kron.prod} - if A has m x n matrix and B is q x p matrix,
then the Kronecker product is the code{(pm × qn)} block matrix
}
}
\details{
getHPI
}
\examples{
x <- matrix(c(1, 2, 3, 1), nrow = 2, ncol = 2, byrow = TRUE)
y <- matrix(c(0, 3, 2, 1), nrow = 2, ncol = 2, byrow = TRUE)
getHPI(x, y, "combine")
getHPI(x, y, "kron.prod")
}
\author{
Matineh Rahmatbakhsh \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCTDT.R
\name{calculateCTDT}
\alias{calculateCTDT}
\title{Calculate CTD Descriptors - Transition (T)}
\usage{
calculateCTDT(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and
their fasta sequences.}
}
\value{
A length 21 named vector for the data input.
}
\description{
This function calculates Transition (T) descriptor
for data input.
}
\details{
calculateCTDT
}
\examples{
data(UP000464024_df)
x_df <- calculateCTDT(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Dubchak, I., Muchnik, I., Holbrook, S. R., and Kim, S.-H. (1995).
Prediction of protein folding class using global description of
amino acid sequence.
\emph{Proc. Natl. Acad. Sci.} 92, 8700–8704.
}
\seealso{
See \code{\link{calculateCTDC}} and \code{\link{calculateCTDD}}
for Composition and Distribution descriptors.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateQD_Sm.R
\name{calculateQD_Sm}
\alias{calculateQD_Sm}
\title{Calculate Quadruples Composition (QC) Descriptor from
Biochemical Similarity Classes}
\usage{
calculateQD_Sm(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their fasta
sequences.}
}
\value{
A length 1296  named vector for the data input.
}
\description{
This function calculates Quadruples Composition (QC)
descriptor from biochemical similarity classes.
}
\details{
calculateQD_Sm
}
\examples{
data(UP000464024_df)
x_df <- calculateQD_Sm(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Ahmed, I., Witbooi, P., and Christoffels, A. (2018). Prediction of
human-Bacillus anthracis protein–protein interactions using
multi-layer neural network.\emph{Bioinformatics} 34, 4159–4164.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPPI.R
\name{plotPPI}
\alias{plotPPI}
\title{Plot the Predicted PPI}
\usage{
plotPPI(
  ppi,
  edge.name = "ensemble_score",
  node.color = "grey",
  edge.color = "orange",
  cex.node = 4,
  node.label.dist = 1.5
)
}
\arguments{
\item{ppi}{A data.frame containing protein-protein interactions with edge
score.}

\item{edge.name}{A character string giving an edge attribute name.}

\item{node.color}{The fill color of the node.}

\item{edge.color}{The color of the edge.}

\item{cex.node}{The size of the node.}

\item{node.label.dist}{The distance of the label from the center of the
node.}
}
\value{
A PPI plot.
}
\description{
Plot the predicted PPIs. This function uses the plot
function of the \code{igraph}.
}
\details{
plotPPI
}
\examples{
df <- data.frame(
    node1 = c("A", "B", "C", "D", "E"),
    node2 = c("C", "E", "E", "E", "A"),
    edge.scores = c(0.5, 0.4, 0.3, 0.2, 0.7)
)
plotPPI(df, edge.name = "edge.scores")
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFASTA.R
\name{getFASTA}
\alias{getFASTA}
\title{Fetch FASTA Sequence from the UniProt Database}
\usage{
getFASTA(uniprot.id, filename = "FASTA.RData", path = "FASTASeq")
}
\arguments{
\item{uniprot.id}{A character vector of UniProt identifiers.}

\item{filename}{A character string, indicating the output filename as an
RData object to store the retrieved sequences.}

\item{path}{A character string indicating the path to the project
directory that contains the interaction data.
If the directory is missing, it will be stored in the current directory.
Default is FASTASeq.}
}
\value{
A list containing protein FASTA sequences.
}
\description{
This function retrieves protein sequences in FASTA format
directly from the UniProt database via UniProt protein IDs. This function
also checks if the amino-acid composition of protein sequences
is in the 20 default types.
}
\details{
getFASTA
}
\examples{
# get fasta sequences for three proteins of SARS-Cov-2
local = tempdir()
uniprot.id <- c("P0DTC4", "P0DTC5", "P0DTC9")
fasta_df <- getFASTA(uniprot.id, filename = 'FASTA.RData', path = local)
head(fasta_df)
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/host_se.R
\docType{data}
\name{host_se}
\alias{host_se}
\title{Host SummarizedExperiment object}
\usage{
data(host_se)
}
\description{
SummarizedExperiment object of numerical features for host
proteins.
}
\details{
To construct this object, first protein sequences were converted to
numerical features using (CTD) descriptors provided in the HPiP package,
followed by converting each numerical features matrix to
SummarizedExperiment object.Each object is then merged into one object
using `cbind()`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateTC_Sm.R
\name{calculateTC_Sm}
\alias{calculateTC_Sm}
\title{Calculate Tripeptide Composition (TC) Descriptor from Biochemical
Similarity Classes}
\usage{
calculateTC_Sm(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their fasta
sequences.}
}
\value{
A length 216  named vector for the data input.
}
\description{
This function calculates Tripeptide Composition (TC)
descriptor from biochemical similarity classes.
}
\details{
calculateTC_Sm
}
\examples{
data(UP000464024_df)
x_df <- calculateTC_Sm(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Ahmed, I., Witbooi, P., and Christoffels, A. (2018). Prediction of
human-Bacillus anthracis protein–protein interactions using multi-layer
neural network.\emph{Bioinformatics} 34, 4159–4164.

Cui, G., Fang, C., and Han, K. (2012). Prediction of protein-protein
interactions between viruses and human by an SVM model.
\emph{BMC bioinformatics}, 1–10.
}
\seealso{
See \code{\link{calculateTC}} for Tripeptide Composition
descriptor.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateKSAAP.R
\name{calculateKSAAP}
\alias{calculateKSAAP}
\title{Calculate k-spaced Amino Acid Pairs (KSAAP) Descriptor}
\usage{
calculateKSAAP(x, spc = 3)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their fasta
sequences.}

\item{spc}{A number of spaces separating two adjacent residues by a
distance of spc, which can be any number up to two less than
the length of the peptide; default to 3.}
}
\value{
A length 400 named vector for the data input.
}
\description{
This function calculates k-spaced Amino Acid Pairs
(KSAAP) Descriptor for data input.
This function is adapted from the \code{\link[ftrCOOL]{CkSAApair}}
 function in the ftrCOOL package.
}
\details{
calculateKSAAP
}
\examples{
data(UP000464024_df)
x_df <- calculateKSAAP(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Kao, H.-J., Nguyen, V.-N., Huang, K.-Y., Chang, W.-C., and Lee, T.-Y.
(2020).SuccSite: incorporating amino acid composition and informative
k-spaced amino acid pairs to identify protein succinylation sites.
\emph{Genomics. Proteomics Bioinformatics} 18, 208–219.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichfind_cpx.R
\name{enrichfind_cpx}
\alias{enrichfind_cpx}
\title{Functional Enrichment Analysis for Predicted Modules}
\usage{
enrichfind_cpx(
  predcpx,
  threshold = 0.05,
  sources = c("GO", "KEGG"),
  p.corrction.method = "bonferroni",
  org = "hsapiens"
)
}
\arguments{
\item{predcpx}{Predicted modules resutled from
\code{\link[HPiP]{run_clustering}}.}

\item{threshold}{Custom p-value threshold for significance.}

\item{sources}{A vector of data sources to use.
See \code{\link[gprofiler2]{gost}} for more details.}

\item{p.corrction.method}{The algorithm used for multiple testing
correction;defaults to 'bonferroni'.
See \code{\link[gprofiler2]{gost}} for more details.}

\item{org}{An organism name;defaults to 'hsapiens'.
See \code{\link[gprofiler2]{gost}} for more details.}
}
\value{
A data.frame with the enrichment analysis results.
}
\description{
This function uses \code{\link[gprofiler2]{gost}} function
in \code{gprofiler2} package to perfrom functional enrichment analysis
for predicted modules.
}
\details{
enrichfind_cpx
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichplot.R
\name{enrichplot}
\alias{enrichplot}
\title{Plot the Enrichment Reuslt}
\usage{
enrichplot(x, low = "blue", high = "red", cex.size = 15)
}
\arguments{
\item{x}{A data.frame with the enrichment analysis results.}

\item{low}{Colours for low.}

\item{high}{Colours for high.}

\item{cex.size}{Text size.}
}
\value{
An enrichment plot.
}
\description{
This function plots the enrichment result.
}
\details{
enrichplot
}
\examples{
data('enrich.df')
#select enrichment for one of the example (e.g., E protein)
enrich.df <-
enrich.df[enrich.df$id == "E:P0DTC4", ]
enrichplot(enrich.df, low = "blue", high = "red", cex.size = 10)
}
\seealso{
See \code{\link{enrichfindP}} for functional enrichment
analysis.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FSmethod.R
\name{FSmethod}
\alias{FSmethod}
\title{Feature Selection via Matrix Correlation and
Recursive Feature Elimination (RFE)}
\usage{
FSmethod(
  x,
  type = c("cor", "rfe", "both"),
  cor.cutoff = 0.7,
  resampling.method = "cv",
  iter = 2,
  repeats = 3,
  metric = "Accuracy",
  verbose = TRUE
)
}
\arguments{
\item{x}{A data.frame containing protein-protein interactions,
class labels and features.}

\item{type}{The feature selection type, one or two of
\code{filter.corr} and \code{rfeFS}.}

\item{cor.cutoff}{Correlation coefficient cutoff used for filtering.
See \code{filter.corr} for more details.}

\item{resampling.method}{The resampling method for RFE :'boot',
'boot632', optimism_boot',boot_all', 'cv', 'repeatedcv', 'LOOCV',
'LGOCV';defaults to cv. See \code{rfeFS} and
\code{\link[caret]{rfeControl}} for more details.}

\item{iter}{Number of partitions for cross-validation;
defaults to 2. See \code{rfeFS} and \code{\link[caret]{rfeControl}}
for more details.}

\item{repeats}{For repeated k-fold cross validation only;
defaults to 3.See \code{rfeFS} and \code{\link[caret]{rfeControl}}
for more details.}

\item{metric}{A string that specifies what summary metric will be used
to select the optimal feature ; default to ROC.See \code{rfeFS} and
\code{\link[caret]{rfe}} for more details.}

\item{verbose}{Make the output verbose.See \code{rfeFS} and
\code{\link[caret]{rfeControl}} for more details.}
}
\value{
If the type set to \code{filter.corr} , the output includes
the following elements:
\itemize{
\item{corProfile} - A correlation matrix.
\item{corSelectedFeatures} - Name of features that retained after
the correlation analysis.
\item{cordf} - A data.frame filtered.
}
If the type set to \code{rfeFS} , the output includes the following
elements:
\itemize{
\item{rfProfile} - A list of elements. See \code{\link[caret]{rfe}} for
more details.
\item{rfSelectedFeatures} - Name of features that retained in the feature
selection process.
\item{rfdf} - A data.frame filtered.
}
If type set to \code{both} the output includes the following elements:
\itemize{
\item{rfdf} - The final data.frame that includes the selected features
retained after both \code{filter.corr} and \code{rfeFS} analysis.
}
}
\description{
This function performs feature selections
via two approaches
\itemize{
\item \code{filter.corr} - compute matrix correlation between features
and filter using a threshold.
\item \code{rfeFS} - perform recursive feature elimination (RFE) method
wrapped with a Random Forest (RF) algorithm for feature importance
evaluation.
}
}
\details{
FSmethod
}
\examples{
data('example_data')
x <- na.omit(example_data)
s <- FSmethod(x, type = 'both',
cor.cutoff = 0.7, resampling.method = "repeatedcv",
iter = 5, repeats = 3, metric = "ROC", verbose = TRUE)
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCTDD.R
\name{calculateCTDD}
\alias{calculateCTDD}
\title{Calculate CTD Descriptors - Distribution (D)}
\usage{
calculateCTDD(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their fasta
sequences.}
}
\value{
A length 105 named vector for the data input.
}
\description{
This function calculates Distribution (D) descriptor
for data input.
}
\details{
calculateCTDD
}
\examples{
data(UP000464024_df)
x_df <- calculateCTDD(UP000464024_df)
head(x_df, n = 1L)
}
\references{
Dubchak, I., Muchnik, I., Holbrook, S. R., and Kim, S.-H. (1995).
Prediction of protein folding class using global description of amino
acid sequence.\emph{Proc. Natl. Acad. Sci.} 92, 8700–8704.
}
\seealso{
See \code{\link{calculateCTDC}} and \code{\link{calculateCTDT}}
for Composition and Transition descriptors.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_plot.R
\name{corr_plot}
\alias{corr_plot}
\title{Plot Correlation Matrix between Input Features}
\usage{
corr_plot(cormat, method = "number", cex = 0.9)
}
\arguments{
\item{cormat}{A correlation matrix.}

\item{method}{The visualization method of correlation matrix;
defaults to number.
See \code{\link[corrplot]{corrplot}} for more details.}

\item{cex}{The size of x/y axis label.}
}
\value{
A correlation plot.
}
\description{
A graphical display of a correlation matrix.
}
\details{
corr_plot
}
\examples{
data('example_data')
x <- na.omit(example_data)
#perform feature selection
s <- FSmethod(x, type = 'both',
cor.cutoff = 0.7, resampling.method = "repeatedcv",
iter = 5, repeats = 3, metric = "ROC", verbose = TRUE)
corr_plot(s$cor.result$corProfile, method = 'square' , cex = 0.5)
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCTriad.R
\name{calculateCTriad}
\alias{calculateCTriad}
\title{Calculate Conjoint Triad Descriptor}
\usage{
calculateCTriad(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their
fasta sequences.}
}
\value{
A length 343  named vector for the data input.
}
\description{
This function calculates Conjoint Triad descriptor
for data input.
}
\details{
calculateCTriad
}
\examples{
data(UP000464024_df)
x_df <- calculateCTriad(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Shen, J., Zhang, J., Luo, X., Zhu, W., Yu, K., Chen, K., et al.
(2007). Predicting protein–protein interactions based only on
sequences information. \emph{Proc. Natl. Acad. Sci.} 104, 4337–4341.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateBE.R
\name{calculateBE}
\alias{calculateBE}
\title{Tranform a Seqeuence into Binary Encoding (BE)}
\usage{
calculateBE(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their
fasta sequences.}
}
\value{
A length 400 named vector for the data input.
}
\description{
This function transform each residue in a peptide
into 20 coding values.
}
\details{
calculateBE
}
\references{
Al-Barakati, H. J., Saigo, H., and Newman, R. H. (2019).
RF-GlutarySite: a random forest based predictor for glutarylation sites.
\emph{Mol. Omi.} 15, 189–204.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateF.R
\name{calculateF}
\alias{calculateF}
\title{Calculate F1 or F2 Descriptors}
\usage{
calculateF(x, type = c("F1", "F2"))
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their
fasta sequences.}

\item{type}{The descriptor type:
\code{F1} or \code{F2}.}
}
\value{
A length 20 named vector for the data input.
}
\description{
This function calculates F1 or F2 descriptors:
\itemize{
\item \code{F1} - sum of squared length of Single Amino Acid Repeats
(SARs) in the entire protein sequence.
\item \code{F2} - maximum of the sum of Single Amino Acid Repeats (SARs)
in a window of 6 residues.
}
}
\details{
calculateF
}
\examples{
data(UP000464024_df)
x_df <- calculateF(UP000464024_df, type = "F1")
head(x_df, n = 2L)
}
\references{
Alguwaizani, S., Park, B., Zhou, X., Huang, D.-S., and Han, K. (2018).
Predicting interactions between virus and host proteins using repeat
patterns and composition of amino acids.
\emph{J. Healthc. Eng.} 2018.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_clustering.R
\name{run_clustering}
\alias{run_clustering}
\title{Module Detection}
\usage{
run_clustering(
  ppi,
  method = c("FC", "RW", "ML", "clp", "MCL"),
  expan = 2,
  infla = 5,
  iter = 50
)
}
\arguments{
\item{ppi}{A data.frame containing pathogen proteins in the
first column,host proteins in the second column, and edge weight in the
third column.}

\item{method}{Module detection algorithms including:
\itemize{
\item \code{FC} - fast-greedy algorithm.
\item \code{RW} - walktrap algorithm.
\item \code{ML} - multi-level community algorithm.
\item \code{clp} - label propagation algorithm.
\item \code{MCL} - markov clustering.
}}

\item{expan}{Numeric value > 1 for the expansion parameter.
See \code{\link[MCL]{mcl}} for more details.}

\item{infla}{Numeric value > 0 for the inflation power coefficient.
See \code{\link[MCL]{mcl}} for more details.}

\item{iter}{An integer, the maximum number of iterations for the MCL.
See \code{\link[MCL]{mcl}} for more details.}
}
\value{
A data.frame with the enrichment analysis results.
}
\description{
This function contains five module detection algorithms
including fast-greedy algorithm
(\code{FC}), walktrap algorithm (\code{RW}), multi-level community
algorithm
(\code{ML}), label propagation algorithm (\code{clp}), and markov
clustering (\code{MCL}).
}
\details{
run_clustering
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pred_ensembel.R
\name{pred_ensembel}
\alias{pred_ensembel}
\title{Predict Interactions via Ensemble Learning Method}
\usage{
pred_ensembel(
  features,
  gold_standard,
  classifier = c("avNNet", "svmRadial", "ranger"),
  resampling.method = "cv",
  ncross = 2,
  repeats = 2,
  verboseIter = TRUE,
  plots = FALSE,
  filename = "plots.pdf"
)
}
\arguments{
\item{features}{A data frame with host-pathogen protein-protein
interactions (HP-PPIs) in the first column, and features to be passed to
the classifier in the remaining columns.}

\item{gold_standard}{A data frame with gold_standard HP-PPIs and class
label indicating if such PPIs are positive or negative.}

\item{classifier}{The type of classifier to use. See \code{caret}
for the availbale classifiers.}

\item{resampling.method}{The resampling method:'boot', 'boot632',
'optimism_boot', boot_all', 'cv', 'repeatedcv', 'LOOCV', 'LGOCV';
defaults to cv. See \code{\link[caret]{trainControl}} for more details.}

\item{ncross}{Number of partitions for cross-validation;
defaults to 5.See \code{\link[caret]{trainControl}} for more details.}

\item{repeats}{for repeated k-fold cross validation only;
defaults to 3.See \code{\link[caret]{rfeControl}}
for more details.}

\item{verboseIter}{Logical value, indicating whether to check the status
of training process;defaults to FALSE.}

\item{plots}{Logical value, indicating whether to plot the performance
of ensemble learning algorithm as compared to individual classifiers;
defaults to TRUE.If the argument set to TRUE, plots will be saved in
the current working directory.
These plots are :
\itemize{
\item{pr_plot} - Precision-recall plot of ensemble classifier vs
selected individual classifiers.
\item{roc_plot} - ROC plot of ensemble classifier vs selected individual
classifiers.
\item{points_plot} - Plot accuracy, F1-score ,positive predictive value
(PPV),sensitivity (SE), and Matthews correlation coefficient (MCC) of
ensemble classifier vs selected individual classifiers.
}}

\item{filename}{A character string, indicating the output filename as an
pdf object.}
}
\value{
Ensemble_training_output
\itemize{
\item{prediction score} - Prediction scores for whole dataset from each
individual classifier.
\item{Best} - Selected hyper parameters.
\item{Parameter range} - Tested hyper parameters.
\item{prediction_score_test} - Scores probabilities for test data from
each individual classifier.
\item{class_label} -  Class probabilities for test data from each
individual classifier.
}
classifier_performance
\itemize{
\item{cm} - A confusion matrix.
\item{ACC} - Accuracy.
\item{SE} - Sensitivity.
\item{SP} - Specificity.
\item{PPV} - Positive Predictive Value.
\item{F1} - F1-score.
\item{MCC} - Matthews correlation coefficient.
\item{Roc_Object} - A list of elements.
See \code{\link[pROC]{roc}} for more details.
\item{PR_Object} - A list of elements.
See \code{\link[PRROC]{pr.curve}} for more details.
}
predicted_interactions - The input data frame of pairwise interactions,
including classifier scores averaged across all models.
}
\description{
This function uses an ensemble of classifiers to
predict interactions from the sequence-based dataset. This ensemble
algorithm combines different results generated from individual
classifiers within the ensemble via average to enhance prediction.
}
\details{
pred_ensembel
}
\examples{
data('example_data')
features <- example_data[, -2]
gd <- example_data[, c(1,2)]
gd <- na.omit(gd)
ppi <-pred_ensembel(features,gd,
classifier = c("avNNet", "svmRadial", "ranger"),
resampling.method = "cv",ncross = 2,verboseIter = FALSE,plots = FALSE,
filename = "plots.pdf")
#extract predicted interactions
pred_interaction <- ppi[["predicted_interactions"]]
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.df.R
\docType{data}
\name{enrich.df}
\alias{enrich.df}
\title{Enrichment Result}
\usage{
data(enrich.df)
}
\description{
Input data for \code{\link[HPiP]{enrichplot}}
}
\details{
To construct this dataset, predicted interactions, generated
from \code{\link[HPiP]{pred_ensembel}} was used as data input for
\code{\link[HPiP]{enrichplot}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_missing_values.R
\name{filter_missing_values}
\alias{filter_missing_values}
\title{Drop the Missing Values Above a Certain Threshold}
\usage{
filter_missing_values(x, max_miss_rate = 20)
}
\arguments{
\item{x}{A numeric matrix as input.}

\item{max_miss_rate}{Maximal missing rate allowed for a feature;default
is 20.}
}
\value{
A dataframe with features with missingness rate of more than
user-defined threshold.
}
\description{
Given an input matrix, compute the missingness rate for
each features and keep only features with missing rate more than
user-defined percentage.
}
\details{
filter_missing_values
}
\examples{
x <- matrix(1:10, ncol = 2)
x[, 2] <- NA
filter_missing_values(x, 30)
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predicted_PPIs.R
\docType{data}
\name{predicted_PPIs}
\alias{predicted_PPIs}
\title{Predicted HP-PPIs}
\usage{
data(predicted_PPIs)
}
\description{
Input data for \code{\link[HPiP]{enrichfindP}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_positivePPI.R
\name{get_positivePPI}
\alias{get_positivePPI}
\title{Fetch Positive Reference Host-Pathogen Protein-Protein
Interactions (HP-PPIs) from the BioGRID Database}
\usage{
get_positivePPI(
  organism.taxID,
  access.key,
  filename = "PositiveInt.RData",
  path = "PositiveInt"
)
}
\arguments{
\item{organism.taxID}{Taxonomy identifier for the pathogen.}

\item{access.key}{Access key for using BioGRID webpage. To retrieve
interactions from the BioGRID database, the users are first required to
register for access key at
\url{https://webservice.thebiogrid.org/}.}

\item{filename}{A character string, indicating the output filename as an
RData object to store the retrieved interactions.}

\item{path}{A character string indicating the path to the project
directory that contains the interaction data. If the directory is
missing, it will be stored in the current directory.
Default is PositiveInt.}
}
\value{
A Data.frame containing true positive protein-protein
interactions for the selected pathogen.
}
\description{
This function retrieves positive reference host-pathogen
protein-protein interactions directly from BioGRID database.
}
\details{
get_positivePPI
}
\examples{
\donttest{
local = tempdir()
try(get_positivePPI(organism.taxID = 2697049,
access.key = 'XXXX',
filename = "PositiveInt.RData",
path = local))
}
}
\seealso{
See \code{\link{get_negativePPI}} for generating negative
protein-protein interaction.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_negativePPI.R
\name{get_negativePPI}
\alias{get_negativePPI}
\title{Construct Negative Reference Host-Pathogen
Protein-Protein Interactions (HP-PPIs)}
\usage{
get_negativePPI(prot1, prot2, TPset)
}
\arguments{
\item{prot1}{A character vector containing pathogen proteins.}

\item{prot2}{A character vector containing host proteins.}

\item{TPset}{A character vector containing positive reference
interactions.}
}
\value{
A Data.frame containing true negative interactions.
}
\description{
Construct true negative protein-protein interactions
from the positive interactions. In the context of PPI prediction,
a negative interaction is a pair of proteins that unlikely to interact.
Since there is no experimentally verified
non-interacting pair, the negative sampling can be used to construct the
negative reference set. The negative sampling can be constructed from a
set of host proteins, a set of pathogen proteins, and a list of positive
reference interactions between members of host and pathogen proteins
(Eid et al., 2016).
}
\details{
get_negativePPI
}
\examples{
prot1 <- c("P0DTC4", "P0DTC5", "P0DTC9")
prot2 <- c("Q9Y679", "Q9NW15", "Q9NXF8")
TPset <- c("P0DTC4~P31948", "P0DTC8~Q13438")
TN_PPI <- get_negativePPI(prot1, prot2, TPset)
head(TN_PPI)
}
\references{
Eid, F.-E., ElHefnawi, M., and Heath, L. S. (2016).
DeNovo: virus-host sequence-based protein–protein interaction
prediction. Bioinformatics 32, 1144–1150.
}
\seealso{
See \code{\link{get_positivePPI}} for generating positive
protein-protein interaction.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FreqInteractors.R
\name{FreqInteractors}
\alias{FreqInteractors}
\title{Plot the Pathogen Proteins' frequency of Interactions with Host
Proteins}
\usage{
FreqInteractors(ppi, cex.size = 12)
}
\arguments{
\item{ppi}{A data.frame containing pathogen proteins in the first column
and host proteins in the second column.}

\item{cex.size}{Text size.}
}
\value{
A frequency plot.
}
\description{
This function plots the pathogen proteins' Frequency of
interactions with host proteins
}
\details{
FreqInteractors
}
\examples{
ppi <- data.frame(
    node1 = c("A", "A", "A", "B", "B", "B", "B"),
    node2 = c("C", "E", "D", "F", "G", "H", "I")
)
FreqInteractors(ppi)
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichfind_hp.R
\name{enrichfind_hp}
\alias{enrichfind_hp}
\title{Functional Enrichment Analysis of all Host Proteins}
\usage{
enrichfind_hp(
  ppi,
  threshold = 0.05,
  sources = c("GO", "KEGG"),
  p.corrction.method = "bonferroni",
  org = "hsapiens"
)
}
\arguments{
\item{ppi}{A data.frame containing pathogen proteins in the
first column and host proteins in the second column.}

\item{threshold}{Custom p-value threshold for significance.}

\item{sources}{A vector of data sources to use.
See \code{\link[gprofiler2]{gost}} for more details.}

\item{p.corrction.method}{The algorithm used for multiple testing
correction;defaults to 'bonferroni'.
See \code{\link[gprofiler2]{gost}} for more details.}

\item{org}{An organism name;defaults to 'hsapiens'.
See \code{\link[gprofiler2]{gost}} for more details.}
}
\value{
A data.frame with the enrichment analysis results.
}
\description{
This function uses \code{\link[gprofiler2]{gost}} function
in \code{gprofiler2} package to perfrom functional enrichment analysis
for all predicted host proteins in the high-confidence network.
}
\details{
enrichfind_hp
}
\examples{
data('predicted_PPIs')
#perform enrichment
enrich.df <- enrichfind_hp(predicted_PPIs,
threshold = 0.05,
sources = c("GO", "KEGG"),
p.corrction.method = "bonferroni",
org = "hsapiens")
}
\seealso{
See \code{\link{enrichplot}} for plotting enrichment analysis.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateAAC.R
\name{calculateAAC}
\alias{calculateAAC}
\title{Calculate Amino Acid Composition (AAC) Descriptor}
\usage{
calculateAAC(x)
}
\arguments{
\item{x}{A data.frame containing gene/protein names and their
fasta sequences.}
}
\value{
A length 20 named vector for the data input.
}
\description{
This function calculates Amino Acid Composition
(AAC) descriptor for the data input.
}
\details{
calculateAAC
}
\examples{
data(UP000464024_df)
x_df <- calculateAAC(UP000464024_df)
head(x_df, n = 2L)
}
\references{
Dey, L., Chakraborty, S., and Mukhopadhyay, A. (2020).
Machine learning techniques for sequence-based prediction of
viral–host interactions between SARS-CoV-2 and human proteins.
\emph{Biomed. J.} 43, 438–450.
}
\seealso{
See \code{\link{calculateDC}} and
\code{\link{calculateTC}} for Dipeptide Composition and
Tripeptide Composition descriptors.
}
\author{
Matineh Rahmatbakhsh, \email{matinerb.94@gmail.com}
}
